/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Initialisation_Com.h"
#include "Commun.h"
#include "Debug.h"
#include "Declaration_Constante.h"
#include "Thread_Notification_Equipement.h"
#include "Thread_Rafraichir_Tout_Equipement.h"
#include "Thread_Scan_Arp_Detect_Nouveau_Equipement.h"
#include "Thread_Notification_Port.h"
#include "Thread_Info.h"
#include "Notification_Port.h"
#include "Reseau.h"
#include "Thread_Detect.h"
#include "Thread.h"
#include "Info.h"
#include "Semaphore.h"
#include "Scan_Signature.h"
#include "Scan_Service.h"
#include "Thread_Ping_All.h"
#include "Configuration.h"
#include "config.h"

void Envoie_Version_Agent (void)
{
	struct struct_Agent_Version Message;
	gint      pid = 0;

	pid = getpid ();
	Message.ID_Reseau = Info_Demarrage.ID_Reseau;
	ERROR_if (strlen (VERSION_AGENT) > TAILLE_MAX_VERSION_AGENT);
	STRCPYMAX(Message.Version, VERSION_AGENT,TAILLE_MAX_VERSION_AGENT);
	STRCPYMAX(Message.Config, Config_Path,TAILLE_MAX_INFO);
	if (Config_Path_Logs == NULL)
		STRCPYMAX(Message.Logs,DEFAULT_LOG,TAILLE_MAX_INFO);
	else
		STRCPYMAX(Message.Logs, Config_Path_Logs,TAILLE_MAX_INFO);
#ifndef    __MSVC_ENABLE
	STRCPYMAX(Message.Compil, HOSTNAME_COMPILE " (" PLATFORM " " KERNEL " ) compiled at " __TIME__ " on " __DATE__  " gcc " __VERSION__ ,TAILLE_MAX_INFO);
	STRCPYMAX(Message.Compil_Option, OPTIONS ,TAILLE_MAX_INFO);
#else
	STRCPYMAX(Message.Compil, HOSTNAME_COMPILE,TAILLE_MAX_INFO);
	STRCPYMAX(Message.Compil_Option, HOSTNAME_COMPILE ,TAILLE_MAX_INFO);
#endif
	Message.Taille_Struct = sizeof(struct struct_ALL);
	Message.pid = pid;
	Communication_Envoie_Donnee (AGENT_VERSION, (gpointer *) & Message, sizeof (Message));
}

void Demarrage_Agent (gpointer * Donnee)
{
	ERROR_if (Donnee == NULL);
	memcpy (&Info_Demarrage, Donnee, sizeof (struct struct_Demarrage_Agent));
/*	if (Info_Demarrage.Normal_Mode){
	Time_Out_Service = 10;
	Time_Out_Connect_Service = 60;  
	}
	else {
		Time_Out_Connect_Service = 5;
		Time_Out_Service = 5;
	}*/
	Debug = Info_Demarrage.Niveau_Debugage;
	DEBUG_INFO ( "Interface : %s  [%s]\n",Info_Demarrage.Interface,Info_Demarrage.nom);

#ifdef __DEBUG_COM
	Log_Send = Create_Log(CATEG_COMM_SEND,Info_Demarrage.nom,NULL,FALSE);
	Log_Recv = Create_Log(CATEG_COMM_RECV,Info_Demarrage.nom,NULL,FALSE);
#endif
	Get_My_Ip ();
	Get_My_MAC ();	
	Initialisation_Semaphore_Gestion_Thread ();
	Start_Thread (&Boucle_Scan, NULL);
	if (Nb_Machine_Dans_La_Liste!=0){
		Scan_Machine_Local_Fait = TRUE;
		Start_Scan_Plage();
	}	
	Start_Thread (&Thread_Detect, NULL);
	Start_Thread (&Thread_Notification_Equipement, NULL);
	Start_Thread (&Thread_Notification_Port, NULL);
	Start_Thread (&Info_Agent, NULL);
	Start_Thread (&Thread_Rafraichir_Tout_Equipement, NULL); 
	Start_Thread (&Thread_Ping_All, NULL);
	Start_Thread (&Thread_Scan_Arp_Detect_Nouveau_Equipement, NULL);
	Demarrage_Complet_Agent = TRUE;  
}

void Envoie_Liste_Interface (gpointer * Donnee)
{
	int nb_Interface=0;
#if defined(__DARWIN_ENABLE) || defined(__LINUX_ENABLE) 
	struct if_nameindex *ifn_list=NULL , *ifnp=NULL;
	struct struct_Nouvelle_Interface Message;
	struct struct_Fin_Liste_Interface Message_Fin;
	struct sockaddr_in sin;
	struct ifreq ifr;
	int ifs=0;


	if ((ifn_list = if_nameindex()) == NULL) 
		DEBUG_ERROR ("if_nameindex failed");

	if ((ifs = socket(AF_INET, SOCK_DGRAM, 0)) < 0) 
		DEBUG_ERROR ("socket");

	for (ifnp = ifn_list; ifnp->if_name != NULL; ifnp++){
		if 	((strcmp(ifnp->if_name,"lo0")!=0) && ((strcmp(ifnp->if_name,"lo")!=0))){
		
			strncpy(ifr.ifr_name, ifnp->if_name, sizeof(ifr.ifr_name));
			if (ioctl(ifs, SIOCGIFADDR, (caddr_t)&ifr) < 0) {
				if (errno == EADDRNOTAVAIL)
					DEBUG_WARNING ("EADDRNOTAVAIL   %s",ifnp->if_name);
				else
					DEBUG_ERROR ( "SIOCGIFADDR : %s",strerror (errno));
			} 
			else {
				sin.sin_addr = ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr;
				DEBUG_WARNING("%s  %s",ifnp->if_name,inet_ntoa(sin.sin_addr));
				if (strlen(ifnp->if_name)>=TAILLE_NOM_INTERFACE_STRUCT_NOUVELLE_INTERFACE){
					strncpy(Message.Interface, ifnp->if_name, TAILLE_NOM_INTERFACE_STRUCT_NOUVELLE_INTERFACE-1);
					Message.Interface[TAILLE_NOM_INTERFACE_STRUCT_NOUVELLE_INTERFACE-1] = 0;
				}
				else
					strcpy (Message.Interface, ifnp->if_name);
			
				strcpy (Message.Ip, inet_ntoa(sin.sin_addr));
				DEBUG_INFO ("Nouvelle Interface : interface[%s] ip[%s]", Message.Interface,Message.Ip);
				Communication_Envoie_Donnee (NOUVELLE_INTERFACE, (gpointer *) & Message, sizeof (Message));
				nb_Interface++;
			}
		}
	}

	close(ifs);

	if_freenameindex(ifn_list);
	Communication_Envoie_Donnee (FIN_LISTE_INTERFACE, (gpointer *) & Message_Fin, sizeof (Message_Fin));
#endif	

#ifdef	__WINDOWS_ENABLE
	struct struct_Nouvelle_Interface Message;
	struct struct_Fin_Liste_Interface Message_Fin;
	char errbuf[PCAP_ERRBUF_SIZE];
	pcap_if_t * alldevs = NULL;
	pcap_if_t * d = NULL;
	pcap_addr_t *a = NULL;
	
	ERROR_if (Donnee == NULL);
	if (pcap_findalldevs(&alldevs, errbuf) == -1)
    	DEBUG_ERROR ( "Error in pcap_findalldevs: %s", errbuf);
    for(d= alldevs; d != NULL; d= d->next){
		if (d->description){
			a=d->addresses;
			if ((a)&&(a->addr->sa_family==AF_INET)&&(a->addr)){
			
				if (strlen(d->description)>TAILLE_NOM_INTERFACE_STRUCT_NOUVELLE_INTERFACE){
					strncpy(Message.Interface, d->description,TAILLE_NOM_INTERFACE_STRUCT_NOUVELLE_INTERFACE-1);
					Message.Interface[TAILLE_NOM_INTERFACE_STRUCT_NOUVELLE_INTERFACE-1] = 0;
					}
				else
					strcpy (Message.Interface, d->description);
			
				strcpy (Message.Ip, inet_ntoa(((struct sockaddr_in *)(a->addr))->sin_addr));
				DEBUG_INFO ( "Nouvelle Interface : interface[%s] ip[%s]", Message.Interface,Message.Ip);
				Communication_Envoie_Donnee (NOUVELLE_INTERFACE, (gpointer *) & Message, sizeof (Message));
				nb_Interface++;
			}
		}
	}

    pcap_freealldevs(alldevs);

	Communication_Envoie_Donnee (FIN_LISTE_INTERFACE, (gpointer *) & Message_Fin, sizeof (Message_Fin));
#endif
	if (nb_Interface == 0)
		DEBUG_WARNING( "No interfaces found!");	
}
