/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Gestion_Signal.h"
#include "BackTrace.h"
#include "Debug.h"
#include "Socket.h"
#include "Thread.h"
#include "Semaphore.h"
#include "Initialisation_Com.h"
#include "Declaration_Constante.h"
#include "Quit.h"
#include "Mem.h"

#ifdef         __SIGNAL_ENABLE
extern struct Struct_Categorie Liste_Categorie[];
	
char *segv[] = {
	"SEGV_MAPERR = Address not mapped to object",
	"SEGV_ACCERR = Invalid permissions for mapped object"
};


void segfault_handler_thread ( int signum, siginfo_t *info, void *secret)
{	
	printf("segfault_handler_thread signum=%d, info=%p, ptr=%p   [%s]\n", signum, info, secret, segv[info->si_code-1]);
	if (info->si_errno != 0)
		printf("Errorno : [%s]\n", strerror(info->si_errno));
	printf("segfault_handler_thread si_signo=%d, si_errno=%d, si_code=%d, si_value=%i, si_addr=%p\n",info->si_signo, info->si_errno, info->si_code, info->si_value.sival_int, info->si_addr);
	segfault_handler(signum);
}


void segfault_handler (int sig)
{	
	struct Struct_Liste_Equipement* Equipement = NULL;
	struct struct_Cle_Privee *pCle = NULL;
	
	printf("SegFault : %d\n",sig);

	signal (SIGSEGV, Arret_Immediat);
#ifdef __DEBUG_MEM
	DebugMem_Disable(); 
#endif

	if ((pCle = pthread_getspecific (Cle_privee_thread)) == NULL)
		pCle = Donnee_Cle_Main;

	if (pCle->Categorie==CATEG_THREAD_SCAN)
		Equipement = pCle->Pointeur_Data;
	print_trace ();
	printf ("Segfault : Invalid memory reference (%s)\n", Liste_Categorie[pCle->Categorie].Info);

	display_backtrace(NULL,NULL,NULL,0);
	printf("Segfault #################################\n");

	if (Equipement == NULL) {
		Log (pCle, "#####################################\n");
		Log (pCle, "---------- Segfault(Agent) ----------\n");
		display_backtrace (pCle,NULL,NULL,0);
	}
	else {
		Log_Equipement(Equipement, "#####################################\n");
		Log_Equipement(Equipement, "---------- Segfault(Scan Equipement) ----------\n");
		display_backtrace (NULL,Equipement,NULL,0);
	}
	printf("GUI\n");
	Message_Erreur_Critique(pCle,D_SEG_FAULT);
	printf("Close Log\n");
	Close_Log_Main();	
	Ckeck_Fermeture_Log(); 

	Close_Communication_Interface();
	printf ("Stopping %s (http://autoscan-network.com)\n", VERSION_AGENT);
	exit(EXIT_SUCCESS);	
}

void Control_C (int sig)
{
	printf ("Control_C  %d\n",sig);
	if (Niveau_Arret != A_START)
		exit(-1);
	DEBUG_INFO("Arret (Control_C) Socket ouvert[%d]  Thread[%d]", Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan));
	Arret_Demande (NULL);
}

void Deconnection_Socket (int sig)
{
	printf ("Deconnection_Socket  %d\n",sig);
}

void sig_abrt_hand ( int signum, siginfo_t* info, void* secret) 
{
	struct Struct_Liste_Equipement* Equipement = NULL;
	struct struct_Cle_Privee *pCle = NULL;
		
	printf("sig_abrt_hand signum=%d, info=%p, ptr=%p\n", signum, info, secret);
	printf("sig_abrt_hand si_signo=%d, si_errno=%d, si_code=%d, si_value=%i, si_addr=%p\n",info->si_signo, info->si_errno, info->si_code, info->si_value.sival_int, info->si_addr);

 	printf ( "\n\n##########################\n");
	printf ( "Stack Overflow\n"); 	
	printf ( "##########################\n");

	signal (SIGSEGV, Arret_Immediat);
	

	if ((pCle = pthread_getspecific (Cle_privee_thread)) == NULL)
		pCle = Donnee_Cle_Main;

	if (pCle->Categorie==CATEG_THREAD_SCAN)
		Equipement = pCle->Pointeur_Data;
	print_trace ();
	printf ("Stack Overflow (%s)\n", Liste_Categorie[pCle->Categorie].Info);

	display_backtrace(NULL,NULL,NULL,0);
	printf("Stack Overflow #################################\n");

	if (Equipement == NULL) {
		Log (pCle, "#####################################\n");
		Log (pCle, "---------- Stack Overflow(Agent) ----------\n");
		display_backtrace (pCle,NULL,NULL,0);
	}
	else {
		Log_Equipement(Equipement, "#####################################\n");
		Log_Equipement(Equipement, "---------- Stack Overflow(Scan Equipement) ----------\n");
		display_backtrace (NULL,Equipement,NULL,0);
	}

	Message_Erreur_Critique(pCle,D_SEG_FAULT);

	Close_Communication_Interface();
	printf ("Stopping %s (http://autoscan-network.com)\n", VERSION_AGENT);
    exit (EXIT_SUCCESS);
}

void Initialisation_Signal_Erreur (void)
{
 struct sigaction    sa;
 sigset_t            sigmsk;

 
	sigfillset( &sigmsk );
    sigdelset( &sigmsk, SIGUSR1 );
	sigdelset( &sigmsk, SIGUSR2 );
    sigdelset( &sigmsk, SIGHUP );
    sigdelset( &sigmsk, SIGWINCH );
#ifndef         __COREDUMP_DAEMON_ENABLE
	printf("Debug mode : SIGSEGV\n");
    sigdelset( &sigmsk, SIGSEGV );
 #endif
    sigdelset( &sigmsk, SIGINT );
    sigdelset( &sigmsk, SIGILL );
    sigdelset( &sigmsk, SIGFPE );
    pthread_sigmask( SIG_SETMASK, &sigmsk, NULL );
 
 	sa.sa_sigaction = sig_abrt_hand;
 	sigemptyset( &sa.sa_mask );
    sigaction ( SIGABRT, &sa, NULL);
  
    sa.sa_sigaction = Pause_Thread;
    sigemptyset( &sa.sa_mask );
    sa.sa_flags = SA_RESTART | SA_SIGINFO;
    sigaction( SIGUSR1, &sa, NULL );
 
    sa.sa_sigaction = Transmet_Thread_Main;
    sigemptyset( &sa.sa_mask );
    sa.sa_flags = SA_RESTART | SA_SIGINFO;
    sigaction( SIGUSR2, &sa, NULL );
#ifndef         __COREDUMP_DAEMON_ENABLE

    sa.sa_sigaction = segfault_handler_thread;
    sigemptyset( &sa.sa_mask );
    sa.sa_flags = SA_RESTART | SA_SIGINFO;
    sigaction( SIGSEGV, &sa, NULL );	
 
	signal (SIGSEGV, segfault_handler);
#endif
	signal (SIGINT, Control_C);
	signal(SIGIO, Deconnection_Socket);
	signal(SIGURG, Deconnection_Socket);
}

void Arret_Service(){
	kill (pid_Instance, SIGINT);
	exit(0);
}

void Signal_Arret_Service(){
	signal (SIGTERM, Arret_Service);
}

void Kill_Term(pid_t pid){
	kill (pid, SIGTERM);
}
#endif


/*

---------------  T H R E A D  ---------------

Current thread (0x085c9c50):  JavaThread "http-80-Processor196" daemon [_thread_
in_native, id=32032]

siginfo:si_signo=11, si_errno=0, si_code=1, si_addr=0xb26b307c

Registers:
EAX=0xb26b307c, EBX=0xb7fdce80, ECX=0x00000a0a, EDX=0x0a0a0a0a
ESP=0x2a1253f4, EBP=0x2a125424, ESI=0x000000e7, EDI=0xb26b307c
EIP=0xb7f24900, CR2=0xb26b307c, EFLAGS=0x00010206

  Top of Stack: (sp=0x2a1253f4)
0x2a1253f4:   b26b307c 000000f7 b7f0d1fe b26b307c
0x2a125404:   0000000a 000000f7 00000000 b26b307c
0x2a125414:   2a1254e8 b7fdce80 28c2c738 fbad2418
0x2a125424:   2a125448 b7f0d181 28c2c738 2a1254e8
0x2a125434:   000003e7 0000000a 00000001 00000000
0x2a125444:   b7fdce80 2a125474 b7f1661d 28c2c738
0x2a125454:   2a1254e8 000003e7 0000000a 00000001
0x2a125464:   00000000 b7e851dc 2a1254d0 000003e8

Instructions: (pc=0xb7f24900)
0xb7f248f0:   00 40 4e e9 8e 00 00 00 90 8d b4 26 00 00 00 00
0xb7f24900:   8b 08 bf ff fe fe fe 31 d1 01 cf 0f 83 04 01 00

Stack: [0x2a0a8000,0x2a128000),  sp=0x2a1253f4,  free space=500k
Native frames: (J=compiled Java code, j=interpreted, Vv=VM code, C=native code)
C  [libc.so.6+0x79900]  memchr+0x70
C  [libc.so.6+0x62181]  _IO_getline+0x41
C  [libc.so.6+0x6b61d]  fgets_unlocked+0x5d
C  [libnss_files.so.2+0x3e31]
C  [libnss_files.so.2+0x3471]  _nss_files_gethostbyname_r+0x211
C  [libc.so.6+0xeb19f]  gethostbyname_r+0x10f
C  [libnet.so+0x4d61]  Java_java_net_Inet4AddressImpl_getLocalHostName+0x91
j  java.net.Inet4AddressImpl.getLocalHostName()Ljava/lang/String;+0
j  java.net.InetAddress.getLocalHost()Ljava/net/InetAddress;+7
j  com.sun.mail.smtp.SMTPTransport.getLocalHost()Ljava/lang/String;+122
j  com.sun.mail.smtp.SMTPTransport.protocolConnect(Ljava/lang/String;ILjava/lang
/String;Ljava/lang/String;)Z+254
j  javax.mail.Service.connect(Ljava/lang/String;ILjava/lang/String;Ljava/lang/St
ring;)V+378

*/
