/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Ping.h"
#include "Log.h"
#include "Socket.h"
#include "Semaphore.h"
#include "Debug.h"
#include "Win32.h"

char      Trame_Ping[] = 			{ 0x45, 0x00, 0x00, 0x3B, 0x00, 0x01, 0x40, 0x00, 0x01,	0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x00, 0x00, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x3e, 0x01, 0xF0, 0xDB, 0x44, 0x6A, 0x7C, 0x9F, 0x0C, 0x8D, 0x1F, 0x5d, 0xE3, 0x55, 0x4F, 0x59, 0xA4, 0x15, 0xC3, 0xEF, 0xC7, 0xB1, 0xFF, 0x1c, 0x63, 0x28, 0x88, 0x73, 0x90, 0xD3, 0xBB };


void Ping_equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct struct_trame PacketOs;
	unsigned short Icmp_Id = 0, Icmp_Seq = 0;
#ifdef  __PCAP_ENABLE
	pcap_t    * captureHandle;
	char errbuf[PCAP_ERRBUF_SIZE];
#endif
	ERROR_if (Equipement == NULL);
	PROTECT_CODE(Semaphore_Ping_Equipement);

#ifndef  __PCAP_ENABLE	
	struct sockaddr sock;
	int       Socket;
	OPEN_SOCKET(Socket,sock,Info_Demarrage,AF_INET,SOCK_PACKET,htons (ETH_P_ALL));
#else
	OPEN_SOCKET_PCAP(captureHandle,Info_Demarrage,errbuf);
#endif	
	bzero (&PacketOs, sizeof (struct struct_trame));
	Icmp_Id = (unsigned short) rand_r(&graine_rand_r);
	Icmp_Seq = 1;
	memcpy (&PacketOs.Dest_MAC, &Equipement->Adresse_MAC, ETH_ALEN);
	memcpy (&PacketOs.Source_MAC, Mon_Adresse_MAC, ETH_ALEN);
	PacketOs.Proto = htons (TRAME_IP);
	memcpy (&PacketOs.Datagramme, Trame_Ping, 59);
	memcpy (&PacketOs.Datagramme[12], &Mon_Adresse_Ip, IP_ADDR_LEN);
	memcpy (&PacketOs.Datagramme[16], &Equipement->Adresse_Ip, IP_ADDR_LEN);
	ckecksum_r(Equipement,&PacketOs.Datagramme[0], 20, 0);
	Equipement->CheckSum_Calc = htons (Equipement->CheckSum_Calc);
	memcpy (&PacketOs.Datagramme[10], &Equipement->CheckSum_Calc, 2);
	PacketOs.Datagramme[24] = (Icmp_Id >> 8) & 0xFF;
	PacketOs.Datagramme[25] = Icmp_Id & 0xFF;
	PacketOs.Datagramme[26] = (Icmp_Seq >> 8) & 0xFF;
	PacketOs.Datagramme[27] = Icmp_Seq & 0xFF;
	ckecksum_r(Equipement,&PacketOs.Datagramme[20], 39, 0);
	Equipement->CheckSum_Calc = htons (Equipement->CheckSum_Calc);
	memcpy (&PacketOs.Datagramme[22], &Equipement->CheckSum_Calc, 2);
#ifndef   __PCAP_ENABLE
	SENDTO(Socket, &PacketOs, 73, 0, &sock, sizeof (struct sockaddr));
	CLOSE_SOCKET(Socket);
#else
	SENDTO_PCAP(captureHandle,(u_char *) &PacketOs, 73);
	CLOSE_SOCKET_PCAP(captureHandle);
 #endif
	DEPROTECT_CODE(Semaphore_Ping_Equipement);
}


gboolean Check_Etat_Arp_Ping_Equipement(struct Struct_Liste_Equipement *Equipement,gboolean envoie_info_agent){
	struct struct_Retour_Rachaichir_Etat_Equipement Message;
	gboolean  Sauve_Etat = FALSE;
	gint	Timeout = 0;
	
	ERROR_if (Equipement == NULL);
	Sauve_Etat = Sem_Get_Boolean(&Equipement->Equipement_Allumer);
	Sem_Set_Int(&Equipement->Compteur_Notification, 0);
	Sem_Set_Boolean(&Equipement->Equipement_Allumer, FALSE);
	Sem_Set_Boolean(&Equipement->Rafraichissement_Etat_En_Cours,TRUE);

	Timeout = 5;
	while ((Timeout!=0) && (!Sem_Get_Boolean(&Equipement->Equipement_Allumer))){
		if (envoie_info_agent)
			Retour_Info_Scan((5-Timeout)*2,"Ping",Equipement);
	
		Envoie_Arp_Equipement (Equipement, 4);
	
		Ping_equipement(Equipement);
		sleep (1);
		Timeout--;
	}
	Sem_Set_Boolean(&Equipement->Rafraichissement_Etat_En_Cours ,FALSE);

	if (Sauve_Etat != Sem_Get_Boolean(&Equipement->Equipement_Allumer)) {	
		bzero (&Message, sizeof (struct struct_Retour_Rachaichir_Etat_Equipement));
		Message.ID_Reseau = Info_Demarrage.ID_Reseau;
		memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
		Message.Equipement_Allumer = Equipement->Equipement_Allumer;
		Message.Derniere_Modification_Etat_Equipement = Equipement->Derniere_Modification_Etat_Equipement;
		Communication_Envoie_Donnee (RETOUR_RAFRAICHIR_ETAT_EQUIMEMENT, (gpointer *) & Message, sizeof (Message));
	}
	return Sem_Get_Boolean(&Equipement->Equipement_Allumer);
}
