/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Reseau.h"
#include "Debug.h"
#include "Initialisation_Com.h"
#include "Semaphore.h"
#include "Liste_Equipement.h"
#include "Win32.h"

#include <errno.h>
#include <sys/types.h>

#include <stdio.h>
#include <string.h>
#include <sys/types.h>


#ifdef __DARWIN_ENABLE
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <sys/ioctl.h>
	#include <sys/sysctl.h>
	#include <net/if.h>
	#include <net/if_dl.h>
	#include <netinet/in.h>
	#include <arpa/inet.h>
	#include <errno.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include <netdb.h>
#endif

#ifdef         __WINDOWS_ENABLE
	#include "Win32.h"
	#include <windows.h>
	#include <pcap-int.h>
	#include <ntddndis.h>
	#include <remote-ext.h>
	#include <winsock2.h>
	#include <ws2tcpip.h>
#endif


void Get_My_MAC (void)
{
#ifdef	__WINDOWS_ENABLE
	pcap_t      *hadapter = NULL;

	hadapter = pcap_open_live(Info_Demarrage.nom, 0, 0, 0, 0);
  	if(!getmac(hadapter))
		DEBUG_ERROR( "getmac FAILED");
	pcap_close(hadapter);
#endif

#ifdef	__LINUX_ENABLE
	struct ifreq ifr;
	struct sockaddr sock;
	int   Socket = 0;

	strcpy (sock.sa_data, Info_Demarrage.Interface);
	sock.sa_family = AF_INET;
	if ((Socket = socket_control (PF_PACKET, SOCK_RAW, htons (ETH_P_ALL))) < 0) {
		DEBUG_ERROR( "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));
	}
	ifr.ifr_addr.sa_family = AF_INET;
	sprintf (ifr.ifr_name, "%s", Info_Demarrage.Interface);
	if (ioctl (Socket, SIOCGIFHWADDR, &ifr) == -1) {
		DEBUG_ERROR("%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));
	}
	memcpy (Mon_Adresse_MAC, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	if (close_control (Socket) < 0) {
		DEBUG_ERROR( "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));
	}
#endif

#ifdef	__DARWIN_ENABLE
	int                     mib[6], len;
	char                    *buf = NULL;
	unsigned char           *ptr = NULL;
	struct if_msghdr        *ifm = NULL;
	struct sockaddr_dl      *sdl = NULL;

	mib[0] = CTL_NET;
	mib[1] = AF_ROUTE;
	mib[2] = 0;
	mib[3] = AF_LINK;
	mib[4] = NET_RT_IFLIST;
	if ((mib[5] = if_nametoindex(Info_Demarrage.Interface)) == 0)
		DEBUG_ERROR("if_nametoindex error");
	if (sysctl(mib, 6, NULL, (size_t *)&len, NULL, 0) < 0)
		DEBUG_ERROR( "sysctl 1 error");
	if ((buf = malloc(len)) == NULL)
		DEBUG_ERROR("malloc error");
	if (sysctl(mib, 6, buf,  (size_t *)&len, NULL, 0) < 0)
		DEBUG_ERROR("sysctl 2 error");

	ifm = (struct if_msghdr *)buf;
	sdl = (struct sockaddr_dl *)(ifm + 1);
	ptr = (unsigned char *)LLADDR(sdl);
	printf("%02x:%02x:%02x:%02x:%02x:%02x\n", *ptr, *(ptr+1), *(ptr+2), *(ptr+3), *(ptr+4), *(ptr+5));
	memcpy (Mon_Adresse_MAC, ptr, ETH_ALEN);
#endif
}

void Get_My_Ip (void)
{
#if defined(__LINUX_ENABLE) || defined(__DARWIN_ENABLE)
	struct ifreq ifr;
	struct sockaddr_in sin;
	struct sockaddr sock;
	int       Socket = 0;

	strcpy (sock.sa_data, Info_Demarrage.Interface);
	sock.sa_family = AF_INET;
	if ((Socket = socket_control (AF_INET, SOCK_RAW, htons (ETH_P_ALL))) < 0)
		DEBUG_ERROR("%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));
	strcpy (ifr.ifr_name, Info_Demarrage.Interface);
	if (ioctl (Socket, SIOCGIFADDR, &ifr) == -1)
		DEBUG_ERROR("%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));
	memcpy (&sin, &ifr.ifr_addr, sizeof (struct sockaddr_in));
	memcpy (Mon_Adresse_Ip, &sin.sin_addr.s_addr, IP_ADDR_LEN);
	DEBUG_WARNING("IP  %d.%d.%d.%d",Mon_Adresse_Ip[0],Mon_Adresse_Ip[1],Mon_Adresse_Ip[2],Mon_Adresse_Ip[3]);
	if (close_control (Socket) < 0)
		DEBUG_ERROR( "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));
#endif

#ifdef         __WINDOWS_ENABLE
	char errbuf[PCAP_ERRBUF_SIZE];
	pcap_if_t * alldevs = NULL;
	pcap_if_t * d = NULL;
	pcap_addr_t *a = NULL;
	int resultat = 0,nb_Interface=0;


	if (pcap_findalldevs(&alldevs, errbuf) == -1)
    	DEBUG_ERROR("Error in pcap_findalldevs: %s", errbuf);
  
    for(d= alldevs; d != NULL; d= d->next){
		if (d->description){
			if (strlen(d->description)<TAILLE_NOM_INTERFACE_STRUCT_NOUVELLE_INTERFACE)
				resultat = strcmp(d->description,Info_Demarrage.Interface);
			else
				resultat = strncmp(d->description,Info_Demarrage.Interface,TAILLE_NOM_INTERFACE_STRUCT_NOUVELLE_INTERFACE - 1);
			if (resultat==0){
				a=d->addresses;
				if ((a)&&(a->addr->sa_family==AF_INET)&&(a->addr)){
					memcpy (Mon_Adresse_Ip, &((struct sockaddr_in *)(a->addr))->sin_addr , IP_ADDR_LEN);
				
					if (strlen(d->name)>TAILLE_NOM_ID_STRUCT_NOUVELLE_INTERFACE){
						strncpy(Info_Demarrage.nom, d->name,TAILLE_NOM_ID_STRUCT_NOUVELLE_INTERFACE-1);
						Info_Demarrage.nom[TAILLE_NOM_ID_STRUCT_NOUVELLE_INTERFACE-1] = 0;
					}
					else
						strcpy (Info_Demarrage.nom, d->name);
					nb_Interface++;
				}
			}
		}
	}
	if (nb_Interface == 0)
		DEBUG_ERROR("No interfaces found!");

	pcap_freealldevs(alldevs);
#endif
}

unsigned short ckecksum (char *donnee, int longueur, int valeur)
{
	int       i = 0;
	int       somme = 0;
	int       Nombre = 0;

	ERROR_if (donnee == NULL);
	i = 0;
	somme = 0;
	while (i < longueur) {
		if (longueur == (i + 1)) {
			donnee[i + 1] = 0;
		}
		Nombre = (((donnee[i] << 8) & 0xFF00) + (donnee[i + 1] & 0xFF));
		somme = somme + Nombre;
		i = i + 2;
	}
	while ((somme >> 16) != 0) {
		somme = (somme >> 16) + (somme & 0xffff);
	}
	somme = (0xFFFF - (somme & 0xFFFF)) - valeur;
	if ((valeur != 0) && (somme < 0))
		somme--;
	return (unsigned short) somme;
}


void ckecksum_r (struct Struct_Liste_Equipement *Equip,unsigned char *donnee, int longueur, int valeur)
{
	ERROR_if (Equip == NULL);
	ERROR_if (donnee == NULL);
	Equip->ck_i = 0;
	Equip->ck_somme = 0;
	while (Equip->ck_i < longueur) {
		if (longueur == (Equip->ck_i + 1)) {
			donnee[Equip->ck_i + 1] = 0;
		}
		Equip->ck_Nombre = (((donnee[Equip->ck_i] << 8) & 0xFF00) + (donnee[Equip->ck_i + 1] & 0xFF));
		Equip->ck_somme = Equip->ck_somme + Equip->ck_Nombre;
		Equip->ck_i = Equip->ck_i + 2;
	}
	while ((Equip->ck_somme >> 16) != 0) {
		Equip->ck_somme = (Equip->ck_somme >> 16) + (Equip->ck_somme & 0xffff);
	}
	Equip->ck_somme = (0xFFFF - (Equip->ck_somme & 0xFFFF)) - valeur;
	if ((valeur != 0) && (Equip->ck_somme < 0))
		Equip->ck_somme--;
	Equip->CheckSum_Calc = (unsigned short) Equip->ck_somme;
}



int Connection_Equipement (unsigned char *AddressIP, int Port, int Tentative,gboolean Block,int Unite_Temp)
{
	struct sockaddr_in sock;
	int       Socket = 0, TimeOut = 0, Etat = 0;

	ERROR_if (AddressIP == NULL);
	bzero(&sock,sizeof(struct sockaddr_in));
#ifndef         __WINDOWS_ENABLE
	CREATE_SOCKET(Socket,AddressIP,Port,sock,AF_INET, SOCK_STREAM, 0);
#else
	CREATE_SOCKET(Socket,AddressIP,Port,sock,AF_INET,SOCK_STREAM,IPPROTO_TCP);
#endif
	DEBUG_INFO("Creation d'une socket TCP/IP  (%d)", Socket);
	if (!Block)
		NONBLOCK_SOCKET(Socket,SOCKET_NON_BLOQUANT);
	if (connect_control(Socket,&sock,AddressIP,Port) == SOCK_OK) {
		if (!Block) {
			TimeOut = 0;
			do {
				WAITING_WRITE_SOCKET(Etat,Socket,Unite_Temp);
				DEBUG_INFO("Tentative [%s] Socket(%d) Connection port %d : TimeOut n[%d]", AddressIP,Socket,Port,TimeOut);
				TimeOut++;
			} while ((Etat != SOCK_OK) && (TimeOut <= Tentative));
		}
	}
	if (Etat != SOCK_OK) {
		DEBUG_INFO("Impossible de se connecter Socket(%d). Fermeture de la socket",Socket);
		CLOSE_SOCKET(Socket);
		return SOCK_ERROR;
	}
	return Socket;
}

gint create_tcp_server (int Port, int Nb_Max_Client)
{
	int       Socket = 0;
	struct sockaddr_in sock;

	CREATE_SOCKET(Socket,NULL,Port,sock,PF_INET, SOCK_STREAM, 0);
	REUSEADDR_SOCKET(Socket);


	BIND(Socket,&sock);
	LISTEN(Socket, Nb_Max_Client);
	return (Socket);
}



void Recupere_Nom_DNS (struct Struct_Liste_Equipement *Equip,gchar *Resultat)
{
#ifndef __STATIC_ENABLE
#ifdef         __LINUX_ENABLE
	#define SIZEBUFFER_R 32768
	char fixed_buffer[SIZEBUFFER_R];
  	char *buffer_r = fixed_buffer;
	struct in_addr addr;
    struct hostent hst_ent;
	struct hostent *h = NULL;
	int herr = 0, retour = 0, Taille_Chaine = 0;

	ERROR_if (Equip->Adresse_Ip_Char == NULL);
	bzero (buffer_r, SIZEBUFFER_R);
	addr.s_addr = inet_addr((char *) Equip->Adresse_Ip_Char);



	retour = gethostbyaddr_r((char *) &addr, sizeof(struct in_addr), AF_INET, &hst_ent, buffer_r, SIZEBUFFER_R, &h, &herr);
	DEBUG_WARNING("retour(%d)  herr(%d)  h(%04X)",retour,herr,h);
	if ((retour != 0) || (h == NULL) || (herr != 0)){
		Resultat[0] = 0 ;
		if (retour == ERANGE)
			DEBUG_WARNING( "Dns Failed (buf is too small).[%s]",Equip->Adresse_Ip_Char);
		switch (herr) {
			case HOST_NOT_FOUND:
			    DEBUG_WARNING("L'hôte indiqué est inconnu.(%s) [%s]", hstrerror (herr), Equip->Adresse_Ip_Char);
			    break;
		    case NO_ADDRESS:
			    DEBUG_WARNING("Le nom est valide mais ne possède pas d'adresse IP.(%s) [%s]", hstrerror (herr), Equip->Adresse_Ip_Char);
			    break;
		    case NO_RECOVERY:
			    DEBUG_WARNING( "Une erreur fatale du serveur de noms est apparue.(%s) [%s]", hstrerror (herr), Equip->Adresse_Ip_Char);
			    break;
		    case TRY_AGAIN:
			    DEBUG_WARNING("Aucune connection à  internet.(%s) [%s]", hstrerror (herr), Equip->Adresse_Ip_Char);
			    break;
		    default:
			    DEBUG_WARNING("Erreur Inconnue(%d) pour [%s]", herr, Equip->Adresse_Ip_Char);
			    break;
		}
    }
    else {
        Taille_Chaine = strlen (hst_ent.h_name);
		if (Taille_Chaine != 0) {
			if (Taille_Chaine > TAILLE_CHAINE_NOM_DNS)
				Taille_Chaine = TAILLE_CHAINE_NOM_DNS;
			strncpy (Resultat, hst_ent.h_name, Taille_Chaine);
			Resultat[Taille_Chaine] = 0;
			DEBUG_INFO("Nom DNS = [%s]", Resultat);
		}
	}
#else
	struct hostent 	*host;
	unsigned long int  addr;
	int Taille_Chaine = 0;

	ERROR_if (Equip->Adresse_Ip_Char == NULL);
	DEBUG_INFO("Recupere_Nom_DNS = [%s]",Equip->Adresse_Ip_Char);
	addr = inet_addr ((char *) Equip->Adresse_Ip_Char);
	host = gethostbyaddr((char *)&addr, sizeof(struct in_addr),AF_INET);

	
	if (host == NULL) {
#ifndef         __WINDOWS_ENABLE
		switch (h_errno) {
		    case HOST_NOT_FOUND:
			    DEBUG_WARNING("L'hôte indiqué est inconnu.(%s) [%s]", hstrerror (h_errno), Equip->Adresse_Ip_Char);
			    break;
		    case NO_ADDRESS:
			    DEBUG_WARNING("Le nom est valide mais ne possède pas d'adresse IP.(%s) [%s]", hstrerror (h_errno), Equip->Adresse_Ip_Char);
			    break;
		    case NO_RECOVERY:
			    DEBUG_WARNING("Une erreur fatale du serveur de noms est apparue.(%s) [%s]", hstrerror (h_errno), Equip->Adresse_Ip_Char);
			    break;
		    case TRY_AGAIN:
			    DEBUG_WARNING("Aucune connection à  internet.(%s) [%s]", hstrerror (h_errno), Equip->Adresse_Ip_Char);
			    break;
		    default:
			    DEBUG_WARNING("Erreur Inconnue(%d) pour [%s]", h_errno, Equip->Adresse_Ip_Char);
			    break;
		}
#else
		DEBUG_WARNING("Dns Failed.[%s]",Equip->Adresse_Ip_Char);
#endif
		Resultat[0] = 0 ;
	}
	else {
		Taille_Chaine = strlen (host->h_name);
		if (Taille_Chaine != 0) {
			if (Taille_Chaine > TAILLE_CHAINE_NOM_DNS)
				Taille_Chaine = TAILLE_CHAINE_NOM_DNS;
			strncpy (Resultat, host->h_name, Taille_Chaine);
			Resultat[Taille_Chaine] = 0;
			DEBUG_INFO("Nom DNS = [%s]", Resultat);
		}
	}
#endif
#endif
}
