/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Notification_Equipement.h"

#ifdef         __WINDOWS_ENABLE  
#include "Win32.h"  
#endif  


void Ajout_Notification_Equipement (gpointer * Donnee)
{
	struct struct_Demande_Notication_Critique_Etat_Equipement *Message = (struct struct_Demande_Notication_Critique_Etat_Equipement *) Donnee;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement;

	ERROR_if (Donnee == NULL);
	Recherche_Index_Equipement (Message->ID_Equipement, FALSE, &ID_Equipement);
	switch (ID_Equipement) {
		case EQUIPEMENT_UNKNOWN :
				DEBUG_WARNING("Add Notification (Equipment unknown)");
				return;	
		case EQUIPEMENT_NO_IP :
				DEBUG_WARNING("Add (no IP)");
				return;			
	}
	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);
	ERROR_if (Equipement == NULL);
	DEBUG_INFO("Add Notification (%s)(%s)",Equipement->Adresse_Ip_Char,Equipement->Adresse_MAC_Char);
	Equipement->Alerte_Equipement_Active = FALSE;
	Ajout_Notification_Equipement_Glist (Equipement);
}

void Enlever_Notification_Equipement (gpointer * Donnee)
{
	struct struct_Annulation_Notification_Etat_Equipement *Message = (struct struct_Annulation_Notification_Etat_Equipement *) Donnee;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement;

	ERROR_if (Donnee == NULL);
	Recherche_Index_Equipement (Message->ID_Equipement, TRUE, &ID_Equipement);
	switch (ID_Equipement) {
		case EQUIPEMENT_UNKNOWN :
				DEBUG_WARNING("Delete Notification (Equipment unknown)");
				return;	
		case EQUIPEMENT_NO_IP :
				DEBUG_WARNING("Delete Notification (no IP)");
				return;			
	}
	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);
	ERROR_if (Equipement == NULL);
	Enlever_Notification_Equipement_Glist (Equipement);
}

gint Recherche_Index_Notification_Equipement_Non_Securiser (struct Struct_Liste_Equipement * Equipement)
{			
	struct Struct_Liste_Notification_Equipement *Notification_Equipement = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	Nb_Objet_Liste = g_list_length (GList_Notification_Equipement);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Notification_Equipement = g_list_nth_data (GList_Notification_Equipement, i);
		if (Notification_Equipement->Equipement == Equipement)
			return i;
	}
	return -1;	
}

void Enlever_Notification_Equipement_Glist (struct Struct_Liste_Equipement *Equipement)
{
	gint      ID_Equipement;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_NOTIFICATION_EQUIPEMENT();
	Sem_Set_Boolean(&Equipement->Notification_Equipement ,FALSE);
	if ((ID_Equipement = Recherche_Index_Notification_Equipement_Non_Securiser (Equipement)) != -1)
		G_LIST_REMOVE(GList_Notification_Equipement, g_list_nth_data (GList_Notification_Equipement, ID_Equipement));
	else
		DEBUG_WARNING("Enlever Notification Equipement n'existe pas: [%s]", Equipement->Adresse_MAC_Char);
	DEPROTECT_GLIST_NOTIFICATION_EQUIPEMENT();
}


void Ajout_Notification_Equipement_Glist (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Liste_Notification_Equipement *Notification_Equipement = NULL;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_NOTIFICATION_EQUIPEMENT();// Permet de securiser la suppression (ex si le thread de notification d'equipement est en train de lire la liste des notifications et quand meme temps on enleve un equipement dans cette liste il y aurait une erreur de segmentation)
	Sem_Set_Boolean(&Equipement->Notification_Equipement, TRUE);
	if (Recherche_Index_Notification_Equipement_Non_Securiser (Equipement) == -1) {
		MALLOC(Notification_Equipement,sizeof (struct Struct_Liste_Notification_Equipement));
		Notification_Equipement->Equipement = Equipement;
		Sem_Set_Int(&Equipement->Compteur_Notification, 0);
		G_LIST_APPEND (GList_Notification_Equipement, Notification_Equipement);
	}
	else
		DEBUG_WARNING("Nouveau Notification Equipement existe deja: [%s]", Equipement->Adresse_MAC_Char);
	DEPROTECT_GLIST_NOTIFICATION_EQUIPEMENT();
}


void Initialisation_Notification_Equipement (void)
{
	GList_Notification_Equipement = NULL;
}
