/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Notification_Port.h"
#include "Debug.h"
#include "Port_TCP.h"
#include "Initialisation_Com.h"
#include "Commun.h"
#include "Mem.h"
#include "Win32.h"


void Ajout_Notification_Port (gpointer * Donnee)
{
	struct struct_Demande_Notication_Critique_Etat_Port *Message = (struct struct_Demande_Notication_Critique_Etat_Port *) Donnee;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement;

	ERROR_if (Donnee == NULL);
	Recherche_Index_Equipement (Message->ID_Equipement,FALSE, &ID_Equipement);
	if (ID_Equipement == EQUIPEMENT_UNKNOWN) {
		DEBUG_WARNING("Add (Equipment unknown)");
		return;	
	}
	/*if (ID_Equipement == EQUIPEMENT_NO_IP) {
		debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Add (no IP)");
		return;	
	}*/
	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);
	ERROR_if (Equipement == NULL);
	Ajout_Notification_Port_Glist (Equipement, Message->Num_Port);
}

void Enlever_Notification_Port (gpointer * Donnee)
{
	struct struct_Annulation_Notification_Etat_Port *Message = (struct struct_Annulation_Notification_Etat_Port *) Donnee;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement;

	ERROR_if (Donnee == NULL);
	Recherche_Index_Equipement (Message->ID_Equipement,TRUE, &ID_Equipement);
	if (ID_Equipement == EQUIPEMENT_UNKNOWN) {
		DEBUG_WARNING("Delete (Equipment unknown)");
		return;	
	}
	if (ID_Equipement == EQUIPEMENT_NO_IP) {
		DEBUG_WARNING("Delete (no IP)");
		return;	
	}
	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);
	ERROR_if (Equipement == NULL);
	Enlever_Notification_Port_Glist (Equipement, Message->Num_Port);
}

void Affiche_Debug_Liste_Notification_Port_Securiser (void)
{
	struct Struct_Liste_Notification_Port *Notification_Port = NULL;
	gint      Nb_Objet_Liste, i;

	PROTECT_GLIST_NOTIFICATION_PORT();
	Nb_Objet_Liste = g_list_length (GList_Notification_Port);
	for (i = 0; i < Nb_Objet_Liste; i++)
		Notification_Port = g_list_nth_data (GList_Notification_Port, i);
	DEPROTECT_GLIST_NOTIFICATION_PORT();
}

struct Struct_Port *Recupere_Pointeur_Notification_Port_Dans_Une_Liste (int rang)
{
	return g_list_nth_data (GList_Notification_Port, rang);
}

gint Recherche_Index_Notification_Port_Non_Securiser (struct Struct_Liste_Equipement * Equipement, gint Num_Port)
{
	struct Struct_Liste_Notification_Port *Notification_Port = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	Nb_Objet_Liste = g_list_length (GList_Notification_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Notification_Port = g_list_nth_data (GList_Notification_Port, i);
		if ((Notification_Port->Equipement == Equipement) && (Notification_Port->Num_Port == Num_Port))
			return i;
	}
	return -1;	
}

void Enlever_Notification_Port_Glist (struct Struct_Liste_Equipement *Equipement, gint Num_Port)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      ID_Port;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_NOTIFICATION_PORT();
	Port_Equipement = Recupere_Pointeur_Port_Glist (Equipement, Num_Port);
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	if (Port_Equipement != NULL) {
		Port_Equipement->Notification = FALSE;
		if ((ID_Port = Recherche_Index_Notification_Port_Non_Securiser (Equipement, Num_Port)) != -1)
			G_LIST_REMOVE(GList_Notification_Port, Recupere_Pointeur_Notification_Port_Dans_Une_Liste (ID_Port));
		else
			DEBUG_WARNING("Enlever Notification Port n'existe pas: [%s]:[%d]", Equipement->Adresse_MAC_Char, Num_Port);
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_NOTIFICATION_PORT();
	Affiche_Debug_Liste_Notification_Port_Securiser ();
}

void Ajout_Notification_Port_Glist (struct Struct_Liste_Equipement *Equipement, gint Num_Port)
{
	struct Struct_Liste_Notification_Port *Notification_Port = NULL;
	struct Struct_Port *Port_Equipement = NULL;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_NOTIFICATION_PORT();
	Port_Equipement = Recupere_Pointeur_Port_Glist (Equipement, Num_Port);
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	if (Port_Equipement != NULL) {
		Port_Equipement->Notification = TRUE;
		if (Recherche_Index_Notification_Port_Non_Securiser (Equipement, Num_Port) == -1) {
			MALLOC(Notification_Port,sizeof (struct Struct_Liste_Notification_Port));
			Notification_Port->Equipement = Equipement;
			Notification_Port->Num_Port = Num_Port;
			Notification_Port->Port = Port_Equipement;
			G_LIST_APPEND (GList_Notification_Port, Notification_Port);
		}
		else
			DEBUG_WARNING("Nouveau Notification Port existe deja : %s [%s] Port(%d)", Notification_Port->Equipement->Adresse_Ip_Char, Notification_Port->Equipement->Adresse_MAC_Char, Num_Port);
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_NOTIFICATION_PORT();
	Affiche_Debug_Liste_Notification_Port_Securiser ();
}


void Initialisation_Notification_Port (void)
{
	GList_Notification_Port = NULL;
}
