/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Port_TCP.h"
#include "Debug.h"
#include "Mem.h"
#include "Reseau.h"
#include "Win32.h"

void Cherche_Un_Port_Avec_Etat (struct Struct_Liste_Equipement * Equipement, gint Etat, gint *resultat)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint Nb_Objet = 0, i = 0;
	
	ERROR_if (Equipement == NULL);
	ERROR_if (resultat == NULL);
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Nb_Objet = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Etat_Port == Etat){
			*resultat = Port_Equipement->Num_Port;
			DEPROTECT_STRUCT_PORT_EQUIPEMENT();
			DEPROTECT_GLIST_PORT();
			return;
		}
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
	*resultat = -1;
}

gboolean Cherche_Si_Port_Ouvert (struct Struct_Liste_Equipement * Equipement, gint Num_Port)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint Nb_Objet = 0, i = 0;
	gboolean Etat = FALSE;
	
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	ERROR_if (Equipement == NULL);
	Nb_Objet = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Num_Port == Num_Port){
			Etat = (Port_Equipement->Etat_Port == OUVERT);
			DEPROTECT_STRUCT_PORT_EQUIPEMENT();
			DEPROTECT_GLIST_PORT();
			return Etat; 			
		}
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
	return FALSE;
}


struct Struct_Port *Recupere_Pointeur_Port_Glist (struct Struct_Liste_Equipement *Equipement, gint Num_Port)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      Nb_Objet_Liste = 0, i = 0;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Num_Port == Num_Port){
			DEPROTECT_STRUCT_PORT_EQUIPEMENT();
			DEPROTECT_GLIST_PORT();
			return Port_Equipement;
		}
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
	return NULL;
}

void Copie_Ancien_Etat_Liste_Port_TCP_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port_Equipement = NULL;	
	gint      Nb_Objet = 0, i = 0;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Nb_Objet = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		Port_Equipement->Ancien_Etat_Port = Port_Equipement->Etat_Port;
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
}

void Compare_Ancien_Etat_Liste_Port_Tcp_Equipement (struct Struct_Liste_Equipement *Equipement){
	struct struct_Retour_Agent_Ajout_Nouveau_Port_Ouvert Message_Port_Ouvert;
	struct struct_Retour_Agent_Ajout_Nouveau_Port_Fermer Message_Port_Fermer;
	struct Struct_Port *Port_Equipement = NULL;	
	gint      Nb_Objet = 0, i = 0;
	
	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Nb_Objet = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Ancien_Etat_Port != Port_Equipement->Etat_Port){
			if (Port_Equipement->Etat_Port == OUVERT){
				DEBUG_INFO("[%s] Port %d ouvert",(gchar *)Equipement->Adresse_Ip_Char,Port_Equipement->Num_Port);
				bzero (&Message_Port_Ouvert, sizeof (struct struct_Retour_Agent_Ajout_Nouveau_Port_Ouvert));
				Message_Port_Ouvert.ID_Reseau = Info_Demarrage.ID_Reseau;
				memcpy (Message_Port_Ouvert.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
				Message_Port_Ouvert.Num_Port = Port_Equipement->Num_Port;
				Communication_Envoie_Donnee (RETOUR_AGENT_AJOUT_NOUVEAU_PORT_OUVERT, (gpointer *) & Message_Port_Ouvert, sizeof (Message_Port_Ouvert));
			}
			else{
				if (Port_Equipement->Ancien_Etat_Port == OUVERT) {
					DEBUG_INFO("[%s] Port %d Fermer",(gchar *)Equipement->Adresse_Ip_Char,Port_Equipement->Num_Port);
					bzero (&Message_Port_Fermer, sizeof (struct struct_Retour_Agent_Ajout_Nouveau_Port_Fermer));
					Message_Port_Fermer.ID_Reseau = Info_Demarrage.ID_Reseau;
					memcpy (Message_Port_Fermer.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
					Message_Port_Fermer.Num_Port = Port_Equipement->Num_Port;
					Communication_Envoie_Donnee (RETOUR_AGENT_AJOUT_NOUVEAU_PORT_FERMER, (gpointer *) & Message_Port_Fermer, sizeof (Message_Port_Fermer));
				}
			}
		}
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();	
}

void Initialisation_Liste_Port_TCP_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port_Equipement = NULL;	
	gint      Nb_Objet = 0, i = 0, j = 0;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Nb_Objet = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		Port_Equipement->Etat_Port = FIREWALLED;
		for (j = 0; j < NB_TENTATIVE_MAX_SCAN_PORT_TCP; j++){
			Port_Equipement->Port_Scan[j] = 0;
			Port_Equipement->MonNumSeq_Scan[j] = 0;
		}
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
}

gboolean Test_Retour_Port_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      Nb_Objet = 0, i = 0;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Nb_Objet = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Etat_Port == FIREWALLED){
			DEPROTECT_STRUCT_PORT_EQUIPEMENT();
			DEPROTECT_GLIST_PORT();
			return FALSE;
		}
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
	return TRUE;
}

void Creer_Liste_Port_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port_Equipement = NULL;
	struct struct_Liste_Port *Port = NULL;
	gint      Nb_Objet_Liste = 0, i = 0;

	ERROR_if (Equipement == NULL);
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Nb_Objet_Liste = g_list_length (GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_Equipement = NULL;
		Port = g_list_nth_data (GList_Port, i);
		MALLOC(Port_Equipement,sizeof (struct Struct_Port));
		Port_Equipement->Num_Port = Port->Num_Port;
		Port_Equipement->Notification = FALSE;
		Port_Equipement->Niveau_Notification = NOTIFICATION_STANDARD;
		Port_Equipement->Etat_Port = FIREWALLED;
		Port_Equipement->Ancien_Etat_Port = FIREWALLED;
		Port_Equipement->Alerte_Port_Active = FALSE;
		Port_Equipement->Derniere_Modification_Etat_Port = 0;
		Port_Equipement->Compteur_Notification = 0;
		G_LIST_APPEND (Equipement->GList_Port, Port_Equipement);
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
}


void Ajout_Un_Port (gpointer * Donnee)
{
	struct struct_Liste_Port *Port = NULL, *Donnee_Port = (struct struct_Liste_Port *) Donnee;

	ERROR_if (Donnee == NULL);
	PROTECT_GLIST_PORT();
	MALLOC(Port,sizeof (struct struct_Liste_Port));
	Port->Num_Port = Donnee_Port->Num_Port;
	Port->Serveur = Donnee_Port->Serveur;
	G_LIST_APPEND (GList_Port, Port);
	DEPROTECT_GLIST_PORT();
}

void Initialisation_Liste_port (void)
{
	GList_Port = NULL;
}

void Set_Etat_Port(struct Struct_Port *Port_Equipement,gint Etat){
	ERROR_if (Port_Equipement == NULL);
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Port_Equipement->Etat_Port = Etat;
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
}
gint Get_Etat_Port(struct Struct_Port *Port_Equipement){
	gint Etat = 0;

	ERROR_if (Port_Equipement == NULL);
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Etat = Port_Equipement->Etat_Port;
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	return Etat;
}
gint Get_Num_Port(struct Struct_Port *Port_Equipement){
	gint Num_Port = 0;

	ERROR_if (Port_Equipement == NULL);
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	Num_Port = Port_Equipement->Num_Port;
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	return Num_Port;
}
