/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Port_UDP.h"
#include "Debug.h"
#include "Info.h"
#include "Mem.h"
#include "Reseau.h"

struct Struct_Liste_Port_UDP Liste_Port_UDP[] = {
	{DHCPC,68},
	{SVRLOC,427},
	{LISTENER,1025},
	{UPNP,1900},
	{SIP, 5060},
	{ZEROCONF,5353}	
};

void Initialisation_Liste_Port_UDP_Equipement (struct Struct_Liste_Equipement *Equip)
{
	int i;

	ERROR_if (Equip == NULL);
	PROTECT_LISTE_ETAT_UDP();
	for (i = 0; i < NB_PORT_UDP; i++) {
		Equip->Liste_Etat_Udp[i] = UNKOWN_UDP;
	}
	DEPROTECT_LISTE_ETAT_UDP();
}

gboolean Test_Retour_Port_UDP_Equipement (struct Struct_Liste_Equipement *Equip)
{
	int i;

	ERROR_if (Equip == NULL);
	PROTECT_LISTE_ETAT_UDP();
	for (i = 0; i < NB_PORT_UDP; i++) {
		if (Equip->Liste_Etat_Udp[i] == UNKOWN_UDP) {
			DEBUG_INFO("Toujours pas de reponse  port %d", Liste_Port_UDP[i].Num_Port_UDP);
			DEPROTECT_LISTE_ETAT_UDP();
			return FALSE;
		}
	}
	DEPROTECT_LISTE_ETAT_UDP();
	return TRUE;
}

void Modifie_Etat_Port_UDP_Equipement (struct Struct_Liste_Equipement *Equip, gint Num_Port, gint Nouvelle_Etat)
{
	int i;
	
	ERROR_if (Equip == NULL);
	PROTECT_LISTE_ETAT_UDP();
	for (i = 0; i < NB_PORT_UDP; i++) {
		if (Liste_Port_UDP[i].Num_Port_UDP == Num_Port) {
			Equip->Liste_Etat_Udp[i] = Nouvelle_Etat;
/*#ifdef __LOG_ENABLE
			switch (Nouvelle_Etat){
						case FERMER_UDP :
							cEtat = "FERMER_UDP";
							break;
						default :
							cEtat = "UNKOWN";
			}
			STRDUP_PRINTF(Texte,"[LOG] [%s](%s:%d): Etat du port %d = [%s]\n",__FILE__, __PRETTY_FUNCTION__, __LINE__,Num_Port,cEtat);
			Log_Equipement(Equip,Texte);
			MYFREE(Texte);
#endif*/
			DEPROTECT_LISTE_ETAT_UDP();
			return;
		}
	}
	DEPROTECT_LISTE_ETAT_UDP();
	DEBUG_WARNING("Port %d Inconnu", Num_Port);
}


gint Get_Etat_Port_UDP(struct Struct_Liste_Equipement *Equipement, gint i){
	gint Etat = 0;

	ERROR_if (Equipement == NULL);
	PROTECT_LISTE_ETAT_UDP();
	Etat = Equipement->Liste_Etat_Udp[i];
	DEPROTECT_LISTE_ETAT_UDP();
	return Etat;
}
