/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Scan_Equipement.h"
#include "Thread_Detect.h"
#include "Debug.h"
#include "Port_TCP.h"
#include "Scan_Signature.h"
#include "Scan_Netbios.h"
#include "Thread.h"
#include "Scan_Service.h"
#include "Commun.h"
#include "Communication.h"
#include "Initialisation_Com.h"
#include "Semaphore.h"
#include "Snmp.h"
#include "Scan_Arp.h"
#include "Mem.h"
#include "Ping.h"
#include "Port_UDP.h"
#include "Win32.h"

struct Struct_Modele_Scan_Port Liste_Scan_port[] = {
	{SCAN_FTP			, "Ftp"			, &Recupere_Banniere_FTP			, PORT_FTP 	 		,50	, 499},
	{SCAN_HTTP			, "Http"		, &Recupere_Banniere_HTTP			, PORT_HTTP	 		,53	, 499},
	{SCAN_IMAP			, "Imap"		, &Recupere_Banniere_IMAP			, PORT_IMAP	 		,56	, 499},	
	{SCAN_SMTP		, "Smtp"		, &Recupere_Banniere_SMTP			, PORT_SMTP	 		,59	, 499},	
	{SCAN_SSH			, "Ssh"			, &Recupere_Banniere_SSH			, PORT_SSH			,62	, 499},	
	{SCAN_TELNET		, "Telnet"		, &Recupere_Banniere_TELNET		, PORT_TELNET	 	,65	, 999},	
	{SCAN_SIP			, "Sip"			, &Recupere_Banniere_SIP				, PORT_SIP	 		,68	, 499},	
	{SCAN_POP3		, "Pop3"		, &Recupere_Banniere_POP3			, PORT_POP3	 		,71	, 499},	
	{SCAN_VNC0		, "Vnc:0"		, &Recupere_Banniere_VNC0			, PORT_VNC0	 		,74	, 499},	
	{SCAN_VNC1		, "Vnc:1"		, &Recupere_Banniere_VNC1			, PORT_VNC1	 		,77	, 499},	
	{SCAN_RTSP			, "Rtsp"		, &Recupere_Banniere_RTSP			, PORT_RTSP	 		,80	, 499},	
	{SCAN_IPP			, "Ipp"			, &Recupere_Banniere_IPP				, PORT_IPP	 		,83	, 499},	
	{SCAN_VNC_HTTP	, "Vnc_http"	, &Recupere_Banniere_VNC_HTTP	, PORT_VNC_HTTP	,86	, 499},	
};

void Copie_ChaineAlloue_ListeCaractere_FREE(struct Struct_Liste_Equipement *Equip,gchar *Copie, gint Taille_Max_B){
	gint size = 0;
	
	ERROR_if (Equip == NULL);
	if (Equip->vs_banniere == NULL) {
		Copie[0] = '\0';
	}
	else { 
	
		size = 0;
		while ((size<10000) && (Equip->vs_banniere[size] != '\0')) size++;
		if (size == 10000)
			DEBUG_ERROR("Chaine Sans caractère final");
		if (size > Taille_Max_B)
			size = Taille_Max_B;
		DEBUG_INFO("Chaine Copie (%d)",size);
		strncpy (Copie, Equip->vs_banniere, size);
		Copie[size] = '\0';
		MYFREE (Equip->vs_banniere);
	}	
}


void Scan_Port (gint Id_Scan,struct Struct_Liste_Equipement *Equipement,gchar *Entete,time_t64 *Start,time_t64 *Stop, gint *Temp,gint *TempResultat) {
		ERROR_if (Equipement == NULL);
		ERROR_if (Entete == NULL);
		Retour_Info_Scan(Liste_Scan_port[Id_Scan].NiveauProgress, Liste_Scan_port[Id_Scan].Info ,Equipement);
		if (Cherche_Si_Port_Ouvert (Equipement, Liste_Scan_port[Id_Scan].Num_Port)) {
			*Start = time(NULL);
			DEBUG_INFO(Liste_Scan_port[Id_Scan].Info);
			(*Liste_Scan_port[Id_Scan].FonctionRecupBanniere) (Equipement);
			Copie_ChaineAlloue_ListeCaractere_FREE(Equipement, Entete,Liste_Scan_port[Id_Scan].MaxSizeEntete);
			*Stop = time(NULL);
			*Temp = (gint) difftime(*Start, *Stop);
			*TempResultat =  (gint) *Temp;
		}
		else
			*TempResultat = -1;
}

void Retour_Post_Nouveau_Equipement(struct Struct_Liste_Equipement *Equipement){
	struct struct_Post_Nouveau_Equipement Message;
	
	bzero (&Message, sizeof (struct struct_Post_Nouveau_Equipement));
	Message.ID_Reseau = Info_Demarrage.ID_Reseau;
	memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
	if (memcmp (Equipement->ID_Equipement, Mon_Adresse_MAC, MAC_ADDR_LEN) != 0) {
		Message.Agent_AutoScan = FALSE;
		Equipement->Agent_AutoScan = FALSE;
	}
	else {
		Message.Agent_AutoScan = TRUE;
		Equipement->Agent_AutoScan = TRUE;
	}		
	Message.Agent_AutoScan = Equipement->Agent_AutoScan;
	memcpy (Message.Adresse_Ip_Char, Equipement->Adresse_Ip_Char, 15 + 1);
	memcpy (Message.Adresse_MAC_Char, Equipement->Adresse_MAC_Char, 17 + 1);
	Communication_Envoie_Donnee (RETOUR_POST_NOUVEAU_EQUIPEMENT, (gpointer *) & Message, sizeof (Message));	
}

void Retour_Info_Scan(gint Pourcentage,gchar *Info,struct Struct_Liste_Equipement *Equipement){
	struct struct_Retour_Etat_Scan Message;
	
	bzero (&Message, sizeof (struct struct_Retour_Etat_Scan));
	Message.ID_Reseau = Info_Demarrage.ID_Reseau;
	memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
	Message.Pourcentage = Pourcentage;
	strcpy(Message.Info,Info);
	Communication_Envoie_Donnee (RETOUR_ETAT_SCAN, (gpointer *) & Message, sizeof (Message));
}

void Rescanne_Equipement (struct Struct_Liste_Equipement *Equip)
{
	struct struct_Retour_Rafraichir_Equipement *Resultat = NULL;
	struct struct_Retour_Rachaichir_Etat_Equipement *EtatChanger = NULL;
	struct Struct_Port *Port_Equipement = NULL;
	gint      Timeout = 0,i = 0,Nb_Port_Ajouter_Ouvert = 0,Nb_Port_Ajouter_Fermer = 0, Nb_Port_Ajouter_Firewalled = 0, Tentative_NetBios = 0,Tentative_NetBios_MAX = 0;
	gboolean Sauve_Etat;
	
	ERROR_if (Equip == NULL);
#ifdef __LOG_ENABLE
	Create_Log_Equipement(Equip);  

  	pthread_once(&once_cle_privee,create_cle_privee);
	Equip->vs_Donnee_Cle = NULL;
	MALLOC(Equip->vs_Donnee_Cle,sizeof(struct struct_Cle_Privee));
	Equip->vs_Donnee_Cle->Categorie = CATEG_THREAD_SCAN;
	Equip->vs_Donnee_Cle->Pointeur_Data = Equip;
	Equip->vs_Donnee_Cle->ID_Fichier_Log = Equip->ID_Fichier_Log; 
	pthread_setspecific(Cle_privee_thread,Equip->vs_Donnee_Cle);
#endif
	DEBUG_INFO("Rescanne_Equipement  Socket ouvert[%d]  Thread[%d] ",Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan));
	Retour_Info_Scan(0,"start",Equip);
	Sauve_Etat = Sem_Get_Boolean(&Equip->Equipement_Allumer);
	Sem_Set_Int(&Equip->Compteur_Notification,0);
	Sem_Set_Boolean(&Equip->Rafraichissement_Etat_En_Cours,TRUE);
	Sem_Set_Boolean(&Equip->Equipement_Allumer, FALSE);
	



	Timeout = 0;
	while ((Timeout<5) && (!Sem_Get_Boolean(&Equip->Equipement_Allumer))){
		Envoie_Arp_Equipement (Equip, 4);
	
		Ping_equipement(Equip);
		sleep (1);
		Timeout++;
	}
	Sem_Set_Boolean(&Equip->Rafraichissement_Etat_En_Cours, FALSE);

	MALLOC(Resultat,sizeof(struct struct_Retour_Rafraichir_Equipement));
	Resultat->ID_Reseau = Info_Demarrage.ID_Reseau;
	memcpy (Resultat->ID_Equipement, Equip->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
	if (Sauve_Etat != Sem_Get_Boolean(&Equip->Equipement_Allumer)) {
	
		MALLOC(EtatChanger,sizeof(struct struct_Retour_Rachaichir_Etat_Equipement));
		EtatChanger->ID_Reseau = Info_Demarrage.ID_Reseau;
		memcpy (EtatChanger->ID_Equipement, Equip->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
		EtatChanger->Equipement_Allumer = Equip->Equipement_Allumer;
		EtatChanger->Derniere_Modification_Etat_Equipement = Equip->Derniere_Modification_Etat_Equipement;
		Communication_Envoie_Donnee (RETOUR_RAFRAICHIR_ETAT_EQUIMEMENT, (gpointer *) EtatChanger, sizeof (struct struct_Retour_Rachaichir_Etat_Equipement));
		MYFREE(EtatChanger);
	}

	if (!Sem_Get_Boolean(&Equip->Equipement_Allumer)) {	
		MYFREE(Resultat);
		Stop_Thread (NULL);
	}
	Sem_Set_Boolean(&Equip->Scan_Port_En_Cours, TRUE);

	Equip->Time_Debut = time(NULL);
	

	Retour_Info_Scan(1,"Dns",Equip);
	DEBUG_INFO("Resolution du nom DNS");
	Equip->Time_Debut_Resolv_DNS = time(NULL);
	Recupere_Nom_DNS (Equip,Resultat->Nom_DNS);
	Equip->Time_Fin_Resolv_DNS = time(NULL);
	Equip->Temps_Resolv_DNS = (gint) difftime(Equip->Time_Fin_Resolv_DNS, Equip->Time_Debut_Resolv_DNS);
	Resultat->Temps_Resolv_DNS = Equip->Temps_Resolv_DNS;

	

	Retour_Info_Scan(2,"TCP/IP",Equip);
	DEBUG_INFO("Scanne des Ports");
	Equip->Time_Debut_Scan_Port = time(NULL);
	Scanne_Toutes_Les_Ports_TCP (Equip,Equip->mss,FALSE);
	Equip->Time_Fin_Scan_Port = time(NULL);
	Equip->Temps_Scan_Port = (gint) difftime(Equip->Time_Fin_Scan_Port, Equip->Time_Debut_Scan_Port);
	Resultat->Temps_Scan_Port = Equip->Temps_Scan_Port;

	
	

	Retour_Info_Scan(25,"Udp",Equip);
	DEBUG_INFO("Scanne des Ports UDP");
	Equip->Time_Debut_Scan_Port_Udp = time(NULL);
	Scanne_Toutes_Les_Ports_UDP (Equip);
	Equip->Time_Fin_Scan_Port_Udp = time(NULL);
	Equip->Temps_Scan_Port_Udp = (gint) difftime(Equip->Time_Fin_Scan_Port_Udp, Equip->Time_Debut_Scan_Port_Udp);
	Resultat->Temps_Scan_Port_Udp = Equip->Temps_Scan_Port_Udp;

	

	Retour_Info_Scan(30,"Ports",Equip);

	DEBUG_INFO("Traitement des Ports");
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	i= 0;
	Resultat->Nb_Port_Total_Scanner = g_list_length (Equip->GList_Port);

	Resultat->Nb_Port_Ouvert = 0;
	Resultat->Nb_Port_Fermer = 0;
	Resultat->Nb_Port_Firewaller = 0;
	Nb_Port_Ajouter_Ouvert = 0;
	Nb_Port_Ajouter_Fermer = 0;
	Nb_Port_Ajouter_Firewalled = 0;
	while (i < Resultat->Nb_Port_Total_Scanner)  {
		Port_Equipement = g_list_nth_data (Equip->GList_Port, i);
	
		if (Port_Equipement->Etat_Port == OUVERT){			
			if (Nb_Port_Ajouter_Ouvert<MAX_PORTS) {
				Resultat->Port_Ouvert[Nb_Port_Ajouter_Ouvert].Num_Port = Port_Equipement->Num_Port;
				Resultat->Port_Ouvert[Nb_Port_Ajouter_Ouvert].Derniere_Modification_Etat_Port = Port_Equipement->Derniere_Modification_Etat_Port;
				Resultat->Port_Ouvert[Nb_Port_Ajouter_Ouvert].Flags = Port_Equipement->Flags;
				Nb_Port_Ajouter_Ouvert++;
			}
			Resultat->Nb_Port_Ouvert++;
		}
	
		if  (Port_Equipement->Etat_Port == FERMER) {
			if (Nb_Port_Ajouter_Fermer<MAX_PORTS){
				Resultat->Port_Fermer[Nb_Port_Ajouter_Fermer].Num_Port = Port_Equipement->Num_Port;
				Resultat->Port_Fermer[Nb_Port_Ajouter_Fermer].Derniere_Modification_Etat_Port = Port_Equipement->Derniere_Modification_Etat_Port;
				Resultat->Port_Fermer[Nb_Port_Ajouter_Fermer].Flags = Port_Equipement->Flags;
				Nb_Port_Ajouter_Fermer++;
			}
			Resultat->Nb_Port_Fermer++;
		}
	
		if (Port_Equipement->Etat_Port == FIREWALLED) {
			if (Nb_Port_Ajouter_Firewalled<MAX_PORTS){
				Resultat->Port_Firewalled[Nb_Port_Ajouter_Firewalled].Num_Port = Port_Equipement->Num_Port;
				Resultat->Port_Firewalled[Nb_Port_Ajouter_Firewalled].Derniere_Modification_Etat_Port = Port_Equipement->Derniere_Modification_Etat_Port;
				Nb_Port_Ajouter_Firewalled++;
			}
			Resultat->Nb_Port_Firewaller++;
		}
		i++;
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
	

	PROTECT_LISTE_ETAT_UDP();
	for (i = 0; i < NB_PORT_UDP; i++)
		Resultat->Liste_Etat_Udp[i] = Equip->Liste_Etat_Udp[i];
	DEPROTECT_LISTE_ETAT_UDP();

		
	if (memcmp (Equip->ID_Equipement, Mon_Adresse_MAC, MAC_ADDR_LEN) != 0) {
		Equip->Agent_AutoScan = FALSE;
		Resultat->Agent_AutoScan = FALSE;
		Cherche_Un_Port_Avec_Etat (Equip, OUVERT, &Resultat->Port_Signature_Ouvert);
		Cherche_Un_Port_Avec_Etat (Equip, FERMER, &Resultat->Port_Signature_Fermer);
		DEBUG_INFO("Scan os (%s) avec comme num port ouvert[%d]   num port fermer[%d]", Equip->Adresse_Ip_Char,Resultat->Port_Signature_Ouvert,Resultat->Port_Signature_Fermer);

		Retour_Info_Scan(30,"  Fingerprinting  ",Equip);
		DEBUG_INFO("Scan  Signature");
		Equip->Time_Debut_Scan_Signature = time(NULL);
		if ((Resultat->Port_Signature_Ouvert != -1) || (Resultat->Port_Signature_Fermer!= -1))
			Envoie_Trames_Scanne_Os (Equip, Resultat->Port_Signature_Ouvert, Resultat->Port_Signature_Fermer);
		Equip->Time_Fin_Scan_Signature = time(NULL);
		Equip->Temps_Scan_Signature = (gint) difftime(Equip->Time_Fin_Scan_Signature, Equip->Time_Debut_Scan_Signature);
		Resultat->Temps_Scan_Signature = Equip->Temps_Scan_Signature;
		
	
		memcpy (Resultat->Reponse_Signature, Equip->Reponse_Signature, sizeof (struct struct_Reponse) * NB_SCAN_OS);
		memcpy (Resultat->Reponse_Signature_Debug, Equip->Reponse_Signature_Debug, sizeof (struct struct_Reponse_debug) * NB_SCAN_OS);
	
		Resultat->GateWay = Equip->GateWay;
	
		Resultat->Routage = Equip->Routage_Activer;
		Scan_Port (SCAN_FTP, Equip, Resultat->Entete_FTP, &Equip->Time_Debut_Scan_Service_Ftp, &Equip->Time_Fin_Scan_Service_Ftp, &Equip->Temps_Scan_Service_Ftp,&Resultat->Temps_Scan_Service_Ftp);
		Scan_Port (SCAN_HTTP, Equip, Resultat->Entete_HTTP, &Equip->Time_Debut_Scan_Service_Http, &Equip->Time_Fin_Scan_Service_Http, &Equip->Temps_Scan_Service_Http,&Resultat->Temps_Scan_Service_Http);
		Scan_Port (SCAN_IMAP, Equip, Resultat->Entete_IMAP, &Equip->Time_Debut_Scan_Service_Imap, &Equip->Time_Fin_Scan_Service_Imap, &Equip->Temps_Scan_Service_Imap, &Resultat->Temps_Scan_Service_Imap);
		Scan_Port (SCAN_SMTP, Equip, Resultat->Entete_SMTP, &Equip->Time_Debut_Scan_Service_Smtp, &Equip->Time_Fin_Scan_Service_Smtp, &Equip->Temps_Scan_Service_Smtp, &Resultat->Temps_Scan_Service_Smtp);
		Scan_Port (SCAN_SSH, Equip, Resultat->Entete_SSH, 	&Equip->Time_Debut_Scan_Service_Ssh, &Equip->Time_Fin_Scan_Service_Ssh, &Equip->Temps_Scan_Service_Ssh, &Resultat->Temps_Scan_Service_Ssh);
		Scan_Port (SCAN_TELNET, Equip, Resultat->Entete_TELNET, &Equip->Time_Debut_Scan_Service_Telnet, &Equip->Time_Fin_Scan_Service_Telnet, &Equip->Temps_Scan_Service_Telnet, &Resultat->Temps_Scan_Service_Telnet);
		Scan_Port (SCAN_SIP, Equip, Resultat->Entete_SIP, &Equip->Time_Debut_Scan_Service_Sip, &Equip->Time_Fin_Scan_Service_Sip, &Equip->Temps_Scan_Service_Sip, &Resultat->Temps_Scan_Service_Sip);
		Scan_Port (SCAN_POP3, Equip, Resultat->Entete_POP3, &Equip->Time_Debut_Scan_Service_Pop3, &Equip->Time_Fin_Scan_Service_Pop3, &Equip->Temps_Scan_Service_Pop3, &Resultat->Temps_Scan_Service_Pop3);
		Scan_Port (SCAN_VNC0, Equip, Resultat->Entete_VNC0, &Equip->Time_Debut_Scan_Service_Vnc0, &Equip->Time_Fin_Scan_Service_Vnc0,	&Equip->Temps_Scan_Service_Vnc0, &Resultat->Temps_Scan_Service_Vnc0);
		Scan_Port (SCAN_VNC1, Equip, Resultat->Entete_VNC1, &Equip->Time_Debut_Scan_Service_Vnc1, &Equip->Time_Fin_Scan_Service_Vnc1,	&Equip->Temps_Scan_Service_Vnc1, &Resultat->Temps_Scan_Service_Vnc1);
		Scan_Port (SCAN_RTSP, Equip, Resultat->Entete_RTSP, &Equip->Time_Debut_Scan_Service_Rtsp, &Equip->Time_Fin_Scan_Service_Rtsp,	&Equip->Temps_Scan_Service_Rtsp, &Resultat->Temps_Scan_Service_Rtsp);
		Scan_Port (SCAN_IPP, Equip, Resultat->Entete_IPP, &Equip->Time_Debut_Scan_Service_Ipp, &Equip->Time_Fin_Scan_Service_Ipp,&Equip->Temps_Scan_Service_Ipp,&Resultat->Temps_Scan_Service_Ipp);
		Scan_Port (SCAN_VNC_HTTP, Equip, Resultat->Entete_VNC_HTTP, &Equip->Time_Debut_Scan_Service_VncHttp, &Equip->Time_Fin_Scan_Service_VncHttp,&Equip->Temps_Scan_Service_VncHttp,&Resultat->Temps_Scan_Service_VncHttp);
	}
	else {
		Equip->Agent_AutoScan = TRUE;
		Resultat->Agent_AutoScan = TRUE;
	}
			

	if (Cherche_Si_Port_Ouvert (Equip, PORT_NETBIOS)) {
		Retour_Info_Scan(90,"Netbios",Equip);
		DEBUG_INFO("Scan  NETBIOS");
		Equip->Time_Debut_Samba = time(NULL);
		Tentative_NetBios = 0;
		if (Info_Demarrage.Normal_Mode)
			Tentative_NetBios_MAX = 2;
		else
			Tentative_NetBios_MAX = 1;
	
		if (Equip->Agent_AutoScan)
			Tentative_NetBios_MAX = 1;
		Demande_NetBios (Equip, Tentative_NetBios);
		while ((Tentative_NetBios < Tentative_NetBios_MAX) && (!Equip->Resultat_Netbios)) {
			Tentative_NetBios++;
			sleep (1);
			DEBUG_INFO("Nouvelle tentative NETBIOS tentatice(%d) (%s)", Tentative_NetBios, Equip->Adresse_Ip_Char);
			Demande_NetBios (Equip, Tentative_NetBios);
		}
		memcpy (&Resultat->Smb, &Equip->Smb, sizeof (struct struct_Smb)); 
		Equip->Time_Fin_Samba = time(NULL);
		Equip->Temps_Samba = (gint) difftime(Equip->Time_Fin_Samba, Equip->Time_Debut_Samba);
		Resultat->Temps_Samba = Equip->Temps_Samba;
	}



	Equip->Time_Fin = time(NULL);
	Equip->Temps_Scan_Complet = (gint) difftime(Equip->Time_Fin, Equip->Time_Debut);
	Resultat->Temps_Scan_Complet = Equip->Temps_Scan_Complet;
	DEBUG_INFO("Rescan Nb_Socket_Ouverte [%d] %s Total : %d - ResolvDNS :%d - ScanPort :%d - ScanSignature :%d - Ftp : %d - http:%d - imap:%d - telnet:%d - samba:%d - snmp:%d",Get_Nb_Socket_Ouvert(),Equip->Adresse_Ip_Char,Resultat->Temps_Scan_Complet,Resultat->Temps_Resolv_DNS,	Resultat->Temps_Scan_Port,Resultat->Temps_Scan_Signature,Resultat->Temps_Scan_Service_Ftp,Resultat->Temps_Scan_Service_Http,Resultat->Temps_Scan_Service_Imap,Resultat->Temps_Scan_Service_Telnet,Resultat->Temps_Samba,Resultat->Temps_Scan_Snmp);

	Resultat->Scan_Accompli = TRUE;
	Communication_Envoie_Donnee (RETOUR_RAFRAICHIR_EQUIPEMENT, (gpointer *) Resultat, sizeof (struct struct_Retour_Rafraichir_Equipement));
	Sem_Set_Boolean(&Equip->Scan_Signature, FALSE);
	Sem_Set_Boolean(&Equip->Scan_Port_En_Cours ,FALSE);
	MYFREE(Resultat);
	Stop_Thread (NULL);
}

void Demande_Rafraichir_Equipement (gpointer * Donnee)
{
	struct struct_Demande_Rachaichir_Etat_Equipement *Message = (struct struct_Demande_Rachaichir_Etat_Equipement *) Donnee;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement;

	ERROR_if (Donnee == NULL);
	Recherche_Index_Equipement (Message->ID_Equipement,TRUE, &ID_Equipement);
	if (ID_Equipement == EQUIPEMENT_UNKNOWN) {
		DEBUG_WARNING("Refresh (Equipment unknown)");
		return;	
	}
	if (ID_Equipement == EQUIPEMENT_NO_IP) {
		DEBUG_WARNING("Refresh (no IP)");
		return;	
	}
	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);
	ERROR_if (Equipement == NULL);
	Start_Thread (&Rescanne_Equipement, (gpointer *) Equipement); 
}


void Scanne_Equipement (struct Struct_Liste_Equipement *Equip)
{
	struct struct_Retour_Affiche_Nouveau_Equipement Affiche_Nouveau_Equipement;
	struct struct_Nouveau_Equipement *Resultat = NULL;
	struct Struct_Port *Port_Equipement = NULL;
	gint    i = 0,Nb_Port_Ajouter_Ouvert = 0,Nb_Port_Ajouter_Fermer = 0, Nb_Port_Ajouter_Firewalled = 0, Tentative_NetBios = 0,Tentative_NetBios_MAX = 0;
	
	ERROR_if (Equip == NULL);
#ifdef __LOG_ENABLE
  	pthread_once(&once_cle_privee,create_cle_privee);
	MALLOC(Equip->vs_Donnee_Cle,sizeof(struct struct_Cle_Privee));
	Equip->vs_Donnee_Cle->Categorie = CATEG_THREAD_SCAN;
	Equip->vs_Donnee_Cle->Pointeur_Data = Equip;
	Equip->vs_Donnee_Cle->ID_Fichier_Log = Equip->ID_Fichier_Log; 
	pthread_setspecific(Cle_privee_thread,Equip->vs_Donnee_Cle);
#endif
	
	DEBUG_INFO("Scanne_Equipement  Socket ouvert[%d]  Thread[%d] ",Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan));

	MALLOC(Resultat,sizeof(struct struct_Nouveau_Equipement));

	Equip->Time_Debut = time(NULL);

	Resultat->ID_Reseau = Info_Demarrage.ID_Reseau;
	memcpy (Resultat->ID_Equipement, Equip->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
	Sem_Set_Boolean(&Equip->Scan_Port_En_Cours,TRUE);
	

	if (!Equip->Agent_AutoScan){
		Retour_Info_Scan(1,"Dns",Equip);
		DEBUG_INFO("Resolution du nom DNS");
		Equip->Time_Debut_Resolv_DNS = time(NULL);
		Recupere_Nom_DNS (Equip,Resultat->Nom_DNS);
		Equip->Time_Fin_Resolv_DNS = time(NULL);
		Equip->Temps_Resolv_DNS = (gint) difftime(Equip->Time_Fin_Resolv_DNS, Equip->Time_Debut_Resolv_DNS);
		Resultat->Temps_Resolv_DNS = Equip->Temps_Resolv_DNS;
	}

	

	Retour_Info_Scan(2,"TCP/IP",Equip);
	DEBUG_INFO("Scanne des Ports TCP");
	Equip->Time_Debut_Scan_Port = time(NULL);
	Equip->mss = FALSE;
	if (Scanne_Toutes_Les_Ports_TCP (Equip,FALSE,FALSE)){
		DEBUG_WARNING("Scanne des Ports TCP + Mss");
		Equip->mss = !Scanne_Toutes_Les_Ports_TCP (Equip,TRUE,FALSE); 
	}
	Resultat->mss = Equip->mss;
	Equip->Time_Fin_Scan_Port = time(NULL);
	Equip->Temps_Scan_Port = (gint) difftime(Equip->Time_Fin_Scan_Port, Equip->Time_Debut_Scan_Port);
	Resultat->Temps_Scan_Port = Equip->Temps_Scan_Port;

	

	Retour_Info_Scan(25,"Udp",Equip);
	DEBUG_INFO("Scanne des Ports UDP");
	Equip->Time_Debut_Scan_Port_Udp = time(NULL);
	Scanne_Toutes_Les_Ports_UDP (Equip);
	Equip->Time_Fin_Scan_Port_Udp = time(NULL);
	Equip->Temps_Scan_Port_Udp = (gint) difftime(Equip->Time_Fin_Scan_Port_Udp, Equip->Time_Debut_Scan_Port_Udp);
	Resultat->Temps_Scan_Port_Udp = Equip->Temps_Scan_Port_Udp;

	

	Retour_Info_Scan(30,"Ports",Equip);

	DEBUG_INFO("Traitement des Ports");
	PROTECT_GLIST_PORT();
	PROTECT_STRUCT_PORT_EQUIPEMENT();
	i= 0;
	Resultat->Nb_Port_Total_Scanner = g_list_length (Equip->GList_Port);

	Resultat->Nb_Port_Ouvert = 0;
	Resultat->Nb_Port_Fermer = 0;
	Resultat->Nb_Port_Firewaller = 0;
	Nb_Port_Ajouter_Ouvert = 0;
	Nb_Port_Ajouter_Fermer = 0;
	Nb_Port_Ajouter_Firewalled = 0;
	while (i< Resultat->Nb_Port_Total_Scanner)  {
		Port_Equipement = g_list_nth_data (Equip->GList_Port, i);
	
		if (Port_Equipement->Etat_Port == OUVERT){			
			if (Nb_Port_Ajouter_Ouvert<MAX_PORTS) {
				Resultat->Port_Ouvert[Nb_Port_Ajouter_Ouvert].Num_Port = Port_Equipement->Num_Port;
				Resultat->Port_Ouvert[Nb_Port_Ajouter_Ouvert].Derniere_Modification_Etat_Port = Port_Equipement->Derniere_Modification_Etat_Port;
				Resultat->Port_Ouvert[Nb_Port_Ajouter_Ouvert].Flags = Port_Equipement->Flags;
				Nb_Port_Ajouter_Ouvert++;
			}
			Resultat->Nb_Port_Ouvert++;
		}
	
		if  (Port_Equipement->Etat_Port == FERMER) {
			if (Nb_Port_Ajouter_Fermer<MAX_PORTS){
				Resultat->Port_Fermer[Nb_Port_Ajouter_Fermer].Num_Port = Port_Equipement->Num_Port;
				Resultat->Port_Fermer[Nb_Port_Ajouter_Fermer].Derniere_Modification_Etat_Port = Port_Equipement->Derniere_Modification_Etat_Port;
				Resultat->Port_Fermer[Nb_Port_Ajouter_Fermer].Flags = Port_Equipement->Flags;
				Nb_Port_Ajouter_Fermer++;
			}
			Resultat->Nb_Port_Fermer++;
		}
	
		if (Port_Equipement->Etat_Port == FIREWALLED) {
			if (Nb_Port_Ajouter_Firewalled<MAX_PORTS){
				Resultat->Port_Firewalled[Nb_Port_Ajouter_Firewalled].Num_Port = Port_Equipement->Num_Port;
				Resultat->Port_Firewalled[Nb_Port_Ajouter_Firewalled].Derniere_Modification_Etat_Port = Port_Equipement->Derniere_Modification_Etat_Port;
				Nb_Port_Ajouter_Firewalled++;
			}
			Resultat->Nb_Port_Firewaller++;
		}
		i++;
	}
	DEPROTECT_STRUCT_PORT_EQUIPEMENT();
	DEPROTECT_GLIST_PORT();
	

	PROTECT_LISTE_ETAT_UDP();
	for (i = 0; i < NB_PORT_UDP; i++)
		Resultat->Liste_Etat_Udp[i] = Equip->Liste_Etat_Udp[i];
	DEPROTECT_LISTE_ETAT_UDP();

	
	if (!Equip->Agent_AutoScan){
		Cherche_Un_Port_Avec_Etat (Equip, OUVERT, &Resultat->Port_Signature_Ouvert);
		Cherche_Un_Port_Avec_Etat (Equip, FERMER, &Resultat->Port_Signature_Fermer);
		DEBUG_INFO("Scan os (%s) avec comme num port ouvert[%d]   num port fermer[%d]", Equip->Adresse_Ip_Char,Resultat->Port_Signature_Ouvert,Resultat->Port_Signature_Fermer);
	
		Retour_Info_Scan(30,"  Fingerprinting  ",Equip);
		DEBUG_INFO("Scan  Signature");
		Equip->Time_Debut_Scan_Signature = time(NULL);
		if ((Resultat->Port_Signature_Ouvert != -1) || (Resultat->Port_Signature_Fermer!= -1))
			Envoie_Trames_Scanne_Os (Equip, Resultat->Port_Signature_Ouvert, Resultat->Port_Signature_Fermer);
		Equip->Time_Fin_Scan_Signature = time(NULL);
		Equip->Temps_Scan_Signature = (gint) difftime(Equip->Time_Fin_Scan_Signature, Equip->Time_Debut_Scan_Signature);
		Resultat->Temps_Scan_Signature = Equip->Temps_Scan_Signature;
	
	
		memcpy (Resultat->Reponse_Signature, Equip->Reponse_Signature, sizeof (struct struct_Reponse) * NB_SCAN_OS);
		memcpy (Resultat->Reponse_Signature_Debug, Equip->Reponse_Signature_Debug, sizeof (struct struct_Reponse_debug) * NB_SCAN_OS);
	
		Resultat->GateWay = Equip->GateWay;
	
		Resultat->Routage = Equip->Routage_Activer;
		Scan_Port (SCAN_FTP, Equip, Resultat->Entete_FTP, &Equip->Time_Debut_Scan_Service_Ftp, &Equip->Time_Fin_Scan_Service_Ftp, &Equip->Temps_Scan_Service_Ftp,&Resultat->Temps_Scan_Service_Ftp);
		Scan_Port (SCAN_HTTP, Equip, Resultat->Entete_HTTP, &Equip->Time_Debut_Scan_Service_Http, &Equip->Time_Fin_Scan_Service_Http, &Equip->Temps_Scan_Service_Http,&Resultat->Temps_Scan_Service_Http);
		Scan_Port (SCAN_IMAP, Equip, Resultat->Entete_IMAP, &Equip->Time_Debut_Scan_Service_Imap, &Equip->Time_Fin_Scan_Service_Imap, &Equip->Temps_Scan_Service_Imap, &Resultat->Temps_Scan_Service_Imap);
		Scan_Port (SCAN_SMTP, Equip, Resultat->Entete_SMTP, &Equip->Time_Debut_Scan_Service_Smtp, &Equip->Time_Fin_Scan_Service_Smtp, &Equip->Temps_Scan_Service_Smtp, &Resultat->Temps_Scan_Service_Smtp);
		Scan_Port (SCAN_SSH, Equip, Resultat->Entete_SSH, 	&Equip->Time_Debut_Scan_Service_Ssh, &Equip->Time_Fin_Scan_Service_Ssh, &Equip->Temps_Scan_Service_Ssh, &Resultat->Temps_Scan_Service_Ssh);
		Scan_Port (SCAN_TELNET, Equip, Resultat->Entete_TELNET, &Equip->Time_Debut_Scan_Service_Telnet, &Equip->Time_Fin_Scan_Service_Telnet, &Equip->Temps_Scan_Service_Telnet, &Resultat->Temps_Scan_Service_Telnet);
		Scan_Port (SCAN_SIP, Equip, Resultat->Entete_SIP, &Equip->Time_Debut_Scan_Service_Sip, &Equip->Time_Fin_Scan_Service_Sip, &Equip->Temps_Scan_Service_Sip, &Resultat->Temps_Scan_Service_Sip);
		Scan_Port (SCAN_POP3, Equip, Resultat->Entete_POP3, &Equip->Time_Debut_Scan_Service_Pop3, &Equip->Time_Fin_Scan_Service_Pop3, &Equip->Temps_Scan_Service_Pop3, &Resultat->Temps_Scan_Service_Pop3);
		Scan_Port (SCAN_VNC0, Equip, Resultat->Entete_VNC0, &Equip->Time_Debut_Scan_Service_Vnc0, &Equip->Time_Fin_Scan_Service_Vnc0,	&Equip->Temps_Scan_Service_Vnc0, &Resultat->Temps_Scan_Service_Vnc0);
		Scan_Port (SCAN_VNC1, Equip, Resultat->Entete_VNC1, &Equip->Time_Debut_Scan_Service_Vnc1, &Equip->Time_Fin_Scan_Service_Vnc1,	&Equip->Temps_Scan_Service_Vnc1, &Resultat->Temps_Scan_Service_Vnc1);
		Scan_Port (SCAN_RTSP, Equip, Resultat->Entete_RTSP, &Equip->Time_Debut_Scan_Service_Rtsp, &Equip->Time_Fin_Scan_Service_Rtsp,	&Equip->Temps_Scan_Service_Rtsp, &Resultat->Temps_Scan_Service_Rtsp);
		Scan_Port (SCAN_IPP, Equip, Resultat->Entete_IPP, &Equip->Time_Debut_Scan_Service_Ipp, &Equip->Time_Fin_Scan_Service_Ipp,&Equip->Temps_Scan_Service_Ipp,&Resultat->Temps_Scan_Service_Ipp);
		Scan_Port (SCAN_VNC_HTTP, Equip, Resultat->Entete_VNC_HTTP, &Equip->Time_Debut_Scan_Service_VncHttp, &Equip->Time_Fin_Scan_Service_VncHttp,&Equip->Temps_Scan_Service_VncHttp,&Resultat->Temps_Scan_Service_VncHttp);
	}
	

	Retour_Info_Scan(88,"Netbios",Equip);
	if (Cherche_Si_Port_Ouvert (Equip, PORT_NETBIOS)) {
		DEBUG_INFO("Scan  NETBIOS");
		Equip->Time_Debut_Samba = time(NULL);
		Tentative_NetBios = 0;
		if (Info_Demarrage.Normal_Mode)
			Tentative_NetBios_MAX = 2;
		else
			Tentative_NetBios_MAX = 1;
	
		if (Equip->Agent_AutoScan)
			Tentative_NetBios_MAX = 1;
		Demande_NetBios (Equip, Tentative_NetBios);
		while ((Tentative_NetBios < Tentative_NetBios_MAX) && (!Equip->Resultat_Netbios)) {
			Tentative_NetBios++;
			sleep (1);
			DEBUG_INFO("Nouvelle tentative NETBIOS tentatice(%d) (%s)", Tentative_NetBios, Equip->Adresse_Ip_Char);
			Demande_NetBios (Equip, Tentative_NetBios);
		}
		memcpy (&Resultat->Smb, &Equip->Smb, sizeof (struct struct_Smb)); 
		Equip->Time_Fin_Samba = time(NULL);
		Equip->Temps_Samba = (gint) difftime(Equip->Time_Fin_Samba, Equip->Time_Debut_Samba);
		Resultat->Temps_Samba = Equip->Temps_Samba;
	}

			

#ifdef        __SNMP_ENABLE
	Retour_Info_Scan(98,"Snmp",Equip);
	DEBUG_INFO("Scan  Snmp");
	Equip->Time_Debut_Scan_Snmp = time(NULL);
	Demande_Snmp_Equipement (Equip);
	Equip->Time_Fin_Scan_Snmp = time(NULL);
	Equip->Temps_Scan_Snmp = (gint) difftime(Equip->Time_Fin_Scan_Snmp, Equip->Time_Debut_Scan_Snmp);
	Resultat->Temps_Scan_Snmp = Equip->Temps_Scan_Snmp;

	Resultat->Snmp_Ouvert = Equip->Snmp_Ouvert;
	memcpy (&Resultat->Snmp, &Equip->Snmp, sizeof (struct Struct_Snmp));
#endif
	

	Equip->Time_Fin = time(NULL);
	Equip->Temps_Scan_Complet = (gint) difftime(Equip->Time_Fin, Equip->Time_Debut);
	Resultat->Temps_Scan_Complet = Equip->Temps_Scan_Complet;
	DEBUG_INFO("Nb_Socket_Ouverte [%d]   %s Total : %d - ResolvDNS :%d - ScanPort :%d - ScanSignature :%d - Ftp : %d - http:%d - imap:%d - telnet:%d - samba:%d - snmp:%d",Get_Nb_Socket_Ouvert(),Equip->Adresse_Ip_Char,Resultat->Temps_Scan_Complet,Resultat->Temps_Resolv_DNS,	Resultat->Temps_Scan_Port,Resultat->Temps_Scan_Signature,Resultat->Temps_Scan_Service_Ftp,Resultat->Temps_Scan_Service_Http,Resultat->Temps_Scan_Service_Imap,Resultat->Temps_Scan_Service_Telnet,Resultat->Temps_Samba,Resultat->Temps_Scan_Snmp);

	Communication_Envoie_Donnee (NOUVEAU_EQUIPEMENT, (gpointer *) Resultat, sizeof (struct struct_Nouveau_Equipement));
	Sem_Set_Boolean(&Equip->Scan_Signature, FALSE);
	MYFREE(Resultat);



#ifdef        __SNMP_ENABLE
	if (Equip->Snmp_Ouvert){
		DEBUG_INFO("Lecture snmp --> SNMP_SYSTEM_DEVICE");
		Envoie_Snmp_Liste(Equip,SNMP_SYSTEM_DEVICE);
		DEBUG_INFO("Lecture snmp --> SNMP_SYSTEM_DEVICE = OK");
	}
#endif

	bzero (&Affiche_Nouveau_Equipement, sizeof (struct struct_Retour_Affiche_Nouveau_Equipement));
	Affiche_Nouveau_Equipement.ID_Reseau = Info_Demarrage.ID_Reseau;
	memcpy (Affiche_Nouveau_Equipement.ID_Equipement, Equip->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);

	Affiche_Nouveau_Equipement.Intrus = Equip->Intrus;
	Communication_Envoie_Donnee (AFFICHE_NOUVEAU_EQUIPEMENT, (gpointer *) &Affiche_Nouveau_Equipement, sizeof (struct struct_Retour_Affiche_Nouveau_Equipement));
	DEPROTECT_CODE(Sem_GList_Liste_Thread_Scan);
	if (Equip->Agent_AutoScan){
		Sem_Set_Boolean(&Scan_Machine_Local_Fait, TRUE);
		Start_Scan_Plage();
	}
	Sem_Set_Boolean(&Equip->Scan_Port_En_Cours, FALSE);
	Stop_Thread (NULL);
}
