/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Scan_Netbios.h"
#include "Debug.h"
#include "Info.h"
#include "Reseau.h"
#include "Mem.h"
#ifdef         __WINDOWS_ENABLE  
#include "Win32.h"  
#endif  

char      nbtname[] = { 0x80, 0xf0, 0x00, 0x10, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x43, 0x4b, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x00, 0x00, 0x21, 0x00, 0x01 };

#define MAX_NAME_SIZE 16
	
gchar *get_nbns_name(unsigned char *Datagramme){
	gchar  cname_a = '\0',cname_b = '\0',result = '\0';
	gchar  *Name = NULL;
	gint i = 0,LastChar = 0;

	ERROR_if (Datagramme == NULL);
	MALLOC(Name,sizeof(gchar)*MAX_NAME_SIZE);
	result = '?';
	while ((i<(MAX_NAME_SIZE -1)) && (result!=' ')){ 
		cname_a = (Datagramme[(i*2)+1] - 'A') & 0x0F;
		cname_b = (Datagramme[(i*2)+2] - 'A') & 0x0F;
		result = cname_a << 4 | cname_b;
	
		if (result < '!' || result > '}')
			Name[i] = '_';
		else
			if (result != ' '){
				Name[i] = result;
				LastChar = i;
			}
			else
				Name[i] = 0;
		i++;
	}
	if (LastChar<MAX_NAME_SIZE)
		Name[LastChar+1] = 0;
	else
		Name[MAX_NAME_SIZE -1] = 0;
	DEBUG_INFO("Name [%s]", Name);
	return Name;
}
	

void Change_HostName_Netbios(struct Struct_Liste_Equipement *Equipement,char *HostName){
	struct struct_Retour_Rafraichir_Nom_Netbios Message;
		
	ERROR_if (Equipement == NULL);
	ERROR_if (HostName == NULL);	
	if (!Sem_Get_Boolean(&Equipement->Scan_Port_En_Cours)){
		strcpy((char *)Equipement->Smb.HostName,HostName);
		bzero (&Message, sizeof (struct struct_Retour_Rafraichir_Nom_Netbios));
		Message.ID_Reseau = Info_Demarrage.ID_Reseau;
		memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
		strcpy((char *)Message.HostName,HostName);
		Communication_Envoie_Donnee (RETOUR_RAFRAICHIR_NOM_NETBIOS, (gpointer *) & Message, sizeof (Message));
	}
}


void Change_Name_Netbios(struct Struct_Liste_Equipement *Equipement,gchar *Utilisateur){
	struct struct_Retour_Rafraichir_Nom_Utilisateur Message;
		
	ERROR_if (Equipement == NULL);
	ERROR_if (Utilisateur == NULL);	
	if ((!Sem_Get_Boolean(&Equipement->Scan_Port_En_Cours)) && (strcmp((const char *)Equipement->Smb.Utilisateur,Utilisateur)!=0)){
		strcpy((char *)Equipement->Smb.Utilisateur,Utilisateur);
		bzero (&Message, sizeof (struct struct_Retour_Rafraichir_Nom_Utilisateur));
		Message.ID_Reseau = Info_Demarrage.ID_Reseau;
		memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
		strcpy((char *)Message.Utilisateur,Utilisateur);
		Communication_Envoie_Donnee (RETOUR_RAFRAICHIR_NOM_UTILISATEUR, (gpointer *) & Message, sizeof (Message));
	}
	MYFREE(Utilisateur); 
}


void Demande_NetBios (struct Struct_Liste_Equipement * Equipement, gint Tentative)
{
	unsigned char	 buf[TAILLE_BUFFER_NETBIOS];
	struct sockaddr_in sock;
	int Socket=0, i=0, j=0, octet_recu=0;
	gboolean Etat_Socket = FALSE;
	
	ERROR_if (Equipement == NULL);
	usleep (SURCHARGE_EQUIPEMENT);
	bzero (&Equipement->Smb, sizeof (struct struct_Smb));
	bzero (buf, sizeof(buf));
	CREATE_SOCKET(Socket, Equipement->Adresse_Ip_Char, NETBIOS, sock,PF_INET,SOCK_DGRAM,0);
	SENDTO(Socket,nbtname, sizeof (nbtname), 0, &sock, sizeof(struct sockaddr_in));
	DEBUG_INFO("Demande NETBIOS tentative (%d)[%s]", Tentative, Equipement->Adresse_Ip_Char);
	WAITING_READ_SOCKET(Etat_Socket,Socket,2);
	if (Etat_Socket > 0) {
		RECVFROM(octet_recu,Socket,buf,TAILLE_BUFFER_NETBIOS - 1,0,&sock,sizeof(struct sockaddr_in));
		if (octet_recu<0){
			DEBUG_WARNING("recvfrom Failed [%s] Socket %d : %s", Equipement->Adresse_Ip_Char,Socket, strerror (errno));
			CLOSE_SOCKET(Socket); 
			Equipement->Resultat_Netbios = FALSE;	
			return;
		}
		if (octet_recu >= 56) {
			if (buf[56] == 0) {
				CLOSE_SOCKET(Socket); 
				Equipement->Resultat_Netbios = FALSE;
				return;
			}
			for (i = 0; i < buf[56]; i++) {
				strncpy ((char *)Equipement->Smb.Smb_Registration[i].Name, (char *)&buf[57 + (i * 18)], 15);
				Equipement->Smb.Smb_Registration[i].Name[15] = 0;
				j = 14;
				while ((j != 0) && (Equipement->Smb.Smb_Registration[i].Name[j] == 0x20)) {
					Equipement->Smb.Smb_Registration[i].Name[j] = 0;
					j--;
				}
				for (j = 0; j < strlen ((char *)Equipement->Smb.Smb_Registration[i].Name); j++)
					if (Equipement->Smb.Smb_Registration[i].Name[j] < 31 || Equipement->Smb.Smb_Registration[i].Name[j] > 126)
						Equipement->Smb.Smb_Registration[i].Name[j] = '_';
			
				Equipement->Smb.Smb_Registration[i].Numero = buf[57 + 15 + (i * 18)];
			
				if (buf[57 + 15 + 1 + (i * 18)] & 128) {
					Equipement->Smb.Smb_Registration[i].Type = REGISTRATION_GROUP;
					DEBUG_INFO( "GROUP %s <%02X>", Equipement->Adresse_Ip_Char, Equipement->Smb.Smb_Registration[i].Name, Equipement->Smb.Smb_Registration[i].Numero);
				}
				else {
					Equipement->Smb.Smb_Registration[i].Type = REGISTRATION_UNIQUE;
					DEBUG_INFO("UNIQUE   %s <%02X>", Equipement->Adresse_Ip_Char, Equipement->Smb.Smb_Registration[i].Name, Equipement->Smb.Smb_Registration[i].Numero);
				}
				if ((Equipement->Smb.Smb_Registration[i].Numero == 0x20) && (Equipement->Smb.Smb_Registration[i].Type = REGISTRATION_UNIQUE))
					strcpy ((char *)Equipement->Smb.HostName, (char *)Equipement->Smb.Smb_Registration[i].Name);
			
				if ((Equipement->Smb.Smb_Registration[i].Numero == 0x00) && (Equipement->Smb.Smb_Registration[i].Type == REGISTRATION_GROUP))
					strcpy ((char *)Equipement->Smb.DomainName, (char *)Equipement->Smb.Smb_Registration[i].Name);
			
				if ((Equipement->Smb.Smb_Registration[i].Numero == 0x03) && (Equipement->Smb.Smb_Registration[i].Type == REGISTRATION_UNIQUE))
					strcpy ((char *)Equipement->Smb.Utilisateur, (char *)Equipement->Smb.Smb_Registration[i].Name);
			}
		}
	}
	else {
		CLOSE_SOCKET(Socket); 
		DEBUG_INFO("Aucune reponse NETBIOS tentatice(%d) (%s)", Tentative, Equipement->Adresse_Ip_Char);
		Equipement->Resultat_Netbios = FALSE;
		return;
	}		
	CLOSE_SOCKET(Socket); 
	Equipement->Resultat_Netbios = TRUE;
	return;
}
