/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Scan_Port_UDP.h"
#include "Port_UDP.h"
#include "Debug.h"
#include "Info.h"
#include "Mem.h"
#include "Reseau.h"

char      Udp[] = { 0x45, 0x00, 0x00, 0x3A, 0x00, 0x00, 0x00, 0x00, 0x4C, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x26, 0x00, 0x00, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee };
extern struct Struct_Liste_Port_UDP Liste_Port_UDP[];
	
gboolean Traitement_Port_UDP_Fermer (struct Struct_Liste_Equipement *Equipement, struct struct_trame *Packet, unsigned char Taille_Trame, unsigned short Taille_Ip, unsigned char Type_ICMP, unsigned char Code_ICMP)
{
	unsigned short Port_Destination = 0, Port_Source = 0, Id_Ip = 0;
	gint      Id_Reponse_Trame = 0;
#ifdef __LOG_ENABLE
	gchar  *Texte = NULL;
#endif
	
	ERROR_if (Equipement == NULL);
	ERROR_if (Packet == NULL);

	if (Sem_Get_Boolean(&Equipement->Scanne_Port_En_Cours)) {
		Id_Reponse_Trame = 0;
		if ((Type_ICMP == DESTINATION_UNREACHABLE) && (Code_ICMP == PORT_UNREACHABLE)) {
			memcpy (&Id_Ip, &Packet->Datagramme[Taille_Ip + 12], 2);
			Id_Ip = htons (Id_Ip);
			memcpy (&Port_Source, &Packet->Datagramme[Taille_Ip + 28], 2);
			Port_Source = htons (Port_Source);
			memcpy (&Port_Destination, &Packet->Datagramme[Taille_Ip + 30], 2);
			Port_Destination = htons (Port_Destination);
			Modifie_Etat_Port_UDP_Equipement(Equipement,Port_Destination,FERMER_UDP);
#ifdef __LOG_ENABLE
			STRDUP_PRINTF(Texte,"[LOG] [%s](%s:%d):  UDP FERMER (%d)\n",__FILE__, __PRETTY_FUNCTION__, __LINE__,Port_Destination);
			Log_Equipement(Equipement,Texte);
			MYFREE(Texte);
#endif
			Nouveau_Octet_Recu (Taille_Trame);
		}
	}
	return FALSE;
}


#ifndef  __PCAP_ENABLE
void Envoie_Trame_Scan_Port_UDP (struct Struct_Liste_Equipement *Equip, gint Num_Port, gint Socket, struct sockaddr *sock)
#else
void Envoie_Trame_Scan_Port_UDP (struct Struct_Liste_Equipement *Equip, gint Num_Port, pcap_t * captureHandle)
#endif
{
	struct struct_trame PacketScan;
	unsigned short Id_Ip = 0,Port = 0;
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT/10);	
	bzero (&PacketScan, sizeof (struct struct_trame));
	Id_Ip = (unsigned short) rand_r(&graine_rand_r);
	Port = ((unsigned short) rand_r(&graine_rand_r));
	Port = (Port & 0x0FFF) + 0xF000;
	DEBUG_INFO("Envoie UDP (%d)   ID(%02X)  PORT(%d)", Num_Port,Id_Ip, Port);

	memcpy (&PacketScan.Dest_MAC, &Equip->Adresse_MAC, ETH_ALEN);
	memcpy (&PacketScan.Source_MAC, Mon_Adresse_MAC, ETH_ALEN);
	PacketScan.Proto = htons (TRAME_IP);
	memcpy (&PacketScan.Datagramme,Udp , 58);
	PacketScan.Datagramme[4] = (Id_Ip >> 8) & 0xFF;
	PacketScan.Datagramme[5] = Id_Ip & 0xFF;
	memcpy (&PacketScan.Datagramme[12], &Mon_Adresse_Ip, IP_ADDR_LEN);
	memcpy (&PacketScan.Datagramme[16], &Equip->Adresse_Ip, IP_ADDR_LEN);
	ckecksum_r(Equip,&PacketScan.Datagramme[0], 20, 0);
	Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
	memcpy (&PacketScan.Datagramme[10], &Equip->CheckSum_Calc, 2);
	PacketScan.Datagramme[20] = (Port >> 8) & 0xFF;
	PacketScan.Datagramme[21] = Port & 0xFF;
	PacketScan.Datagramme[22] = (Num_Port >> 8) & 0xFF;
	PacketScan.Datagramme[23] = Num_Port & 0xFF;
	ckecksum_r(Equip,&PacketScan.Datagramme[12], 46, 38 + 17);
	Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
	memcpy (&PacketScan.Datagramme[26], &Equip->CheckSum_Calc, 2);
#ifndef   __PCAP_ENABLE
	_SENDTO(Socket, &PacketScan, 72, 0, sock, sizeof (struct sockaddr));
#else
	SENDTO_PCAP(captureHandle,(u_char *) &PacketScan, 72);
 #endif
}

void Scanne_Toutes_Les_Ports_UDP (struct Struct_Liste_Equipement *Equipement){
	gint Tentative = 0, i = 0;
#ifdef  __PCAP_ENABLE
	pcap_t    * captureHandle;
	char errbuf[PCAP_ERRBUF_SIZE];
#endif

	ERROR_if (Equipement == NULL);
	if (memcmp (Equipement->ID_Equipement, Mon_Adresse_MAC, MAC_ADDR_LEN) == 0)
		return;

	Initialisation_Liste_Port_UDP_Equipement (Equipement);
	Sem_Set_Boolean(&Equipement->Scanne_Port_En_Cours, TRUE);
#ifndef  __PCAP_ENABLE
	int       Socket;
	struct sockaddr sock;
	OPEN_SOCKET(Socket,sock,Info_Demarrage,AF_INET,SOCK_PACKET,htons (ETH_P_ALL));
#else
	OPEN_SOCKET_PCAP(captureHandle,Info_Demarrage,errbuf) ;
#endif
	Tentative = 0;	
	do {
		DEBUG_INFO("Tentative %d",Tentative);
		for (i = 0; i < NB_PORT_UDP; i++) {
			if (Get_Etat_Port_UDP(Equipement,i) == UNKOWN_UDP) {
#ifndef  __PCAP_ENABLE
				Envoie_Trame_Scan_Port_UDP (Equipement, Liste_Port_UDP[i].Num_Port_UDP, Socket, &sock);
#else
				Envoie_Trame_Scan_Port_UDP (Equipement, Liste_Port_UDP[i].Num_Port_UDP, captureHandle);
#endif
				if (Tentative>=1)
					sleep(1); 
			}
		}
		Retour_Info_Scan(24 + (Tentative * 3) ,"Udp",Equipement);
		switch (Tentative) {
		    case 0:
			    sleep (1);
			    break;
		    case 1:
			    sleep (1);
			    break;
		}
		Tentative++;
	}
	while (!(Test_Retour_Port_UDP_Equipement (Equipement)) && (Tentative != 2));
	DEBUG_INFO("Scanne port UDP fini");
#ifndef  __PCAP_ENABLE
	CLOSE_SOCKET(Socket);
#else
	CLOSE_SOCKET_PCAP(captureHandle);
#endif
	Sem_Set_Boolean(&Equipement->Scanne_Port_En_Cours, FALSE);
}
