/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Scan_Service.h"
#include "Debug.h"
#include "Reseau.h"
#include "Info.h"
#include "Mem.h"
#include <stdlib.h>
#ifdef         __WINDOWS_ENABLE  
#include "Win32.h"  
#endif  
#include <glib.h> 

gboolean connect_service(struct Struct_Liste_Equipement *Equipement,gchar *Protocol,int Port){
	
	ERROR_if (Equipement->vs_banniere != NULL);
	DEBUG_INFO("Connect to %s  [%s]",Protocol, Equipement->Adresse_Ip_Char);
	if ((Equipement->IdSocket = Connection_Equipement (Equipement->Adresse_Ip_Char, Port, TIMEOUT_SOCKET ,FALSE,1)) == SOCK_ERROR) {
		DEBUG_WARNING("Connexion refusée au serveur %s [%s]:[%d]",Protocol, Equipement->Adresse_Ip_Char,Port);
		STRDUP_PRINTF(Equipement->vs_banniere,"[Error] %s: Connection refused",Protocol);
		return FALSE;
	}
	return TRUE;
}

gboolean send_service(struct Struct_Liste_Equipement *Equip,gchar *Requete,gint Taille_Requete,gchar *Protocol,gboolean check_vs_Banniere){
	int Tentative = 0,NbOctet = 0;
	
	ERROR_if (Equip == NULL);
	ERROR_if (Protocol == NULL);
	ERROR_if (Taille_Requete>=TAILLE_MAX_REQUETE_SERVICE);
	if (check_vs_Banniere) 
		ERROR_if (Equip->vs_banniere != NULL);
	

	if (Waiting_Write_Socket(Equip->IdSocket, TIMEOUT_SOCKET_READY_RW)<0){
		STRDUP_PRINTF(Equip->vs_banniere,"[Error] %s : Socket non prete",Protocol); 
		return FALSE; 
	}
	Tentative = 0;

	do { 
		Tentative++;
		DEBUG_INFO("[%s] Socket (%d) tentative (%d)  Taille_Requete(%d)",Protocol,Equip->IdSocket,Tentative,Taille_Requete);
	
		SEND(NbOctet,Equip->IdSocket,Requete, Taille_Requete);
		if (NbOctet < 1){
			DEBUG_WARNING("[%s] NbOctet == %d   %s  Socket (%d)", Protocol, NbOctet,strerror (errno),Equip->IdSocket);
			if (NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					CLOSE_SOCKET(Equip->IdSocket);
					STRDUP_PRINTF(Equip->vs_banniere,"[Error] %s : send NbOctet == -1",Protocol);
					return FALSE;
				}
				sleep(1);
			}
			else
				sleep(1);
		}
		else{
			if (NbOctet != Taille_Requete){
				DEBUG_WARNING("[%s] NbOctet != Taille_Requete [%s]  Socket (%d)", Protocol, strerror (errno),Equip->IdSocket);
				CLOSE_SOCKET(Equip->IdSocket);
				STRDUP_PRINTF(Equip->vs_banniere,"[Error] %s : NbOctet != Taille_Requete",Protocol);
				return FALSE;
			}
		}
	} while((Tentative<10) && (NbOctet == -1) && (errno == EAGAIN || errno == EINTR));
	if (NbOctet != Taille_Requete)  {
		DEBUG_WARNING("[%s] %s Socket (%d)",Protocol , strerror (errno),Equip->IdSocket);
		CLOSE_SOCKET(Equip->IdSocket);
		STRDUP_PRINTF(Equip->vs_banniere,"[Error] %s :  NbOctet != Taille_Requete    CAS IMPOSSIBLE   Tentative(%d)",Protocol,Tentative);
		return FALSE;  
	}
	Nouveau_Octet_Envoye (NbOctet);
	return TRUE; 
}

void read_service(struct Struct_Liste_Equipement *Equip,gchar *debut_entete, gchar *chaine_fin_entete,gchar *Protocol,gboolean deuxlignes){
	gchar	  buffer[TAILLE_REQUETE_SERVICE],*repere = NULL,*repere2 = NULL;
	gint     tailleBanniere = 0,Tentative = 0,Nb_Error_Consecutive = 0,NbOctet = 0,NbOctetTotal = 0,i = 0,nbLigne = 0,retourLecture = 0,tailleReelFinal = 0,Time_Out_Service;
	gboolean fin = FALSE;
	ERROR_if (Equip == NULL);
	ERROR_if (Protocol == NULL);
	ERROR_if (debut_entete == NULL);
	ERROR_if (chaine_fin_entete == NULL);
	ERROR_if (Equip->vs_banniere != NULL);	


	Tentative = 0; Nb_Error_Consecutive = 0; NbOctetTotal = 0; nbLigne = 0;retourLecture=0;tailleReelFinal=0;
	fin = FALSE;
	bzero (buffer, sizeof(buffer));

	STRDUP(Equip->vs_banniere,debut_entete);
	tailleBanniere = strlen(Equip->vs_banniere) + 1;

	Time_Out_Service = TIMEOUT_SOCKET_READY_R_FIRST;

	while ((!fin) && (Tentative!=50) && (Nb_Error_Consecutive<3) && ((retourLecture = Waiting_Read_Socket (Equip->IdSocket, Time_Out_Service))>0)) {
	
		Time_Out_Service = TIMEOUT_SOCKET_READY_READ;
	
		NbOctet = recvfrom (Equip->IdSocket, &buffer, (sizeof(gchar)*TAILLE_REQUETE_SERVICE) - 1 - NbOctetTotal, 0, NULL, 0);
	
		if (NbOctet < 1){ 
			DEBUG_WARNING("NbOctet==[%d] Total[%d] %s  Socket (%d) Erreur Consécutive (%d)",NbOctet,NbOctetTotal, strerror (errno),Equip->IdSocket,Nb_Error_Consecutive);
			if (NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					CLOSE_SOCKET(Equip->IdSocket);
					MYFREE(Equip->vs_banniere); 
					STRDUP_PRINTF(Equip->vs_banniere,"[Error] %s :  NbOctet == -1",Protocol);
					return;
				}
				sleep(1);
			}
			else
				sleep(1);
			Nb_Error_Consecutive++;
		}
		else {
		
			buffer[NbOctet] = 0; 
		
			i = 0;
			while ((i<NbOctet) && (buffer[i]!=0)){
				if (((buffer[i]<32) || (buffer[i]>126)) && (buffer[i]!='\r') && (buffer[i]!='\n'))  
					buffer[i] = '_';
				i++;
			}
			if (i!=NbOctet){
				NbOctet = i; 
				DEBUG_WARNING("Caractère final %d %d",NbOctet,i);
			}
		
			NbOctetTotal = NbOctetTotal + NbOctet;
			Nouveau_Octet_Recu (NbOctet);
			if (!deuxlignes){
			
			
				if ((repere = strstr(buffer,chaine_fin_entete))==NULL){
				
					DEBUG_INFO("Banniere[%d=%d]A[%d]T[%d]  --> [%s]",strlen(Equip->vs_banniere)+1,tailleBanniere,NbOctet,NbOctetTotal,buffer);
					REALLOC(Equip->vs_banniere,(sizeof (gchar) * (tailleBanniere + NbOctet)));
					strcat (Equip->vs_banniere, buffer);
					tailleBanniere = tailleBanniere + NbOctet;
					DEBUG_INFO("NBanniere[%d=%d]",strlen(Equip->vs_banniere)+1,tailleBanniere);
				}
				else {
				
					NbOctet = repere - buffer + 2;
					buffer[NbOctet] = 0;
					DEBUG_INFO("FBanniere[%d=%d] --> [%s] ",strlen(Equip->vs_banniere)+1,tailleBanniere,buffer);
					REALLOC(Equip->vs_banniere,(sizeof (gchar) * (tailleBanniere + NbOctet)));
					strcat (Equip->vs_banniere, buffer);  
					tailleBanniere = tailleBanniere + NbOctet;
					DEBUG_INFO("FBanniere[%d=%d]",strlen(Equip->vs_banniere)+1,tailleBanniere);
					fin = TRUE;
				}
			}
			else {
			
				if ((repere = strstr(buffer,chaine_fin_entete))==NULL){
					DEBUG_INFO( "Repere pas trouvé [%d=%d]A[%d]T[%d]  --> [%s]",strlen(Equip->vs_banniere)+1,tailleBanniere,NbOctet,NbOctetTotal,buffer);
					REALLOC(Equip->vs_banniere,(sizeof (gchar) * (tailleBanniere + NbOctet)));
					strcat (Equip->vs_banniere, buffer);
					tailleBanniere = tailleBanniere + NbOctet;
					DEBUG_INFO("NRepere pas trouvé  [%d=%d]",strlen(Equip->vs_banniere)+1,tailleBanniere);
				}
				else {
				
					if ((repere2 = strstr(repere + strlen(chaine_fin_entete),chaine_fin_entete))!=NULL){
						DEBUG_INFO( "2 lignes en un coups");
						nbLigne++;
						repere = repere2;
					}
					nbLigne++;
					NbOctet = repere - buffer + strlen(chaine_fin_entete);
					DEBUG_INFO( "Repere trouvé : lignes %d  NbOctet(%d)   Taille Futur(%d)  bannière(%d)",nbLigne,NbOctet,(sizeof (gchar) * (tailleBanniere + NbOctet)),tailleBanniere);					
					REALLOC(Equip->vs_banniere,(sizeof (gchar) * (tailleBanniere + NbOctet)));
					buffer[NbOctet] = 0;  
					strcat (Equip->vs_banniere, buffer);
					DEBUG_INFO( "Bannière OK : [%d=%d]A[%d]T[%d]  --> [%s]",strlen(Equip->vs_banniere)+1,tailleBanniere,NbOctet,NbOctetTotal,buffer);
					tailleBanniere = tailleBanniere + NbOctet;
					if (nbLigne == 2){
						DEBUG_INFO("Fin des donnees a recuperer (%s)",Protocol);
						fin = TRUE;
					}
				}
			}
			Nb_Error_Consecutive = 0;
		}
		Tentative++;
		retourLecture = 0;
	}

	if ((retourLecture <0) &&(tailleBanniere == (int)(strlen(debut_entete) + 1))){
		MYFREE(Equip->vs_banniere); 
		switch (retourLecture){
			case SOCK_BROKEN_PIPE :
					
						NbOctet = recvfrom (Equip->IdSocket, &buffer, (sizeof(gchar)*TAILLE_REQUETE_SERVICE) - 1 - NbOctetTotal, 0, NULL, 0);
					
						if (NbOctet < 1){ 
							STRDUP_PRINTF(Equip->vs_banniere,"[TcpWrapped] %s : Port %d - %s",Protocol,(-retourLecture), strerror(-retourLecture));
							tailleBanniere = strlen(Equip->vs_banniere) + 1;
							break;
						}
						DEBUG_INFO("Socket Broken Pipe mais on a des données à recuperer (%s)",Protocol);
					
						buffer[NbOctet] = 0; 
					
						i = 0;
						while ((i<NbOctet) && (buffer[i]!=0)){
							if (((buffer[i]<32) || (buffer[i]>126)) && (buffer[i]!='\r') && (buffer[i]!='\n'))  
							buffer[i] = '_';
						i++;
						}
						if (i!=NbOctet){
							NbOctet = i; 
							DEBUG_WARNING("Caractère final %d %d",NbOctet,i);
						}
					
						NbOctetTotal = NbOctetTotal + NbOctet;
						Nouveau_Octet_Recu (NbOctet);
					
						STRDUP_PRINTF(Equip->vs_banniere,"%s : %s",debut_entete,buffer);
						tailleBanniere = strlen(Equip->vs_banniere) + 1;
						break;
			default :
						STRDUP_PRINTF(Equip->vs_banniere,"[Error] %s  %d - %s",Protocol,(-retourLecture), strerror(-retourLecture));
						tailleBanniere = strlen(Equip->vs_banniere) + 1;
		}
	}	
	tailleReelFinal = strlen (Equip->vs_banniere);
	DEBUG_INFO( "########### Bannière (%d:%d)(Err:%d) : [%s]",tailleBanniere,tailleReelFinal,retourLecture,Equip->vs_banniere);
	if (tailleBanniere != ( tailleReelFinal + 1))
		DEBUG_ERROR("Erreur TailleBanniere (%d)!=(%d)",strlen(Equip->vs_banniere)+1,tailleBanniere);
	CLOSE_SOCKET(Equip->IdSocket);
}


void Recupere_Banniere_FTP (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_FTP  "FTP : "
	gchar   buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"FTP",PORT_FTP))
		return;
	strcpy (buffer, "Bad command\n");
	if (!send_service(Equip,buffer,strlen(buffer),"FTP",TRUE))
		return;
	read_service(Equip,BANNIERE_FTP,"\r\n", "FTP",TRUE);
}


void Recupere_Banniere_HTTP (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_HTTP  "HTTP : "
	gchar  buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"HTTP",PORT_HTTP))
		return;
	sprintf (buffer, "GET / HTTP/1.1\r\nHost: %s\r\n\r\n", Equip->Adresse_Ip_Char);
	if (!send_service(Equip,buffer,strlen(buffer),"HTTP",TRUE))
		return;
	read_service(Equip,BANNIERE_HTTP,"\r\n\r\n", "HTTP",FALSE);
}


void Recupere_Banniere_IMAP (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_IMAP  "IMAP : "
	gchar   buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"IMAP",PORT_IMAP))
		return;
	strcpy (buffer, "Bad command\n");
	if (!send_service(Equip,buffer,strlen(buffer),"IMAP",TRUE))
		return;
	read_service(Equip,BANNIERE_IMAP,"\r\n", "IMAP",TRUE);
}

void Recupere_Banniere_SMTP (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_SMTP  "SMTP : "
	gchar  buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"SMTP",PORT_SMTP))
		return;
	strcpy (buffer, "helo\n");
	if (!send_service(Equip,buffer,strlen(buffer),"SMTP",TRUE))
		return;
	read_service(Equip,BANNIERE_SMTP,"\r\n", "SMTP",TRUE);
}


void Recupere_Banniere_SSH(struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_SSH  "SSH : "
	gchar   buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"SSH",PORT_SSH))
		return;
	strcpy (buffer, "Help\n");
	if (!send_service(Equip,buffer,strlen(buffer),"SSH",TRUE))
		return;
	read_service(Equip,BANNIERE_SSH,"\n", "SSH",TRUE);
}


void Recupere_Banniere_SIP(struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_SIP  "SIP : "
	gchar   buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"SIP",PORT_SIP))
		return;
	strcpy (buffer, "OPTIONS sip:nm SIP/2.0\r\nVia: SIP/2.0/TCP nm;branch=foo\r\nFrom: <sip:nm@nm>;tag=root\r\nTo: <sip:nm2@nm2>\r\nCall-ID: 50000\r\nCSeq: 42 OPTIONS\r\nMax-Forwards: 70\r\nContent-Length: 0\r\nContact: <sip:nm@nm>\r\nAccept: application/sdp\r\n\r\n");
	if (!send_service(Equip,buffer,strlen(buffer),"SIP",TRUE))
		return;
	read_service(Equip,BANNIERE_SIP,"\r\n\r\n", "SIP",FALSE);
}


void Recupere_Banniere_POP3(struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_POP3  "POP3 : "
	gchar   buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"POP3",PORT_POP3))
		return;
	strcpy (buffer, "Help\n");
	if (!send_service(Equip,buffer,strlen(buffer),"POP3",TRUE))
		return;
	read_service(Equip,BANNIERE_POP3,"\r\n", "POP3",TRUE);
}

void Recupere_Banniere_VNC0(struct Struct_Liste_Equipement *Equip){
	Recupere_Banniere_VNC(Equip,PORT_VNC0);
}

void Recupere_Banniere_VNC1(struct Struct_Liste_Equipement *Equip){
	Recupere_Banniere_VNC(Equip,PORT_VNC1);
}

void Recupere_Banniere_VNC(struct Struct_Liste_Equipement *Equip,int port)
{
#define   BANNIERE_VNC  "VNC : "
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"VNC",port))
		return;
	read_service(Equip,BANNIERE_VNC,"\r\n", "VNC",FALSE);
}




void Recupere_Banniere_RTSP(struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_RTSP  "RTSP : "
	gchar   buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"RTSP",PORT_RTSP))
		return;
	strcpy (buffer, "OPTIONS / RTSP/1.0\r\n\r\n");
	if (!send_service(Equip,buffer,strlen(buffer),"RTSP",TRUE))
		return;
	read_service(Equip,BANNIERE_RTSP,"\r\n\r\n", "RTSP",TRUE);
}


void Recupere_Banniere_IPP(struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_IPP  "IPP : "
	gchar   buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"IPP",PORT_IPP))
		return;
	sprintf (buffer, "GET / HTTP/1.1\r\nHost: %s\r\n\r\n", Equip->Adresse_Ip_Char);
	if (!send_service(Equip,buffer,strlen(buffer),"IPP",TRUE))
		return;
	read_service(Equip,BANNIERE_IPP,"\r\n\r\n", "IPP",TRUE);
}


void Recupere_Banniere_VNC_HTTP(struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_VNC_HTTP  "VNC_HTTP : "
	gchar   buffer[TAILLE_MAX_REQUETE_SERVICE];
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT);	
	if (!connect_service(Equip,"VNC_HTTP",PORT_VNC_HTTP))
		return;
	sprintf (buffer, "GET / HTTP/1.1\r\nHost: %s\r\n\r\n", Equip->Adresse_Ip_Char);
	if (!send_service(Equip,buffer,strlen(buffer),"VNC_HTTP",TRUE))
		return;
	read_service(Equip,BANNIERE_VNC_HTTP,"\r\n\r\n", "VNC_HTTP",TRUE);
}



struct Struct_Option_Telnet opt_telnet[] = {
 	{TELNET_OPT_BINARY					,"8-bit data path"												,FALSE									,FALSE	,0},
	{TELNET_OPT_ECHO  						,"echo"																,FALSE	,TRUE		,TELNET_CMD_DO},
	{TELNET_OPT_RCP 						,"prepare to reconnect"									,FALSE									,FALSE	,0},
	{TELNET_OPT_SGA 							,"suppress go ahead"										,FALSE									,TRUE		,TELNET_CMD_DO},
	{TELNET_OPT_NAMS  					,"approximate message size"							,FALSE									,FALSE	,0},
	{TELNET_OPT_STATUS 					,"give status"													,TRUE 	,FALSE	,0},
	{TELNET_OPT_TM 							,"timing mark"												,TRUE	,FALSE	,0},
	{TELNET_OPT_RCTE 						,"remote controlled transmission and echo"	,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOL						,"negotiate about output line width"				,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOP						,"negotiate about output page size"					,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOCRD 					,"negotiate about CR disposition"					,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOHTS 					,"negotiate about horizontal tabstops"				,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOHTD					,"nego. about horizontal tab disposition"			,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOFFD 					,"negotiate about formfeed disposition"			,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOVTS 					,"negotiate about vertical tab stops"				,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOVTD					,"nego. about vertical tab disposition"				,FALSE									,FALSE	,0},
	{TELNET_OPT_NAOLFD 					,"negotiate about output LF disposition"			,FALSE									,FALSE	,0},
	{TELNET_OPT_XASCII 					,"extended ascii character set"						,FALSE									,FALSE	,0},
	{TELNET_OPT_LOGOUT 					,"force logout"													,FALSE									,FALSE	,0},
	{TELNET_OPT_BM 							,"byte macro"													,FALSE									,FALSE	,0},
	{TELNET_OPT_DET							,"data entry terminal"										,FALSE									,FALSE	,0},
	{TELNET_OPT_SUPDUP 					,"supdup protocol"											,FALSE									,FALSE	,0},
	{TELNET_OPT_SUPDUPOUTPUT  	,"supdup output"												,FALSE									,FALSE	,0},
	{TELNET_OPT_SNDLOC					,"send location"												,FALSE									,FALSE	,0},
	{TELNET_OPT_TTYPE 						,"terminal type"												,TRUE  ,FALSE	,0},
	{TELNET_OPT_EOR							,"end or record"												,FALSE									,FALSE	,0},
	{TELNET_OPT_TUID 						,"TACACS user identification"							,FALSE									,FALSE	,0},
	{TELNET_OPT_OUTMRK					,"output marking"											,FALSE									,FALSE	,0},
	{TELNET_OPT_TTYLOC 					,"terminal location number"							,FALSE									,FALSE	,0},
	{TELNET_OPT_3270REGIME  			,"3270 regime"												,FALSE									,FALSE	,0},
	{TELNET_OPT_X3PAD						,"X.3 PAD"														,FALSE									,FALSE	,0},
	{TELNET_OPT_NAWS  					,"window size"													,TRUE	,FALSE	,0},
	{TELNET_OPT_TSPEED 					,"terminal speed"												,TRUE	,FALSE	,0},
	{TELNET_OPT_LFLOW 					,"remote flow control"										,TRUE	,FALSE	,0},
	{TELNET_OPT_LINEMODE  				,"Linemode option"											,TRUE	,FALSE	,0},
	{TELNET_OPT_XDISPLOC 				,"X Display Location"										,TRUE  ,FALSE	,0},
	{TELNET_OPT_OLD_ENVIRON 		,"Old - Environment variables"							,TRUE	,FALSE	,0},
	{TELNET_OPT_AUTHENTICATION  	,"Authenticate"												,TRUE	,FALSE	,0},
	{TELNET_OPT_ENCRYPT 				,"Encryption option"										,TRUE	,FALSE	,0},
	{TELNET_OPT_NEW_ENVIRON 		,"New - Environment variables"						,TRUE	,FALSE	,0},
	{TELNET_OPT_TN3270E  				,"TN3270E"														,FALSE									,FALSE	,0}
};

gchar *info_option(gint opt){
	if ((opt<TELNET_OPT_MAX) && (opt>=0))
		return opt_telnet[opt].info;
	else
		return "Unknown_opt";
}

void log_negociation_telnet(gchar *type,unsigned char cmd,unsigned char opt ){
	 switch(cmd) {
		 case TELNET_CMD_WILL : 
		 		DEBUG_INFO("[%s] WILL %s",type,info_option((gint)opt));
		 		break;
		 case TELNET_CMD_WONT : 
		 		DEBUG_INFO("[%s] WONT %s",type,info_option((gint)opt));
		 		break;
		 case TELNET_CMD_DO : 
		 		DEBUG_INFO("[%s] DO %s",type,info_option((gint)opt));
		 		break;
	   	 case TELNET_CMD_DONT : 
		 		DEBUG_INFO("[%s] DONT %s",type,info_option((gint)opt));
		 		break;
	   	 default : 
		 		DEBUG_INFO("[%s] UNKNOWN cmd(%d) opt(%d)",type,cmd,opt);
		 		break;
	 }
}

void send_negociation_telnet(struct Struct_Liste_Equipement *Equipement,unsigned char cmd,unsigned char opt){
	gchar negociation[3];
	
	log_negociation_telnet("Send", cmd, opt);
	negociation[0] = TELNET_IAC;
  	negociation[1] = cmd;
  	negociation[2] = opt;
	send_service(Equipement,negociation, 3,"TELNET",FALSE);
}



void Recupere_Banniere_TELNET (struct Struct_Liste_Equipement *Equipement)
{
#define   BANNIERE_TELNET  "TELNET : "
	gchar	   buffer[TAILLE_REQUETE_SERVICE];
	unsigned char cmd = '\0', opt = '\0';
	int Nb_Error_Consecutive = 0,retourLecture = 0,Tentative = 0,NbOctet = 0,tailleBanniere = 0,NbOctetTotal = 0,i = 0,j = 0,iopt = 0,Time_Out_Service = 0;
	
	usleep (SURCHARGE_EQUIPEMENT);	
	ERROR_if (Equipement == NULL);
	ERROR_if (Equipement->vs_banniere != NULL);
	if (!connect_service(Equipement,"TELNET",PORT_TELNET))
		return;

	Nb_Error_Consecutive = 0; retourLecture = 0; NbOctet = 0; tailleBanniere = 0; NbOctetTotal = 0, Tentative = 0;

	STRDUP(Equipement->vs_banniere,BANNIERE_TELNET); 
	tailleBanniere = strlen(Equipement->vs_banniere) + 1;  

	for (i=0;i<TELNET_OPT_MAX;i++)
		if (opt_telnet[i].init)
			send_negociation_telnet(Equipement,opt_telnet[i].cmd_init,i);	

	Time_Out_Service = TIMEOUT_SOCKET_READY_R_FIRST;

	while ( (Tentative!=50) && (Nb_Error_Consecutive<3) && ((retourLecture = Waiting_Read_Socket (Equipement->IdSocket, Time_Out_Service)) >0)) {	
	
		Time_Out_Service = TIMEOUT_SOCKET_READY_READ;
	
		NbOctet = recvfrom (Equipement->IdSocket, &buffer, ((sizeof(gchar)*TAILLE_REQUETE_SERVICE) - 1), 0, NULL, 0);
	
		if (NbOctet < 1){ 
			DEBUG_WARNING("NbOctet == %d   %s  Socket (%d) Erreur Consécutive (%d)",NbOctet, strerror (errno),Equipement->IdSocket,Nb_Error_Consecutive);
			if (NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					if (close_control (Equipement->IdSocket) < 0)
						DEBUG_ERROR("%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));
					MYFREE(Equipement->vs_banniere); 
					STRDUP(Equipement->vs_banniere,"[Error] TELNET :  NbOctet == -1");
					return;
				}
				sleep(1);
			}
			else
				sleep(1);
			Nb_Error_Consecutive++;
		}
		else {
		
			buffer[NbOctet] = 0; 
		
			NbOctetTotal = NbOctetTotal + NbOctet;
			Nouveau_Octet_Recu (NbOctet);
		
			if ((buffer[0] & 0xff) == 0xff) {
			
				i = 0;
				while ((i < (NbOctet / 3)) && ((buffer[(i * 3)] & 0xff) == 0xff)) {
					cmd = buffer[(i * 3)+1];
					opt = buffer[(i * 3) + 2] ;
					log_negociation_telnet("Read",cmd,opt);
				
					iopt = (int) opt;
					if ((iopt>=0) && (iopt<TELNET_OPT_MAX)) 
						if  (opt_telnet[opt].reponse){ 
							switch (cmd){
								case TELNET_CMD_WILL :
									send_negociation_telnet(Equipement,TELNET_CMD_DONT,opt);
									break;
								case TELNET_CMD_DO :
									send_negociation_telnet(Equipement,TELNET_CMD_WONT,opt);
									break;
								default :
									DEBUG_INFO("Aucune reponse necessaire");
							}
						}
					i++;
				}				
				DEBUG_INFO("(%s)[%d]Taille retour negociation : [%d]  total:[%d]", Equipement->Adresse_Ip_Char,Tentative,(i * 3), NbOctet);
			
				if ((i * 3) < NbOctet) {
					DEBUG_INFO("(%s)Des donnees après la negociation  taille(%d)", Equipement->Adresse_Ip_Char, (NbOctet - (i * 3)));
				
					for (j = (i * 3); j < NbOctet; j++)
						if ((buffer[j] != 10) && (buffer[j]  != 13) && ((buffer[j]  < 31) || (buffer[j] > 126)))
							buffer[j] = '$';
				
					buffer[NbOctet] = 0;
					tailleBanniere = tailleBanniere + (NbOctet - (i * 3));
					REALLOC(Equipement->vs_banniere,(sizeof (gchar) * (tailleBanniere)));
					strcat (Equipement->vs_banniere, (buffer + (i * 3)));
					DEBUG_INFO("(%s) %s [%d]  TailleBanniere[%d] ", Equipement->Adresse_Ip_Char, Equipement->vs_banniere, strlen (Equipement->vs_banniere), tailleBanniere);
				}
			}
			else {
			
				for (j = 0; j < NbOctet; j++)
						if ((buffer[j] != 10) && (buffer[j]  != 13) && ((buffer[j]  < 31) || (buffer[j] > 126)))
							buffer[j] = '$';
			
				buffer[NbOctet] = 0;
				tailleBanniere = tailleBanniere + NbOctet;
				REALLOC(Equipement->vs_banniere,(sizeof (gchar) * (tailleBanniere)));
				strcat (Equipement->vs_banniere, buffer);
				DEBUG_INFO("Standard - (%s) %s [%d] TailleBanniere[%d] ", Equipement->Adresse_Ip_Char, Equipement->vs_banniere, strlen (Equipement->vs_banniere),tailleBanniere);
			}
			Nb_Error_Consecutive = 0;
		}
		Tentative++;
		retourLecture = 0;
	}
	DEBUG_INFO("Bannière recuperer : %s\n",Equipement->vs_banniere);

	if ((retourLecture <0)&&(tailleBanniere == (strlen(BANNIERE_TELNET) + 1))){
		MYFREE(Equipement->vs_banniere);
		switch (retourLecture){
			case SOCK_BROKEN_PIPE :
						STRDUP_PRINTF(Equipement->vs_banniere,"[TcpWrapped] TELNET %d - %s",(-retourLecture), strerror(-retourLecture));
						return;
			default:
						STRDUP_PRINTF(Equipement->vs_banniere,"[Error] TELNET  %d - %s",(-retourLecture), strerror(-retourLecture));
						return;
		}
	}

	if (tailleBanniere != (int)(strlen (Equipement->vs_banniere) + 1))
		DEBUG_ERROR( "Erreur TailleBanniere (%d)!=(%d)",tailleBanniere,(strlen (Equipement->vs_banniere) + 1));
	if (close_control (Equipement->IdSocket) < 0)
		DEBUG_ERROR( "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));
}
