/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include <time.h>
#include "Scan_Signature.h"
#include "Debug.h"
#include "Info.h"
#include "Liste_Equipement.h"
#include "Initialisation_Com.h"
#include "Mem.h"
#ifdef	__WINDOWS_ENABLE
#include "Win32.h"
#endif
#include <pthread.h>

char      Syn_Normal[] = 		{ 0x45, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x06, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x02, 0x1F, 0xFE, 0x00, 0x00, 0x00, 0x00 };
char      Syn_Null[] =   		{ 0x45, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x06, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x00, 0x1F, 0xFE, 0x00, 0x00, 0x00, 0x00 };
char      Syn_ECN[] =    		{ 0x45, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x06, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x42, 0x1F, 0xFE, 0x00, 0x00, 0x00, 0x00 };
char      Syn_FIN_URG_PSH[] = 	{ 0x45, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x06, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x2b, 0x1F, 0xFE, 0x00, 0x00, 0x00, 0x00 };
char      Ack[] = 				{ 0x45, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x06, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x10, 0x1F, 0xFE, 0x00, 0x00, 0x00, 0x00 };
char      Fin[] = 				{ 0x45, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x06, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x01, 0x1F, 0xFE, 0x00, 0x00, 0x00, 0x00 };

char      Udp_30[] = 			{ 0x45, 0x00, 0x00, 0x3A, 0x00, 0x00, 0x00, 0x00, 0x4C, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x26, 0x00, 0x00, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee };
char      Icmp_Timestamp[] = 	{ 0x45, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x4C, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x8b, 0x11, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
char      Ping_31[] = 			{ 0x45, 0x00, 0x00, 0x3B, 0x00, 0x01, 0x00, 0x00, 0x4C, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x3e, 0x01, 0xF0, 0xDB, 0x44, 0x6A, 0x7C, 0x9F, 0x0C, 0x8D, 0x1F, 0x5d, 0xE3, 0x55, 0x4F, 0x59, 0xA4, 0x15, 0xC3, 0xEF, 0xC7, 0xB1, 0xFF, 0x1c, 0x63, 0x28, 0x88, 0x73, 0x90, 0xD3, 0xBB };

	
struct Struct_Liste_Type_Trame Liste_Type_Trame[] = {
	{TRAME_SYN_NORMAL_OUVERT			, "TRAME_SYN_NORMAL_OUVERT"},
	{TRAME_SYN_NULL_OUVERT					, "TRAME_SYN_NULL_OUVERT"},
	{TRAME_SYN_ECN_OUVERT					, "TRAME_SYN_ECN_OUVERT"},
	{TRAME_SYN_FIN_URG_PSH_OUVERT		, "TRAME_SYN_FIN_URG_PSH_OUVERT"},
	{TRAME_ACK_OUVERT							, "TRAME_ACK_OUVERT"},
	{TRAME_FIN_OUVERT							, "TRAME_FIN_OUVERT"},
	{TRAME_SYN_NORMAL_FERMER			, "TRAME_SYN_NORMAL_FERMER"},
	{TRAME_SYN_NULL_FERMER					, "TRAME_SYN_NULL_FERMER"},
	{TRAME_SYN_ECN_FERMER					, "TRAME_SYN_ECN_FERMER"},
	{TRAME_SYN_FIN_URG_PSH_FERMER	, "TRAME_SYN_FIN_URG_PSH_FERMER"},
	{TRAME_ACK_FERMER							, "TRAME_ACK_FERMER"},
	{TRAME_FIN_FERMER							, "TRAME_FIN_FERMER"},
	{TRAME_PING_31									, "TRAME_PING_31"},
	{TRAME_ICMP_TIMESTAMP					, "TRAME_ICMP_TIMESTAMP"},
	{TRAME_UDP_30									, "TRAME_UDP_30"}
};	

	
#define AUCUNE_REPONSE_TCP -2
extern void ckecksum_r (struct Struct_Liste_Equipement *Equip,unsigned char *donnee, int longueur, int valeur);
		
gboolean Traitement_ScanOS_ICMP (struct Struct_Liste_Equipement *Equipement, struct struct_trame *Packet, unsigned char Taille_Trame, unsigned short Taille_Ip, unsigned char Type_ICMP, unsigned char Code_ICMP)
{
	unsigned short Icmp_Id = 0, Icmp_Seq = 0, Port_Destination = 0, Port_Source = 0, Id_Ip = 0;
	gint      Id_Reponse_Trame = 0, Tentative = 0;
#ifdef __LOG_ENABLE
	gchar  *Texte = NULL;
#endif
	
	ERROR_if (Equipement == NULL);
	ERROR_if (Packet == NULL);

	if (Sem_Get_Boolean(&Equipement->Scan_Signature)) {
		Id_Reponse_Trame = 0;
		if ((Type_ICMP == ECHO_PING_REPLY) && (Code_ICMP == ICMP_CODE_0)) {
			memcpy (&Icmp_Id, &Packet->Datagramme[Taille_Ip + 4], 2);
			Icmp_Id = htons (Icmp_Id);
			memcpy (&Icmp_Seq, &Packet->Datagramme[Taille_Ip + 6], 2);
			Icmp_Seq = htons (Icmp_Seq);
			PROTECT_LISTE_SCAN_OS();	
		
			Tentative = 0;
			while ((Tentative<NB_TENTATIVE_MAX_SCAN_SIGNATURE) && (Equipement->Scan_OS[TRAME_PING_31].Id[Tentative]  != Icmp_Id) 
					&& (Equipement->Scan_OS[TRAME_PING_31].Seq[Tentative]  != Icmp_Seq)){
					Tentative++;
			}	
			if ((Equipement->Scan_OS[TRAME_PING_31].Id[Tentative]  == Icmp_Id) 
					&& (Equipement->Scan_OS[TRAME_PING_31].Seq[Tentative]  == Icmp_Seq)) {
				Id_Reponse_Trame = TRAME_PING_31;
			}
			DEPROTECT_LISTE_SCAN_OS();
		}
		if ((Type_ICMP == DESTINATION_UNREACHABLE) && (Code_ICMP == PORT_UNREACHABLE)) {
			memcpy (&Id_Ip, &Packet->Datagramme[Taille_Ip + 12], 2);
			Id_Ip = htons (Id_Ip);
			memcpy (&Port_Source, &Packet->Datagramme[Taille_Ip + 28], 2);
			Port_Source = htons (Port_Source);
			memcpy (&Port_Destination, &Packet->Datagramme[Taille_Ip + 30], 2);
			Port_Destination = htons (Port_Destination);
			PROTECT_LISTE_SCAN_OS();
		
			Tentative = 0;
		
			while ((Tentative<NB_TENTATIVE_MAX_SCAN_SIGNATURE) && (Equipement->Scan_OS[TRAME_UDP_30].Id[Tentative] != Id_Ip) 
				&& (Equipement->Scan_OS[TRAME_UDP_30].Port_Scan[Tentative]  != Port_Destination)
				&& (Equipement->Scan_OS[TRAME_UDP_30].Port_Client[Tentative]  != Port_Source)){
					Tentative++;
			}		
			if ((Equipement->Scan_OS[TRAME_UDP_30].Id[Tentative] == Id_Ip) 
				&& (Equipement->Scan_OS[TRAME_UDP_30].Port_Scan[Tentative]  == Port_Destination) 
			   && (Equipement->Scan_OS[TRAME_UDP_30].Port_Client[Tentative]  == Port_Source)) {
				Id_Reponse_Trame = TRAME_UDP_30;
			}
			DEPROTECT_LISTE_SCAN_OS();
		}
		if ((Type_ICMP == TIMESTAMP_REPLY) && (Code_ICMP == ICMP_CODE_0)) {
			memcpy (&Icmp_Id, &Packet->Datagramme[Taille_Ip + 4], 2);
			Icmp_Id = htons (Icmp_Id);
			memcpy (&Icmp_Seq, &Packet->Datagramme[Taille_Ip + 6], 2);
			Icmp_Seq = htons (Icmp_Seq);
			PROTECT_LISTE_SCAN_OS();
		
			Tentative = 0;
			while ((Tentative<NB_TENTATIVE_MAX_SCAN_SIGNATURE) && (Equipement->Scan_OS[TRAME_ICMP_TIMESTAMP].Id[Tentative]  != Icmp_Id) 
						&& (Equipement->Scan_OS[TRAME_ICMP_TIMESTAMP].Seq[Tentative]  != Icmp_Seq)){
					Tentative++;
			}			
			if ((Equipement->Scan_OS[TRAME_ICMP_TIMESTAMP].Id[Tentative]  == Icmp_Id) 
				&& (Equipement->Scan_OS[TRAME_ICMP_TIMESTAMP].Seq[Tentative]  == Icmp_Seq)) {
				Id_Reponse_Trame = TRAME_ICMP_TIMESTAMP;
			}
			DEPROTECT_LISTE_SCAN_OS();
		}
		if (Id_Reponse_Trame != 0) {
#ifdef __LOG_ENABLE
			ERROR_if (Id_Reponse_Trame >= NB_TYPE_TRAME);
			STRDUP_PRINTF(Texte,"[LOG] [%s](%s:%d):   --%s--(%d)  Tentative(%d)\n",__FILE__, __PRETTY_FUNCTION__, __LINE__,Liste_Type_Trame[Id_Reponse_Trame].Info,Id_Reponse_Trame,Tentative);
			Log_Equipement(Equipement,Texte);
			MYFREE(Texte);
#endif
			PROTECT_LISTE_SCAN_OS();
			Equipement->Scan_OS[Id_Reponse_Trame].Reponse_Scan = TRUE;
			DEPROTECT_LISTE_SCAN_OS();
		
			Equipement->Reponse_Signature_Debug[Id_Reponse_Trame].Time_Fin = time(NULL);
			Equipement->Reponse_Signature_Debug[Id_Reponse_Trame].Temps_Reponse  = (gint) difftime(Equipement->Reponse_Signature_Debug[Id_Reponse_Trame].Time_Fin , Equipement->Reponse_Signature_Debug[Id_Reponse_Trame].Time_Debut);
		
			Equipement->Reponse_Signature[Id_Reponse_Trame].TaillePacketReponse = Taille_Trame;
			Equipement->Reponse_Signature[Id_Reponse_Trame].HeaderLengthIP = Taille_Ip;
			Equipement->Reponse_Signature[Id_Reponse_Trame].DSF = Packet->Datagramme[1];
			memcpy (&Equipement->Reponse_Signature[Id_Reponse_Trame].TailleTotal, &Packet->Datagramme[2], 2);
			Equipement->Reponse_Signature[Id_Reponse_Trame].TailleTotal = htons (Equipement->Reponse_Signature[Id_Reponse_Trame].TailleTotal);
			memcpy (&Id_Ip, &Packet->Datagramme[4], 2);
			Id_Ip = htons (Id_Ip);
			if (Id_Ip == 0)
				Equipement->Reponse_Signature[Id_Reponse_Trame].Id = 0;
			else
				Equipement->Reponse_Signature[Id_Reponse_Trame].Id = 1;
			memcpy (&Equipement->Reponse_Signature[Id_Reponse_Trame].Flags_IP_Offset, &Packet->Datagramme[6], 2);
			Equipement->Reponse_Signature[Id_Reponse_Trame].Flags_IP_Offset = htons (Equipement->Reponse_Signature[Id_Reponse_Trame].Flags_IP_Offset);
			Equipement->Reponse_Signature[Id_Reponse_Trame].TTL = Packet->Datagramme[8];
			Nouveau_Octet_Recu (Taille_Trame);
			return TRUE;
		}
	}
	return FALSE;
}

gboolean Traitement_ScanOS_TCP (struct Struct_Liste_Equipement * Equipement, struct struct_trame * Packet, unsigned char Taille_Trame, unsigned short Taille_Ip, gint Port_Source, gint Port_Destination)
{
	gint      i = 0, Tentative = 0;
	unsigned short Id_Ip = 0;
	unsigned int NumAckEnCours = 0;
#ifdef __LOG_ENABLE
	gchar  *Texte = NULL;
#endif
	
	ERROR_if (Equipement == NULL);
	ERROR_if (Packet == NULL);
	PROTECT_LISTE_SCAN_OS();

	if (Sem_Get_Boolean(&Equipement->Scan_Signature)) {
		for (i = 0; i < NB_SCAN_OS; i++) {
		
			Tentative = 0;
			while ((Tentative<NB_TENTATIVE_MAX_SCAN_SIGNATURE) 
							&& (Equipement->Scan_OS[i].Port_Scan[Tentative] != Port_Source) 		
							&& (Equipement->Scan_OS[i].Port_Client[Tentative] != Port_Destination)){
					Tentative++;
			}		
			if ((Equipement->Scan_OS[i].Port_Scan[Tentative] == Port_Source) && (Equipement->Scan_OS[i].Port_Client[Tentative] == Port_Destination)) {
				if (Equipement->Scan_OS[i].Reponse_Scan){
					DEBUG_WARNING("Reponse deja recuperer Port[%d][%d]\n",Port_Source,Port_Destination);
					continue;
				}
#ifdef __LOG_ENABLE
				ERROR_if (i >= NB_TYPE_TRAME);
				STRDUP_PRINTF(Texte,"[LOG] [%s](%s:%d):   --%s---(%d)   Tentative(%d)\n",__FILE__, __PRETTY_FUNCTION__, __LINE__,Liste_Type_Trame[i].Info,i,Tentative);
				Log_Equipement(Equipement,Texte);
				MYFREE(Texte);
#endif
				Equipement->Scan_OS[i].Reponse_Scan = TRUE;
				if (Equipement->Reponse_Signature_Debug[i].Num_Port_Utiliser == AUCUNE_REPONSE_TCP)
					DEBUG_ERROR("AUCUNE_REPONSE_TCP attendu");
			
				Equipement->Reponse_Signature_Debug[i].Time_Fin = time(NULL);
				Equipement->Reponse_Signature_Debug[i].Temps_Reponse  = (gint) difftime(Equipement->Reponse_Signature_Debug[i].Time_Fin , Equipement->Reponse_Signature_Debug[i].Time_Debut);
			
				Equipement->Reponse_Signature[i].TaillePacketReponse = Taille_Trame;
			
				Equipement->Reponse_Signature[i].HeaderLengthIP = Taille_Ip;
				Equipement->Reponse_Signature[i].DSF = Packet->Datagramme[1];
				memcpy (&Equipement->Reponse_Signature[i].TailleTotal, &Packet->Datagramme[2], 2);
				Equipement->Reponse_Signature[i].TailleTotal = htons (Equipement->Reponse_Signature[i].TailleTotal);
				memcpy (&Id_Ip, &Packet->Datagramme[4], 2);
				Id_Ip = htons (Id_Ip);
				if (Id_Ip == 0)
					Equipement->Reponse_Signature[i].Id = 0;
				else
					Equipement->Reponse_Signature[i].Id = 1;
				memcpy (&Equipement->Reponse_Signature[i].Flags_IP_Offset, &Packet->Datagramme[6], 2);
				Equipement->Reponse_Signature[i].Flags_IP_Offset = htons (Equipement->Reponse_Signature[i].Flags_IP_Offset);
				Equipement->Reponse_Signature[i].TTL = Packet->Datagramme[8];
			
				memcpy (&NumAckEnCours, &Packet->Datagramme[Taille_Ip + 4], 4);
				if (NumAckEnCours == 0)
					Equipement->Reponse_Signature[i].NumSeq = 0;
				else
					Equipement->Reponse_Signature[i].NumSeq = 1;
				Equipement->Reponse_Signature[i].HeaderLengthTCP = Packet->Datagramme[Taille_Ip + 12];
				Equipement->Reponse_Signature[i].Flags_TCP = Packet->Datagramme[Taille_Ip + 13];
				memcpy (&Equipement->Reponse_Signature[i].WindowSize, &Packet->Datagramme[Taille_Ip + 14], 2);
				Equipement->Reponse_Signature[i].WindowSize = htons (Equipement->Reponse_Signature[i].WindowSize);
				Nouveau_Octet_Recu (Taille_Trame);
				DEPROTECT_LISTE_SCAN_OS();
				return TRUE;
			}
		}
		DEPROTECT_LISTE_SCAN_OS();
		return TRUE; 
	}
	DEPROTECT_LISTE_SCAN_OS();
	return FALSE;
}


#ifndef		__PCAP_ENABLE
void Envoyer_Donnee_UDP_Detect_Os (struct Struct_Liste_Equipement *Equip, gint Num_Port, gint Socket, struct sockaddr *sock,int tentative)
#else
void Envoyer_Donnee_UDP_Detect_Os (struct Struct_Liste_Equipement *Equip, gint Num_Port, pcap_t * captureHandle,int tentative)
#endif
{
	struct struct_trame PacketOs;
	unsigned short Id_Ip = 0,Port = 0;
	
	ERROR_if (Equip == NULL);
	usleep (SURCHARGE_EQUIPEMENT/10);	
	if (tentative==0){
		Equip->Reponse_Signature_Debug[TRAME_UDP_30].Num_Port_Utiliser = Num_Port ;
		Equip->Reponse_Signature_Debug[TRAME_UDP_30].Time_Debut = time(NULL);
	}	
	
	PROTECT_LISTE_SCAN_OS();
	if (!(Equip->Scan_OS[TRAME_UDP_30].Reponse_Scan)) {
		bzero (&PacketOs, sizeof (struct struct_trame));
		Id_Ip = (unsigned short) rand_r(&graine_rand_r);
		Port = ((unsigned short) rand_r(&graine_rand_r));
		Port = (Port & 0x0FFF) + 0xF000;
		Equip->Scan_OS[TRAME_UDP_30].Id[tentative] = Id_Ip;
		Equip->Scan_OS[TRAME_UDP_30].Port_Scan[tentative] = Num_Port;
		Equip->Scan_OS[TRAME_UDP_30].Port_Client[tentative] = Port;
		memcpy (&PacketOs.Dest_MAC, &Equip->Adresse_MAC, ETH_ALEN);
		memcpy (&PacketOs.Source_MAC, Mon_Adresse_MAC, ETH_ALEN);
		PacketOs.Proto = htons (TRAME_IP);
		memcpy (&PacketOs.Datagramme, Udp_30, 58);
		PacketOs.Datagramme[4] = (Id_Ip >> 8) & 0xFF;
		PacketOs.Datagramme[5] = Id_Ip & 0xFF;
		memcpy (&PacketOs.Datagramme[12], &Mon_Adresse_Ip, IP_ADDR_LEN);
		memcpy (&PacketOs.Datagramme[16], &Equip->Adresse_Ip, IP_ADDR_LEN);
		ckecksum_r(Equip,&PacketOs.Datagramme[0], 20, 0);
		Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
		memcpy (&PacketOs.Datagramme[10], &Equip->CheckSum_Calc, 2);
		PacketOs.Datagramme[20] = (Port >> 8) & 0xFF;
		PacketOs.Datagramme[21] = Port & 0xFF;
		PacketOs.Datagramme[22] = (Num_Port >> 8) & 0xFF;
		PacketOs.Datagramme[23] = Num_Port & 0xFF;
		ckecksum_r(Equip,&PacketOs.Datagramme[12], 46, 38 + 17);
		Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
		memcpy (&PacketOs.Datagramme[26], &Equip->CheckSum_Calc, 2);
#ifndef   __PCAP_ENABLE
			_SENDTO(Socket, &PacketOs, 72, 0, sock, sizeof (struct sockaddr));
#else
			SENDTO_PCAP(captureHandle,(u_char *) &PacketOs, 72);
 #endif
	}
	DEPROTECT_LISTE_SCAN_OS();
}

#ifndef		__PCAP_ENABLE
void Envoyer_Donnee_Ping_Detect_Os (struct Struct_Liste_Equipement *Equip, gint Socket, struct sockaddr *sock,int tentative)
#else
void Envoyer_Donnee_Ping_Detect_Os (struct Struct_Liste_Equipement *Equip, pcap_t * captureHandle ,int tentative)
#endif
{
	struct struct_trame PacketOs;
	unsigned short Icmp_Id = 0, Icmp_Seq = 0;
	
	ERROR_if (Equip == NULL);
#ifndef		__PCAP_ENABLE
	ERROR_if (sock == NULL);
#else
	ERROR_if (captureHandle == NULL);
#endif
	usleep (SURCHARGE_EQUIPEMENT/10);	
	if (tentative==0){
		Equip->Reponse_Signature_Debug[TRAME_PING_31].Num_Port_Utiliser = -1;
		Equip->Reponse_Signature_Debug[TRAME_PING_31].Time_Debut = time(NULL);
	}	
	
	PROTECT_LISTE_SCAN_OS();
	if (!(Equip->Scan_OS[TRAME_PING_31].Reponse_Scan)) {
		bzero (&PacketOs, sizeof (struct struct_trame));
		Icmp_Id = (unsigned short) rand_r(&graine_rand_r);
		Icmp_Seq = 1;
		Equip->Scan_OS[TRAME_PING_31].Id[tentative] = Icmp_Id;
		Equip->Scan_OS[TRAME_PING_31].Seq[tentative] = Icmp_Seq;
		memcpy (&PacketOs.Dest_MAC, &Equip->Adresse_MAC, ETH_ALEN);
		memcpy (&PacketOs.Source_MAC, Mon_Adresse_MAC, ETH_ALEN);
		PacketOs.Proto = htons (TRAME_IP);
		memcpy (&PacketOs.Datagramme, Ping_31, 59);
		memcpy (&PacketOs.Datagramme[12], &Mon_Adresse_Ip, IP_ADDR_LEN);
		memcpy (&PacketOs.Datagramme[16], &Equip->Adresse_Ip, IP_ADDR_LEN);
		ckecksum_r(Equip,&PacketOs.Datagramme[0], 20, 0);
		Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
		memcpy (&PacketOs.Datagramme[10], &Equip->CheckSum_Calc, 2);
		PacketOs.Datagramme[24] = (Icmp_Id >> 8) & 0xFF;
		PacketOs.Datagramme[25] = Icmp_Id & 0xFF;
		PacketOs.Datagramme[26] = (Icmp_Seq >> 8) & 0xFF;
		PacketOs.Datagramme[27] = Icmp_Seq & 0xFF;
		ckecksum_r(Equip,&PacketOs.Datagramme[20], 39, 0);
		Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
		memcpy (&PacketOs.Datagramme[22], &Equip->CheckSum_Calc, 2);
#ifndef   __PCAP_ENABLE
			_SENDTO(Socket, &PacketOs, 73, 0, sock, sizeof (struct sockaddr));
#else
			SENDTO_PCAP(captureHandle,(u_char *) &PacketOs, 73);
 #endif
	}
	DEPROTECT_LISTE_SCAN_OS();
}


#ifndef		__PCAP_ENABLE
void Envoyer_Donnee_ICMP_TimeStamp_Detect_Os (struct Struct_Liste_Equipement *Equip, gint Socket, struct sockaddr *sock,int tentative)
#else
void Envoyer_Donnee_ICMP_TimeStamp_Detect_Os (struct Struct_Liste_Equipement *Equip, pcap_t * captureHandle,int tentative)
#endif
{
	struct struct_trame PacketOs;
	unsigned short Icmp_Id = 0, Icmp_Seq = 0;
	unsigned short Id_Ip = 0;
	
	ERROR_if (Equip == NULL);
#ifndef		__PCAP_ENABLE
	ERROR_if (sock == NULL);
#else
	ERROR_if (captureHandle == NULL);
#endif
	usleep (SURCHARGE_EQUIPEMENT/10);	
	if (tentative==0){
		Equip->Reponse_Signature_Debug[TRAME_ICMP_TIMESTAMP].Num_Port_Utiliser = -1;
		Equip->Reponse_Signature_Debug[TRAME_ICMP_TIMESTAMP].Time_Debut = time(NULL);
	}	
	
	PROTECT_LISTE_SCAN_OS();
	if (!(Equip->Scan_OS[TRAME_ICMP_TIMESTAMP].Reponse_Scan)) {
		bzero (&PacketOs, sizeof (struct struct_trame));
		Id_Ip = (unsigned short) rand_r(&graine_rand_r);
		Icmp_Id = (unsigned short) rand_r(&graine_rand_r);
		Icmp_Seq = (unsigned short) rand_r(&graine_rand_r);
		Equip->Scan_OS[TRAME_ICMP_TIMESTAMP].Id[tentative] = Icmp_Id;
		Equip->Scan_OS[TRAME_ICMP_TIMESTAMP].Seq[tentative] = Icmp_Seq;
		memcpy (&PacketOs.Dest_MAC, &Equip->Adresse_MAC, ETH_ALEN);
		memcpy (&PacketOs.Source_MAC, Mon_Adresse_MAC, ETH_ALEN);
		PacketOs.Proto = htons (TRAME_IP);
		memcpy (&PacketOs.Datagramme, Icmp_Timestamp, 40);
		PacketOs.Datagramme[4] = (Id_Ip >> 8) & 0xFF;
		PacketOs.Datagramme[5] = Id_Ip & 0xFF;
		memcpy (&PacketOs.Datagramme[12], &Mon_Adresse_Ip, IP_ADDR_LEN);
		memcpy (&PacketOs.Datagramme[16], &Equip->Adresse_Ip, IP_ADDR_LEN);
		ckecksum_r(Equip,&PacketOs.Datagramme[0], 20, 0);
		Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
		memcpy (&PacketOs.Datagramme[10], &Equip->CheckSum_Calc, 2);
		PacketOs.Datagramme[24] = (Icmp_Id >> 8) & 0xFF;
		PacketOs.Datagramme[25] = Icmp_Id & 0xFF;
		PacketOs.Datagramme[26] = (Icmp_Seq >> 8) & 0xFF;
		PacketOs.Datagramme[27] = Icmp_Seq & 0xFF;
		ckecksum_r(Equip,&PacketOs.Datagramme[20], 20, 0);
		Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
		memcpy (&PacketOs.Datagramme[22], &Equip->CheckSum_Calc, 2);
#ifndef   __PCAP_ENABLE
			_SENDTO(Socket, &PacketOs, 54, 0, sock, sizeof (struct sockaddr));
#else
			SENDTO_PCAP(captureHandle,(u_char *) &PacketOs, 54);
 #endif
	}
	DEPROTECT_LISTE_SCAN_OS();
}


#ifndef		__PCAP_ENABLE
void Envoyer_Donnee_TCP_Detect_Os (struct Struct_Liste_Equipement *Equip, gint Num_Port, gint Socket, struct sockaddr *sock, gint Num_Trame,int tentative)
#else
void Envoyer_Donnee_TCP_Detect_Os (struct Struct_Liste_Equipement *Equip, gint Num_Port,  pcap_t * captureHandle, gint Num_Trame,int tentative)
#endif
{
	struct struct_trame PacketOs;
	char     *Trame = NULL;
	gint     size_trame = 0;
	unsigned short Id_Ip = 0,Port = 0;
	unsigned int MonNumSeq = 0;
	
	ERROR_if (Equip == NULL);
#ifndef		__PCAP_ENABLE
	ERROR_if (sock == NULL);
#else
	ERROR_if (captureHandle == NULL);
#endif
	usleep (SURCHARGE_EQUIPEMENT/10);	
	if (tentative==0){
		Equip->Reponse_Signature_Debug[Num_Trame].Num_Port_Utiliser = Num_Port;
		Equip->Reponse_Signature_Debug[Num_Trame].Time_Debut = time(NULL);
	}	
	
	PROTECT_LISTE_SCAN_OS();
	if (!(Equip->Scan_OS[Num_Trame].Reponse_Scan)) {
		bzero (&PacketOs, sizeof (struct struct_trame));
		Id_Ip = (unsigned short) rand_r(&graine_rand_r);
		Port = ((unsigned short) rand_r(&graine_rand_r));
		Port = (Port & 0x0FFF) + 0xF000;
		Equip->Scan_OS[Num_Trame].Port_Scan[tentative] = Num_Port;
		Equip->Scan_OS[Num_Trame].Port_Client[tentative]= Port;
		DEPROTECT_LISTE_SCAN_OS();
		switch (Num_Trame) {
		    case TRAME_SYN_NORMAL_OUVERT:
		    case TRAME_SYN_NORMAL_FERMER:
			    Trame = Syn_Normal;
			    break;
		    case TRAME_SYN_NULL_OUVERT:
		    case TRAME_SYN_NULL_FERMER:
			    Trame= Syn_Null;
			    break;
		    case TRAME_SYN_ECN_OUVERT:
		    case TRAME_SYN_ECN_FERMER:
			    Trame = Syn_ECN;
			    break;
		    case TRAME_SYN_FIN_URG_PSH_OUVERT:
		    case TRAME_SYN_FIN_URG_PSH_FERMER:
			    Trame = Syn_FIN_URG_PSH;
			    break;
		    case TRAME_ACK_OUVERT:
		    case TRAME_ACK_FERMER:
			    Trame = Ack;
			    break;
		    case TRAME_FIN_OUVERT:
		    case TRAME_FIN_FERMER:
			    Trame = Fin;
			    break;
		    default:
			   DEBUG_ERROR( "Num trame %d", Num_Trame);
		}
		MonNumSeq = htonl ((int) rand_r(&graine_rand_r));
		memcpy (&PacketOs.Dest_MAC, &Equip->Adresse_MAC, ETH_ALEN);
		memcpy (&PacketOs.Source_MAC, Mon_Adresse_MAC, ETH_ALEN);
		PacketOs.Proto = htons (TRAME_IP);
		memcpy (&PacketOs.Datagramme, Trame, 40);
		if (Equip->mss){
			PacketOs.Datagramme[3] = 0x2C;
			PacketOs.Datagramme[32] = 0x60;
		
			PacketOs.Datagramme[40] = 0x02;
			PacketOs.Datagramme[41] = 0x04;
			PacketOs.Datagramme[42] = 0x05;
			PacketOs.Datagramme[43] = 0xb4;			
		}			
		PacketOs.Datagramme[4] = (Id_Ip >> 8) & 0xFF;
		PacketOs.Datagramme[5] = Id_Ip & 0xFF;
		memcpy (&PacketOs.Datagramme[12], &Mon_Adresse_Ip, IP_ADDR_LEN);
		memcpy (&PacketOs.Datagramme[16], &Equip->Adresse_Ip, IP_ADDR_LEN);
		ckecksum_r(Equip,&PacketOs.Datagramme[0], 20, 0);
		Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
		memcpy (&PacketOs.Datagramme[10], &Equip->CheckSum_Calc, 2);
		PacketOs.Datagramme[20] = (Port >> 8) & 0xFF;
		PacketOs.Datagramme[21] = Port & 0xFF;
		PacketOs.Datagramme[22] = (Num_Port >> 8) & 0xFF;
		PacketOs.Datagramme[23] = Num_Port & 0xFF;
		memcpy (&PacketOs.Datagramme[24], &MonNumSeq, 4);
		if (!Equip->mss)
			ckecksum_r(Equip,&PacketOs.Datagramme[12], 24, 26);
		else
			ckecksum_r(Equip,&PacketOs.Datagramme[12], 32, 30);
		Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
		memcpy (&PacketOs.Datagramme[36], &Equip->CheckSum_Calc, 2);
		if (!Equip->mss)
			size_trame = 54;			
		else
			size_trame = 58;
#ifndef   __PCAP_ENABLE
			_SENDTO(Socket, &PacketOs, size_trame, 0, sock, sizeof (struct sockaddr));
#else
			SENDTO_PCAP(captureHandle,(u_char *) &PacketOs, size_trame);
#endif
	}
	else
		DEPROTECT_LISTE_SCAN_OS();
}

gboolean Test_Retour_Scan_Equipement (struct Struct_Liste_Equipement *Equip)
{
	int i =0;
	
	ERROR_if (Equip == NULL);
	PROTECT_LISTE_SCAN_OS();
	for (i = 0; i < NB_SCAN_OS; i++)
		if (!Equip->Scan_OS[i].Reponse_Scan){
			DEPROTECT_LISTE_SCAN_OS();
			return FALSE;
		}
	DEPROTECT_LISTE_SCAN_OS();
	return TRUE;
}

void Envoie_Trames_Scanne_Os (struct Struct_Liste_Equipement *Equip, gint Port_Ouvert, gint Port_Fermer)
{
	int Note = 0, i =0, Tentative_UDP = 0, Tentative =0, j = 0;
#ifdef  __PCAP_ENABLE
	pcap_t    * captureHandle;
	char errbuf[PCAP_ERRBUF_SIZE];
#endif

	ERROR_if (Equip == NULL);
	Efface_Reponse_Signature (Equip);

	PROTECT_LISTE_SCAN_OS();
	PROTECT_CODE (Semaphore_Boolean);	
	Equip->Scan_Signature = TRUE;
	for (i = 0; i < NB_SCAN_OS; i++){
		Equip->Scan_OS[i].Reponse_Scan = FALSE;
		Equip->Reponse_Signature_Debug[i].Num_Port_Utiliser = AUCUNE_REPONSE_TCP;
		Equip->Reponse_Signature_Debug[i].Time_Debut = AUCUNE_REPONSE_TCP;
		Equip->Reponse_Signature_Debug[i].Time_Fin = AUCUNE_REPONSE_TCP;
		Equip->Reponse_Signature_Debug[i].Temps_Reponse = AUCUNE_REPONSE_TCP;
		for (j = 0; j < NB_TENTATIVE_MAX_SCAN_SIGNATURE; j++){ 
			Equip->Scan_OS[i].Port_Scan[j] = 0;
			Equip->Scan_OS[i].Port_Client[j] = 0;
			Equip->Scan_OS[i].Id[j] = 0;
			Equip->Scan_OS[i].Seq[j] = 0;
		}
	}	
	if (Port_Ouvert==-1) 
		for (i = 0; i < 6; i++)
			Equip->Scan_OS[i].Reponse_Scan = TRUE;
	if (Port_Fermer==-1) 
		for (i = 6; i < 12; i++)
			Equip->Scan_OS[i].Reponse_Scan = TRUE;
	DEPROTECT_CODE (Semaphore_Boolean);
	DEPROTECT_LISTE_SCAN_OS();
#ifndef  __PCAP_ENABLE		
	struct sockaddr sock;
	int       Socket;
	OPEN_SOCKET(Socket,sock,Info_Demarrage,AF_INET,SOCK_PACKET,htons (ETH_P_ALL));
#else
	OPEN_SOCKET_PCAP(captureHandle,Info_Demarrage,errbuf) ;
#endif
#ifndef  __PCAP_ENABLE
	Envoyer_Donnee_TCP_Detect_Passerelle_Routeur(Equip,Socket, &sock);
#else
	Envoyer_Donnee_TCP_Detect_Passerelle_Routeur(Equip, captureHandle);
#endif
	Tentative = 0;	
	Tentative_UDP = 65530;
	do {
		DEBUG_INFO("Tentative %d",Tentative);
#ifndef  __PCAP_ENABLE
		if (Port_Ouvert != -1) 
			for (i = 0; i < 6; i++)
				Envoyer_Donnee_TCP_Detect_Os (Equip, Port_Ouvert, Socket, &sock, i,Tentative);
		if (Port_Fermer != -1) 
			for (i = 6; i < 12; i++)
				Envoyer_Donnee_TCP_Detect_Os (Equip, Port_Fermer, Socket, &sock, i, Tentative);
		Envoyer_Donnee_ICMP_TimeStamp_Detect_Os (Equip, Socket, &sock,Tentative);
		Envoyer_Donnee_Ping_Detect_Os (Equip, Socket, &sock,Tentative);
		Envoyer_Donnee_UDP_Detect_Os (Equip, Tentative_UDP, Socket, &sock,Tentative);
#else
		if (Port_Ouvert!=-1) 
			for (i = 0; i < 6; i++)
				Envoyer_Donnee_TCP_Detect_Os (Equip, Port_Ouvert, captureHandle, i,Tentative);
		if (Port_Fermer != -1) 
			for (i = 6; i < 12; i++)
				Envoyer_Donnee_TCP_Detect_Os (Equip, Port_Fermer, captureHandle, i,Tentative);
		Envoyer_Donnee_ICMP_TimeStamp_Detect_Os (Equip, captureHandle,Tentative);
		Envoyer_Donnee_Ping_Detect_Os (Equip, captureHandle,Tentative);
		Envoyer_Donnee_UDP_Detect_Os (Equip, Tentative_UDP, captureHandle,Tentative);
#endif
		Tentative_UDP++;
		Retour_Info_Scan(30 + (Tentative * 4) ,"  Fingerprinting  ",Equip);
		switch (Tentative) {
		    case 0:
			    sleep (1);
			    break;
		    case 1:
			    sleep (1);
			    break;
		    case 2:
			    sleep (2);
			    break;
		    case 3:
			    sleep (3);
			    break;
		    case 4:
			    sleep (4);
			    break;
		    case 5:
			    DEBUG_INFO("Attend reponse");
			    sleep (4);
			    break;
		}
		Tentative++; 
	}
	while (!(Test_Retour_Scan_Equipement(Equip)) && (Tentative != NB_TENTATIVE_MAX_SCAN_SIGNATURE));

	Note = 0;
	for (i = 0; i < NB_SCAN_OS; i++){
			if (Equip->Reponse_Signature_Debug[i].Num_Port_Utiliser != AUCUNE_REPONSE_TCP) {
				if (Equip->Reponse_Signature_Debug[i].Temps_Reponse==AUCUNE_REPONSE_TCP) {
					Equip->Reponse_Signature_Debug[i].Time_Fin = time(NULL);
					Equip->Reponse_Signature_Debug[i].Temps_Reponse  = (gint) difftime(Equip->Reponse_Signature_Debug[i].Time_Fin , Equip->Reponse_Signature_Debug[i].Time_Debut);
				}
				else 
					Note++;
			}
	}
	DEBUG_INFO("Scanne OS fini   Note(%d/%d)",Note,NB_SCAN_OS);
#ifndef  __PCAP_ENABLE
	CLOSE_SOCKET(Socket); 
#else
	CLOSE_SOCKET_PCAP(captureHandle);
#endif
	DEBUG_INFO("Syn o  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[0].TaillePacketReponse, Equip->Reponse_Signature[0].HeaderLengthIP, Equip->Reponse_Signature[0].DSF, Equip->Reponse_Signature[0].TailleTotal, Equip->Reponse_Signature[0].Id, Equip->Reponse_Signature[0].Flags_IP_Offset, Equip->Reponse_Signature[0].TTL, Equip->Reponse_Signature[0].NumSeq, Equip->Reponse_Signature[0].HeaderLengthTCP, Equip->Reponse_Signature[0].Flags_TCP,Equip->Reponse_Signature[0].WindowSize,Equip->Reponse_Signature_Debug[0].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[0].Temps_Reponse);
	DEBUG_INFO ("Nul o  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[1].TaillePacketReponse, Equip->Reponse_Signature[1].HeaderLengthIP, Equip->Reponse_Signature[1].DSF, Equip->Reponse_Signature[1].TailleTotal, Equip->Reponse_Signature[1].Id, Equip->Reponse_Signature[1].Flags_IP_Offset, Equip->Reponse_Signature[1].TTL, Equip->Reponse_Signature[1].NumSeq, Equip->Reponse_Signature[1].HeaderLengthTCP, Equip->Reponse_Signature[1].Flags_TCP, Equip->Reponse_Signature[1].WindowSize,Equip->Reponse_Signature_Debug[1].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[1].Temps_Reponse);
	DEBUG_INFO("Ecn o  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[2].TaillePacketReponse, Equip->Reponse_Signature[2].HeaderLengthIP, Equip->Reponse_Signature[2].DSF, Equip->Reponse_Signature[2].TailleTotal, Equip->Reponse_Signature[2].Id, Equip->Reponse_Signature[2].Flags_IP_Offset, Equip->Reponse_Signature[2].TTL, Equip->Reponse_Signature[2].NumSeq, Equip->Reponse_Signature[2].HeaderLengthTCP, Equip->Reponse_Signature[2].Flags_TCP, Equip->Reponse_Signature[2].WindowSize,Equip->Reponse_Signature_Debug[2].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[2].Temps_Reponse);
	DEBUG_INFO ("FUP o  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[3].TaillePacketReponse, Equip->Reponse_Signature[3].HeaderLengthIP, Equip->Reponse_Signature[3].DSF, Equip->Reponse_Signature[3].TailleTotal, Equip->Reponse_Signature[3].Id, Equip->Reponse_Signature[3].Flags_IP_Offset, Equip->Reponse_Signature[3].TTL, Equip->Reponse_Signature[3].NumSeq, Equip->Reponse_Signature[3].HeaderLengthTCP, Equip->Reponse_Signature[3].Flags_TCP, Equip->Reponse_Signature[3].WindowSize,Equip->Reponse_Signature_Debug[3].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[3].Temps_Reponse);
	DEBUG_INFO ("Ack o  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[4].TaillePacketReponse, Equip->Reponse_Signature[4].HeaderLengthIP, Equip->Reponse_Signature[4].DSF, Equip->Reponse_Signature[4].TailleTotal, Equip->Reponse_Signature[4].Id, Equip->Reponse_Signature[4].Flags_IP_Offset, Equip->Reponse_Signature[4].TTL, Equip->Reponse_Signature[4].NumSeq, Equip->Reponse_Signature[4].HeaderLengthTCP, Equip->Reponse_Signature[4].Flags_TCP, Equip->Reponse_Signature[4].WindowSize,Equip->Reponse_Signature_Debug[4].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[4].Temps_Reponse);
	DEBUG_INFO ("Fin o  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[5].TaillePacketReponse, Equip->Reponse_Signature[5].HeaderLengthIP, Equip->Reponse_Signature[5].DSF, Equip->Reponse_Signature[5].TailleTotal, Equip->Reponse_Signature[5].Id, Equip->Reponse_Signature[5].Flags_IP_Offset, Equip->Reponse_Signature[5].TTL, Equip->Reponse_Signature[5].NumSeq, Equip->Reponse_Signature[5].HeaderLengthTCP, Equip->Reponse_Signature[5].Flags_TCP, Equip->Reponse_Signature[5].WindowSize,Equip->Reponse_Signature_Debug[5].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[5].Temps_Reponse);
	DEBUG_INFO ("Syn f  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[6].TaillePacketReponse, Equip->Reponse_Signature[6].HeaderLengthIP, Equip->Reponse_Signature[6].DSF, Equip->Reponse_Signature[6].TailleTotal, Equip->Reponse_Signature[6].Id, Equip->Reponse_Signature[6].Flags_IP_Offset, Equip->Reponse_Signature[6].TTL, Equip->Reponse_Signature[6].NumSeq, Equip->Reponse_Signature[6].HeaderLengthTCP, Equip->Reponse_Signature[6].Flags_TCP, Equip->Reponse_Signature[6].WindowSize,Equip->Reponse_Signature_Debug[6].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[6].Temps_Reponse);
	DEBUG_INFO("Nul f  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[7].TaillePacketReponse, Equip->Reponse_Signature[7].HeaderLengthIP, Equip->Reponse_Signature[7].DSF, Equip->Reponse_Signature[7].TailleTotal, Equip->Reponse_Signature[7].Id, Equip->Reponse_Signature[7].Flags_IP_Offset, Equip->Reponse_Signature[7].TTL, Equip->Reponse_Signature[7].NumSeq, Equip->Reponse_Signature[7].HeaderLengthTCP, Equip->Reponse_Signature[7].Flags_TCP, Equip->Reponse_Signature[7].WindowSize,Equip->Reponse_Signature_Debug[7].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[7].Temps_Reponse);
	DEBUG_INFO ("Ecn f  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[8].TaillePacketReponse, Equip->Reponse_Signature[8].HeaderLengthIP, Equip->Reponse_Signature[8].DSF, Equip->Reponse_Signature[8].TailleTotal, Equip->Reponse_Signature[8].Id, Equip->Reponse_Signature[8].Flags_IP_Offset, Equip->Reponse_Signature[8].TTL, Equip->Reponse_Signature[8].NumSeq, Equip->Reponse_Signature[8].HeaderLengthTCP, Equip->Reponse_Signature[8].Flags_TCP, Equip->Reponse_Signature[8].WindowSize,Equip->Reponse_Signature_Debug[8].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[8].Temps_Reponse);
	DEBUG_INFO ("FUP f  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[9].TaillePacketReponse, Equip->Reponse_Signature[9].HeaderLengthIP, Equip->Reponse_Signature[9].DSF, Equip->Reponse_Signature[9].TailleTotal, Equip->Reponse_Signature[9].Id, Equip->Reponse_Signature[9].Flags_IP_Offset, Equip->Reponse_Signature[9].TTL, Equip->Reponse_Signature[9].NumSeq, Equip->Reponse_Signature[9].HeaderLengthTCP, Equip->Reponse_Signature[9].Flags_TCP, Equip->Reponse_Signature[9].WindowSize,Equip->Reponse_Signature_Debug[9].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[9].Temps_Reponse);
	DEBUG_INFO ("Ack f  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[10].TaillePacketReponse, Equip->Reponse_Signature[10].HeaderLengthIP, Equip->Reponse_Signature[10].DSF, Equip->Reponse_Signature[10].TailleTotal, Equip->Reponse_Signature[10].Id, Equip->Reponse_Signature[10].Flags_IP_Offset, Equip->Reponse_Signature[10].TTL, Equip->Reponse_Signature[10].NumSeq, Equip->Reponse_Signature[10].HeaderLengthTCP, Equip->Reponse_Signature[10].Flags_TCP, Equip->Reponse_Signature[10].WindowSize,Equip->Reponse_Signature_Debug[10].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[10].Temps_Reponse);
	DEBUG_INFO ("Fin f  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%i][%d]", Equip->Reponse_Signature[11].TaillePacketReponse, Equip->Reponse_Signature[11].HeaderLengthIP, Equip->Reponse_Signature[11].DSF, Equip->Reponse_Signature[11].TailleTotal, Equip->Reponse_Signature[11].Id, Equip->Reponse_Signature[11].Flags_IP_Offset, Equip->Reponse_Signature[11].TTL, Equip->Reponse_Signature[11].NumSeq, Equip->Reponse_Signature[11].HeaderLengthTCP, Equip->Reponse_Signature[11].Flags_TCP, Equip->Reponse_Signature[11].WindowSize,Equip->Reponse_Signature_Debug[11].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[11].Temps_Reponse);
	DEBUG_INFO ("Ping   %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%d][%d]", Equip->Reponse_Signature[12].TaillePacketReponse, Equip->Reponse_Signature[12].HeaderLengthIP, Equip->Reponse_Signature[12].DSF, Equip->Reponse_Signature[12].TailleTotal, Equip->Reponse_Signature[12].Id, Equip->Reponse_Signature[12].Flags_IP_Offset, Equip->Reponse_Signature[12].TTL, Equip->Reponse_Signature[12].NumSeq, Equip->Reponse_Signature[12].HeaderLengthTCP, Equip->Reponse_Signature[12].Flags_TCP, Equip->Reponse_Signature[12].WindowSize,Equip->Reponse_Signature_Debug[12].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[12].Temps_Reponse);
	DEBUG_INFO ("IcmpT  %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%d][%d]", Equip->Reponse_Signature[13].TaillePacketReponse, Equip->Reponse_Signature[13].HeaderLengthIP, Equip->Reponse_Signature[13].DSF, Equip->Reponse_Signature[13].TailleTotal, Equip->Reponse_Signature[13].Id, Equip->Reponse_Signature[13].Flags_IP_Offset, Equip->Reponse_Signature[13].TTL, Equip->Reponse_Signature[13].NumSeq, Equip->Reponse_Signature[13].HeaderLengthTCP, Equip->Reponse_Signature[13].Flags_TCP, Equip->Reponse_Signature[13].WindowSize,Equip->Reponse_Signature_Debug[13].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[13].Temps_Reponse);
	DEBUG_INFO ("UDP    %02X |%02X %02X %02X %01X %04X %02X %01X %02X %02X %04X|  [%d][%u]", Equip->Reponse_Signature[14].TaillePacketReponse, Equip->Reponse_Signature[14].HeaderLengthIP, Equip->Reponse_Signature[14].DSF, Equip->Reponse_Signature[14].TailleTotal, Equip->Reponse_Signature[14].Id, Equip->Reponse_Signature[14].Flags_IP_Offset, Equip->Reponse_Signature[14].TTL, Equip->Reponse_Signature[14].NumSeq, Equip->Reponse_Signature[14].HeaderLengthTCP, Equip->Reponse_Signature[14].Flags_TCP, Equip->Reponse_Signature[14].WindowSize,Equip->Reponse_Signature_Debug[14].Num_Port_Utiliser,Equip->Reponse_Signature_Debug[14].Temps_Reponse);
}




#ifndef  __PCAP_ENABLE
void Envoyer_Donnee_TCP_Detect_Passerelle_Routeur (struct Struct_Liste_Equipement *Equip,gint Socket, struct sockaddr *sock)
#else
void Envoyer_Donnee_TCP_Detect_Passerelle_Routeur (struct Struct_Liste_Equipement *Equip,pcap_t * captureHandle)
#endif
{
	struct struct_trame PacketOs;
	unsigned short Id_Ip = 0;
	unsigned int MonNumSeq = 0;
		
	ERROR_if (Equip == NULL);
#ifndef  __PCAP_ENABLE
	ERROR_if (sock == NULL);
#else
	ERROR_if (captureHandle == NULL);
#endif
	usleep (SURCHARGE_EQUIPEMENT/10);	
	bzero (&PacketOs, sizeof (struct struct_trame));
	Id_Ip = (unsigned short) rand_r(&graine_rand_r);
	PROTECT_CODE (Semaphore_Int);
	Equip->Port_Source_Test_GateWay = ((unsigned short) rand_r(&graine_rand_r));
	Equip->Port_Source_Test_GateWay = (Equip->Port_Source_Test_GateWay & 0x0FFF) + 0xF000;
	Equip->Port_Destination_Test_GateWay = ((unsigned short) rand_r(&graine_rand_r));
	Equip->Port_Destination_Test_GateWay = (Equip->Port_Destination_Test_GateWay & 0x0FFF) + 0xF000;
	DEPROTECT_CODE (Semaphore_Int);
	DEBUG_INFO("Envoie Trame Detect passerelle Routeur (%s)    port (%d)(%d)", Equip->Adresse_Ip_Char, Equip->Port_Source_Test_GateWay, Equip->Port_Destination_Test_GateWay);
	MonNumSeq = htonl ((int) rand_r(&graine_rand_r));
	memcpy (&PacketOs.Dest_MAC, &Equip->Adresse_MAC, ETH_ALEN);
	memcpy (&PacketOs.Source_MAC, Mon_Adresse_MAC, ETH_ALEN);
	PacketOs.Proto = htons (TRAME_IP);
	memcpy (&PacketOs.Datagramme, Syn_Normal, 40);
	PacketOs.Datagramme[4] = (Id_Ip >> 8) & 0xFF;
	PacketOs.Datagramme[5] = Id_Ip & 0xFF;
	memcpy (&PacketOs.Datagramme[12], &Mon_Adresse_Ip, IP_ADDR_LEN);
	memcpy (&PacketOs.Datagramme[16], &Mon_Adresse_Ip, IP_ADDR_LEN);
	ckecksum_r(Equip,&PacketOs.Datagramme[0], 20, 0);
	Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
	memcpy (&PacketOs.Datagramme[10], &Equip->CheckSum_Calc, 2);
	PacketOs.Datagramme[20] = (Equip->Port_Source_Test_GateWay >> 8) & 0xFF;
	PacketOs.Datagramme[21] = Equip->Port_Source_Test_GateWay & 0xFF;
	PacketOs.Datagramme[22] = (Equip->Port_Destination_Test_GateWay >> 8) & 0xFF;
	PacketOs.Datagramme[23] = Equip->Port_Destination_Test_GateWay & 0xFF;
	memcpy (&PacketOs.Datagramme[24], &MonNumSeq, 4);
	ckecksum_r(Equip,&PacketOs.Datagramme[12], 24, 26);
	Equip->CheckSum_Calc = htons (Equip->CheckSum_Calc);
	memcpy (&PacketOs.Datagramme[36], &Equip->CheckSum_Calc, 2);
#ifndef   __PCAP_ENABLE
	_SENDTO(Socket, &PacketOs, 54, 0, sock, sizeof (struct sockaddr));
#else
	SENDTO_PCAP(captureHandle,(u_char *) &PacketOs, 54);
#endif
}
