/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Declaration_Constante.h"
#include "Semaphore.h"
#include "Initialisation_Com.h"
#include "Debug.h"

#ifndef         __WINDOWS_ENABLE
#include <errno.h>


#ifdef   __MUTEX_ENABLE 
static int sem_init(sem_t *sem, int pshared, int value) {
    int ret;

    sem->value = value;
    ret = pthread_mutex_init(&sem->mutex,NULL);
    if(ret < 0) return -1;
    ret = pthread_cond_init(&sem->cond,NULL);
    if(ret < 0) return -1;
    return 0;
}

static int sem_post(sem_t *sem) {
    if(pthread_mutex_lock(&sem->mutex) < 0)
        return -1;
    sem->value++;
    if(pthread_cond_signal(&sem->cond) < 0) {
        pthread_mutex_unlock(&sem->mutex);
        return -1;
    }
    if(pthread_mutex_unlock(&sem->mutex) < 0)
        return -1;
    return 0;
}

static int sem_wait(sem_t *sem) {
    if(pthread_mutex_lock(&sem->mutex) < 0)
        return -1;
    while(sem->value == 0) {
        pthread_cond_wait(&sem->cond,&sem->mutex);
    }
    sem->value--;
    if(pthread_mutex_unlock(&sem->mutex) < 0)
        return -1;    
    return 0;
}

static int sem_destroy(sem_t *sem) {
    int ret;
    ret = pthread_cond_destroy(&sem->cond);
    if(ret < 0) return -1;
    ret = pthread_mutex_destroy(&sem->mutex);
    if(ret < 0) return -1;
    return 0;
}

static void sem_getvalue(sem_t *sem,int *value) {
	(*value) = sem->value;
}
#endif

void Attendre_Semaphore (sem_t * sem)
{
	ERROR_if (sem == NULL);
	if (sem_wait (sem) == -1) {
		switch (errno) {
		    case EINTR:
			   DEBUG_ERROR("EINTR : (A signal interrupted)  - %s", strerror (errno));
		    case EINVAL:
			    DEBUG_ERROR( "EINVAL : (Does not refer to a valid semaphore)  - %s", strerror (errno));
		    case EAGAIN:
			    DEBUG_ERROR("EAGAIN  - %s", strerror (errno));
		    default:
			    DEBUG_ERROR("%s", strerror (errno));
			    break;
		}
	}
}

void Liberation_Semaphore (sem_t * sem)
{
	ERROR_if (sem == NULL);
	if (sem_post (sem) == -1) {
		switch (errno) {
		    case EINVAL:
			    DEBUG_ERROR("EINVAL : (Does not refer to a valid semaphore)  - %s", strerror (errno));
		    default:
			    DEBUG_ERROR("%s", strerror (errno));
			    break;
		}
	}
}

int Valeur_Semaphore (sem_t * sem)
{
	int       valeur;
	
	ERROR_if (sem == NULL);
	sem_getvalue (sem, &valeur);
	return valeur;
}

void Initialisation_Semaphore_Gestion_Thread (void)
{
	sem_init (&Semaphore_Nouveau_Equipement, 0, 0);
	sem_init (&Sem_GList_Liste_Thread_Scan, 0, Info_Demarrage.Nb_Thread_Max);
}

void Initialisation_Semaphore (void)
{
	sem_init (&Semaphore_Info, 0, 1);
	sem_init (&Semaphore_Control_Usage_Socket, 0, 1);
	sem_init (&Sem_GList_Notification_Equipement, 0, 1);
	sem_init (&Semaphore_Debug, 0, 1);
	sem_init (&Sem_GList_Notification_Port, 0, 1);
	sem_init (&Sem_GList_Equipement, 0, 1);
	sem_init (&Semaphore_Snmp, 0, 1);
	sem_init (&Semaphore_Communication, 0, 1);
	sem_init (&Semaphore_DNS, 0, 1);
	sem_init (&Semaphore_WakeUp, 0, 1);
	sem_init (&Semaphore_Mem, 0, 1); 
	sem_init (&Semaphore_Ping_Equipement, 0, 1); 
	sem_init (&Semaphore_Etat_Thread, 0, 1); 
	sem_init (&Sem_GList_Liste_Thread, 0, 1); 
	sem_init (&Semaphore_Scan_Arp_Sous_Reseau, 0, 2); 
	sem_init (&Semaphore_Fonction_Externe, 0, 1);
	sem_init (&Semaphore_BackTrace, 0, 1);
	sem_init (&Semaphore_Log_File, 0, 1);
	sem_init (&Sem_GList_Scan_Arp_Detect_Nouveau_Equipement, 0, 1);
	sem_init (&Semaphore_Niveau_Arret, 0, 1);
	sem_init (&Semaphore_Boolean, 0, 1);
	sem_init (&Semaphore_Int, 0, 1);
	sem_init (&Sem_Liste_Etat_Udp, 0, 1);
	sem_init (&Sem_GList_Port, 0, 1);
	sem_init (&Sem_Liste_Scan_OS, 0, 1);
	sem_init (&Semaphore_Pointer, 0, 1);
	sem_init (&Sem_Struct_Port_Equipement, 0, 1);
	sem_init (&Sem_GList_Liste_Sous_Reseau, 0, 1);
}

#else


void Attendre_Semaphore (HANDLE sem)
{ 
	if (WaitForSingleObject (sem, INFINITE)== WAIT_FAILED)
		DEBUG_ERROR("WaitForSingleObject  error: %ld\n", GetLastError ());
	if ((sem==Semaphore_Nouveau_Equipement) || (sem==Sem_GList_Liste_Thread_Scan)){
		
		if (WaitForSingleObject (Semaphore_Compteur, INFINITE)== WAIT_FAILED)
			DEBUG_ERROR("WaitForSingleObject  error: %ld\n", GetLastError ());
		if (sem==Semaphore_Nouveau_Equipement)
			Compteur_Nouveau_Equipement++;
		if (sem==Sem_GList_Liste_Thread_Scan)
			Compteur_Thread_Scan--;
		if (!ReleaseSemaphore (Semaphore_Compteur, 1, NULL))
			DEBUG_ERROR("ReleaseSemaphore  error: %ld\n", GetLastError ());
	}
} 

void Liberation_Semaphore (HANDLE sem)
{
	if (!ReleaseSemaphore (sem, 1, NULL))
		DEBUG_ERROR("ReleaseSemaphore  error: %ld\n", GetLastError ());
	if ((sem==Semaphore_Nouveau_Equipement) || (sem==Sem_GList_Liste_Thread_Scan)){

		if (WaitForSingleObject (Semaphore_Compteur, INFINITE)== WAIT_FAILED)
			DEBUG_ERROR("WaitForSingleObject  error: %ld\n", GetLastError ());
		if (sem==Semaphore_Nouveau_Equipement)
			Compteur_Nouveau_Equipement--;
		if (sem==Sem_GList_Liste_Thread_Scan)
			Compteur_Thread_Scan++;
		if (!ReleaseSemaphore (Semaphore_Compteur, 1, NULL))
			DEBUG_ERROR("ReleaseSemaphore  error: %ld\n", GetLastError ());
	}
}

int Valeur_Semaphore (HANDLE *sem)
{
	int       valeur;
 
	if (*sem==Semaphore_Nouveau_Equipement)
		return Compteur_Nouveau_Equipement;
	if (*sem==Sem_GList_Liste_Thread_Scan)
		return Compteur_Thread_Scan;
	return 1;
}

void Initialisation_Semaphore_Gestion_Thread (void)
{
	Compteur_Nouveau_Equipement = 0;  
	Compteur_Thread_Scan = Info_Demarrage.Nb_Thread_Max;

	if ((Semaphore_Nouveau_Equipement = CreateSemaphore (NULL, 0, 65535, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Nouveau_Equipement) error: %ld\n", GetLastError ());
	if ((Sem_GList_Liste_Thread_Scan = CreateSemaphore (NULL, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_GList_Liste_Thread_Scan) error: %ld\n", GetLastError ());

	if ((Semaphore_Compteur = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Compteur) error: %ld\n", GetLastError ());
}

void Initialisation_Semaphore (void)
{
	
	if ((Semaphore_Info = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Info) error: %ld\n", GetLastError ());
	if ((Semaphore_Control_Usage_Socket = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Control_Usage_Socket) error: %ld\n", GetLastError ());
	if ((Sem_GList_Notification_Equipement = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_GList_Notification_Equipement) error: %ld\n", GetLastError ());
	if ((Semaphore_Debug = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Debug) error: %ld\n", GetLastError ());
	if ((Sem_GList_Notification_Port = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_GList_Notification_Port) error: %ld\n", GetLastError ());
	if ((Sem_GList_Equipement = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_GList_Equipement) error: %ld\n", GetLastError ());
	if ((Semaphore_Snmp = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Snmp) error: %ld\n", GetLastError ());
	if ((Semaphore_Communication = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Communication) error: %ld\n", GetLastError ());
	if ((Semaphore_DNS = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_DNS) error: %ld\n", GetLastError ());
	if ((Semaphore_WakeUp = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_WakeUp) error: %ld\n", GetLastError ());
	if ((Semaphore_Mem = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Mem) error: %ld\n", GetLastError ());
	if ((Semaphore_Ping_Equipement = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Ping_Equipement) error: %ld\n", GetLastError ());
	if ((Semaphore_Etat_Thread = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Etat_Thread) error: %ld\n", GetLastError ());
	if ((Sem_GList_Liste_Thread = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR( "CreateSemaphore(Sem_GList_Liste_Thread) error: %ld\n", GetLastError ());
	if ((Semaphore_Scan_Arp_Sous_Reseau = CreateSemaphore (NULL, 1, 2, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Scan_Arp_Sous_Reseau) error: %ld\n", GetLastError ());
	if ((Semaphore_Fonction_Externe = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Fonction_Externe) error: %ld\n", GetLastError ());
	if ((Semaphore_BackTrace = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_BackTrace) error: %ld\n", GetLastError ());
	if ((Semaphore_Log_File = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Log_File) error: %ld\n", GetLastError ());
	if ((Sem_GList_Scan_Arp_Detect_Nouveau_Equipement = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_GList_Scan_Arp_Detect_Nouveau_Equipement) error: %ld\n", GetLastError ());
	if ((Semaphore_Niveau_Arret = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Niveau_Arret) error: %ld\n", GetLastError ());
	if ((Semaphore_Boolean = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Boolean) error: %ld\n", GetLastError ());
	if ((Semaphore_Int = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Int) error: %ld\n", GetLastError ());
	if ((Sem_Liste_Etat_Udp = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_Liste_Etat_Udp) error: %ld\n", GetLastError ());
	if ((Sem_GList_Port = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_GList_Port) error: %ld\n", GetLastError ());
	if ((Sem_Liste_Scan_OS = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_Liste_Scan_OS) error: %ld\n", GetLastError ());	
	if ((Semaphore_Pointer = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Semaphore_Pointer) error: %ld\n", GetLastError ());
	if ((Sem_Struct_Port_Equipement = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_Struct_Port_Equipement) error: %ld\n", GetLastError ());	
	if ((Sem_GList_Liste_Sous_Reseau = CreateSemaphore (NULL, 1, 1, NULL)) == NULL)
		DEBUG_ERROR("CreateSemaphore(Sem_GList_Liste_Sous_Reseau) error: %ld\n", GetLastError ());	
}
#endif


void Sem_Set_uInt(unsigned int *variable,unsigned int valeur){
	PROTECT_CODE (Semaphore_Int);
	*variable = valeur ;
	DEPROTECT_CODE (Semaphore_Int);
}
unsigned int Sem_Get_uInt(unsigned int *valeur){
	unsigned int value;
	
	PROTECT_CODE (Semaphore_Int);
	value = *valeur ;
	DEPROTECT_CODE (Semaphore_Int);
	return value;
}


void Sem_Set_ushort(unsigned short *variable,unsigned short valeur){
	PROTECT_CODE (Semaphore_Int);
	*variable = valeur ;
	DEPROTECT_CODE (Semaphore_Int);
}

unsigned short Sem_Get_ushort(unsigned short *valeur){
	unsigned short value;
	
	PROTECT_CODE (Semaphore_Int);
	value = *valeur ;
	DEPROTECT_CODE (Semaphore_Int);
	return value;
}


void Sem_Incr_Int(int *variable){
	PROTECT_CODE (Semaphore_Int);
	*variable = *variable + 1;
	DEPROTECT_CODE (Semaphore_Int);
}
void Sem_Decr_Int(int *variable){
	PROTECT_CODE (Semaphore_Int);
	*variable = *variable - 1 ;
	DEPROTECT_CODE (Semaphore_Int);
}

void Sem_Set_Int(int *variable,int valeur){
	PROTECT_CODE (Semaphore_Int);
	*variable = valeur ;
	DEPROTECT_CODE (Semaphore_Int);
}
int Sem_Get_Int(int *valeur){
	int value;
	
	PROTECT_CODE (Semaphore_Int);
	value = *valeur ;
	DEPROTECT_CODE (Semaphore_Int);
	return value;
}

void Sem_Set_Boolean(gboolean *variable,gboolean valeur){
	PROTECT_CODE (Semaphore_Boolean);
	*variable = valeur ;
	DEPROTECT_CODE (Semaphore_Boolean);
}
gboolean Sem_Get_Boolean(gboolean *valeur){
	gboolean value;
	
	PROTECT_CODE (Semaphore_Boolean);
	value = *valeur ;
	DEPROTECT_CODE (Semaphore_Boolean);
	return value;
}

void Sem_Set_Pointer(gpointer *variable,gpointer valeur){
	PROTECT_CODE (Semaphore_Pointer);
	*variable = valeur ;
	DEPROTECT_CODE (Semaphore_Pointer);
}
gpointer * Sem_Get_Pointer(gpointer *valeur){
	gpointer value;
	
	PROTECT_CODE (Semaphore_Pointer);
	value = *valeur ;
	DEPROTECT_CODE (Semaphore_Pointer);
	return value;
}
