/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Thread.h"
#include "Mem.h"
#include "Debug.h"
#include "BackTrace.h"
#include "Configuration.h"
#include "Win32.h"


#ifdef __LOG_ENABLE
extern struct Struct_Categorie Liste_Categorie[];
#endif

gint Get_Nb_Thread(){
gint value = 0;
	
	PROTECT_GLIST_LISTE_THREAD();
	value = Nb_Thread;
	DEPROTECT_GLIST_LISTE_THREAD();
	return value;
}

void Start_Thread (void *thread, gpointer * pointeur)
{
	struct Struct_Liste_Thread *Thread = NULL;

	PROTECT_GLIST_LISTE_THREAD();
	MALLOC(Thread,sizeof (struct Struct_Liste_Thread));
	if (pthread_create (&Thread->pThread, NULL, (void *) thread, pointeur) != 0) {

		DEPROTECT_GLIST_LISTE_THREAD();
		DEBUG_ERROR("Start_Thread (%d): Impossible de démarrer le thread  pid(%d)  Id_Thread(%d) NbThread(%d)", getpid (),Id_Thread + 1,Nb_Thread);
	}
	pthread_detach (Thread->pThread);
	Nb_Thread++;
	Id_Thread++;	
	Thread->Id_Thread = Id_Thread;
	G_LIST_APPEND(GList_Liste_Thread, Thread);
	DEPROTECT_GLIST_LISTE_THREAD();
}

void Stop_Thread (void *retval)
{		
	struct Struct_Liste_Thread *Thread = NULL, *Thread_Trouver = NULL;
	pthread_t Thread_Id;
	gint      i = 0, Nb_Objet_Liste = 0;

	PROTECT_GLIST_LISTE_THREAD();
	Thread_Id = pthread_self ();
	Thread_Trouver = NULL;

	Nb_Objet_Liste = g_list_length (GList_Liste_Thread);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Thread = g_list_nth_data (GList_Liste_Thread, i);
		if (pthread_equal (Thread->pThread, Thread_Id)) {
			Thread_Trouver = Thread; 
			break;
		}
	}

	if (Thread_Trouver != NULL){
		G_LIST_REMOVE(GList_Liste_Thread, Thread_Trouver);
		MYFREE(Thread_Trouver);
	}
	else
		printf ("[WARNING] Thread Non référencé\n");
#ifdef         __LOG_ENABLE
	struct struct_Cle_Privee *pCle = NULL;
	

	if ((pCle = pthread_getspecific (Cle_privee_thread)) == NULL)
		pCle = Donnee_Cle_Main;
	Log(pCle, "Stop Thread\n");

	#ifdef         __DEBUG_MEM
	Log_Info_Allocation(TRUE);	
	#endif
	Close_Log_Thread(pCle);
#endif

	Nb_Thread--;
	DEPROTECT_GLIST_LISTE_THREAD();
	pthread_exit (retval);
}


#ifdef         __SIGNAL_ENABLE
void Pause_Thread( int signum, siginfo_t *info, void *secret)
{
	PROTECT_CODE(Semaphore_Etat_Thread);
	struct struct_Cle_Privee *pCle = NULL;
	struct Struct_Liste_Equipement *Equipement=NULL;
	char   Texte[255];
		
	if ((pCle = pthread_getspecific (Cle_privee_thread)) == NULL)
		pCle = Donnee_Cle_Main;
	strcpy(Texte,"Thread [");
	strcat (Texte, Liste_Categorie[pCle->Categorie].Info);
	strcat (Texte,"] (");
	switch (pCle->Categorie) {
	    case CATEG_THREAD_SCAN:
		    Equipement = pCle->Pointeur_Data;
		    strcat (Texte, (char *)Equipement->Adresse_Ip_Char);
		    break;
	    case CATEG_THREAD_SNMP_LISTE_ALL:
		    Equipement = pCle->Pointeur_Data;
		    strcat (Texte, (char *)Equipement->Adresse_Ip_Char);
		    break;
	    case CATEG_THREAD_SNMP_LISTE:
		    Equipement = pCle->Pointeur_Data;
		    strcat (Texte, (char *)Equipement->Adresse_Ip_Char);
		    break;
		case CATEG_SCAN_PLAGE :
			strcat (Texte, pCle->Pointeur_Data);
			break;
		case CATEG_THREAD_SNMP_SCALAIRE:
		    Equipement = pCle->Pointeur_Data;
		    strcat (Texte, (char *)Equipement->Adresse_Ip_Char);
		    break;
		default:
			strcat (Texte,"none");
	}
	strcat (Texte,")\n");
	printf("%s",Texte);

	if (pCle->Categorie != CATEG_THREAD_SCAN ){
		Log (pCle, "#####################################\n");
		Log (pCle, Texte);
		display_backtrace(pCle,NULL,NULL,0);
	
		#ifdef         __DEBUG_MEM
		Log_Info_Allocation(FALSE);
		#endif
		Close_Log_Thread(pCle);
	}
	else {
		Log_Equipement(Equipement, "#####################################\n");
		Log_Equipement(Equipement, Texte);
		display_backtrace(NULL,Equipement,NULL,0);
	}
	PROTECT_GLIST_LISTE_THREAD();
	Nb_Thread--; 
	DEPROTECT_GLIST_LISTE_THREAD();
	DEPROTECT_CODE(Semaphore_Etat_Thread); 
	while (1)	pause(); 
}

void envoie_Signal_All_Thread (int signum)
{
	PROTECT_GLIST_LISTE_THREAD();
	struct Struct_Liste_Thread *Thread = NULL;
	gint      i = 0, Nb_Objet_Liste = 0;

	Nb_Objet_Liste = g_list_length (GList_Liste_Thread);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Thread = g_list_nth_data (GList_Liste_Thread, i);
		pthread_kill (Thread->pThread, signum);
	}
	DEPROTECT_GLIST_LISTE_THREAD();
}


#define STACK_SIZE	(2 * 1024 * 1024)
void Stack_Size_Thread(){
/*	size_t stacksize;
	struct rlimit rlimitbuf;

	pthread_attr_init(&Thread_Attr);
	pthread_attr_getstacksize (&Thread_Attr, &stacksize);
	DEBUG_WARNING("pthread_attr_getstacksize  = %d    (Min = %d)",stacksize,PTHREAD_STACK_MIN);
	stacksize = Config_Rlimit_Stack;
	if (pthread_attr_setstacksize (&Thread_Attr, stacksize) !=0)
		DEBUG_WARNING("pthread_attr_setstacksize  [FAILED]  %s",Config_Rlimit_Stack);
	pthread_attr_getstacksize (&Thread_Attr, &stacksize);
	DEBUG_WARNING("pthread_attr_getstacksize  = %d    (Min = %d)",stacksize,PTHREAD_STACK_MIN);
	getrlimit (RLIMIT_STACK, &rlimitbuf);
	DEBUG_WARNING("getrlimit RLIMIT_STACK current:(%d) max:(%d)\n",rlimitbuf.rlim_cur,rlimitbuf.rlim_max);*/
}
#endif
