/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Thread_Detect.h"
#include "Reseau.h"
#include "Debug.h"
#include "Sous_Reseau_Autorise.h"
#include "Liste_Equipement.h"
#include "Scan_Port_TCP.h"
#include "Scan_Equipement.h"
#include "Scan_Signature.h"
#include "Initialisation_Com.h"
#include "Semaphore.h"
#include "Info.h"
#include "Thread.h"
#include "Mem.h"
#include "Liste_Sous_Reseau.h"
#include "Thread_Ip_Dynamic.h"
#include "Scan_Netbios.h"

void Start_Scan_Plage(){
	gchar Ma_Plage[4 + 1];
	
	Scanne_Liste_Sous_Reseau_Restaurer (g_list_length (GList_Sous_Reseau));
	if (g_list_length (GList_Sous_Reseau) == 0) {
		memcpy (&Ma_Plage, &Mon_Adresse_Ip, IP_ADDR_LEN);
		Ma_Plage[3] = 0;
		if (Recherche_Index_Sous_Reseau (Ma_Plage) == -1)
			Ajout_Sous_Reseau (Ma_Plage, TRUE, TRUE);
	}
}

void Boucle_Scan (void)
{
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint  Num = 0;

#ifdef __LOG_ENABLE
	Create_Log(CATEG_BOUCLE_SCAN,NULL,NULL,TRUE);
#endif
	Num = Nb_Machine_Dans_La_Liste;
	while (1) {
#ifndef         __WINDOWS_ENABLE
		Attendre_Semaphore (&Semaphore_Nouveau_Equipement);
		Attendre_Semaphore (&Sem_GList_Liste_Thread_Scan);
#else
		Attendre_Semaphore (Semaphore_Nouveau_Equipement);
		Attendre_Semaphore (Sem_GList_Liste_Thread_Scan);
#endif
		Retour_Pointeur_Equipement (&Equipement,Num);
		ERROR_if (Equipement == NULL);
		Start_Thread (&Scanne_Equipement, (gpointer *) Equipement); 
		Num++;
	}
}


void Thread_Detect (void)
{
	struct Struct_Liste_Equipement *Equipement = NULL, *Equipement_GateWay = NULL;
#ifndef  __PCAP_ENABLE
	struct struct_trame Packet_struct,*Packet = NULL;
#else
	struct struct_trame *Packet = NULL;
	pcap_t    *captureHandle = NULL;
    struct pcap_pkthdr* packetHeader = NULL;
    const  u_char *     packetData = NULL;
	char errbuf[PCAP_ERRBUF_SIZE];
	int res = 0;
#endif
	unsigned char ID_Equipement[MAC_ADDR_LEN + IP_ADDR_LEN + 1];
	unsigned char Ip_Adresse_Bouclage[IP_ADDR_LEN + 1];
	unsigned char *ID_Equipement_Marquer = NULL, *ID_Equipement_Arp = NULL;
	unsigned short Port_Source = 0, Taille_Ip = 0, Port_Destination = 0, Icmp_Id = 0;
	unsigned char Type_ICMP = '\0', Code_ICMP = '\0';
	unsigned int Ip_MonNum = 0;
	char *Nom_Netbios = NULL, *Utilisateur = NULL;
	gint       n_read = 0, Type_Trame = 0,Etat_Equipement = 0;
	gboolean  Machine_Local,Arret_NBNS,INFO_NETBIOS, Equipement_Detecter, Port_Ouvert, Port_Fermer, GateWay_Detecter;

	
	Ip_Adresse_Bouclage[0] = 127;
	Ip_Adresse_Bouclage[1] = 0;
	Ip_Adresse_Bouclage[2] = 0;
	Ip_Adresse_Bouclage[3] = 1;
	Ip_Adresse_Bouclage[4] = 0;
	
	if (Nb_Machine_Dans_La_Liste==0)	
		Scan_Machine_Local_Fait = FALSE; 
	else
		Scan_Machine_Local_Fait = TRUE; 

#ifdef __LOG_ENABLE
	Create_Log(CATEG_THREAD_DETECT,NULL,NULL,TRUE);
#endif
	DEBUG_INFO("Thread Detect  pid(%d)", getpid ());


#ifndef  __PCAP_ENABLE
	int       Socket;
	struct sockaddr sock;
	Packet = &Packet_struct;
	OPEN_SOCKET(Socket,sock,Info_Demarrage,PF_PACKET, SOCK_RAW,htons (ETH_P_ALL));
#else
	OPEN_SOCKET_PCAP(captureHandle,Info_Demarrage,errbuf) ;
#endif	
	while (1) {
		INFO_NETBIOS = FALSE;
		Arret_NBNS = FALSE;
		Equipement_Detecter = FALSE;
		Port_Ouvert = FALSE;
		Port_Fermer = FALSE;
		Utilisateur = NULL;
		Nom_Netbios = NULL;
		GateWay_Detecter = FALSE;
#ifndef  __PCAP_ENABLE
		n_read = recvfrom (Socket, Packet, sizeof (struct struct_trame)-sizeof(int), 0, NULL, 0);
#else
		res = pcap_next_ex(captureHandle, &packetHeader, &packetData);
		if(res==0)
            continue; 
		n_read = packetHeader->len;
		Packet = (struct struct_trame *) packetData;
#endif
		Type_Trame = 0;
		memcpy (&ID_Equipement, Packet->Source_MAC, MAC_ADDR_LEN);
		switch (htons (Packet->Proto)) {
		    case TRAME_IP:
			    if ((Packet->Datagramme[0] >> 4) == IP_VERSION4) {
				    memcpy (&ID_Equipement[MAC_ADDR_LEN], &Packet->Datagramme[12], IP_ADDR_LEN);
					Equipement_Detecter = TRUE; 
				    switch (Packet->Datagramme[9]) {
					case TCP:
						Type_Trame = TCP;
						Taille_Ip = (Packet->Datagramme[0] & 0x0F) << 2;
						memcpy (&Port_Source, &Packet->Datagramme[Taille_Ip], 2);
						Port_Source = htons (Port_Source);
						memcpy (&Port_Destination, &Packet->Datagramme[Taille_Ip + 2], 2);
						Port_Destination = htons (Port_Destination);
					
					
						switch (Packet->Datagramme[Taille_Ip + 13]) {
						    case 0x14:
						    case 0x1C:
						    case 0x15:
							    Port_Fermer = TRUE;
							    break;
						    case 0x12:
							    Port_Ouvert = TRUE;
							    break;
						    case 0x02:
							   
							   
							    if ((memcmp (&Packet->Datagramme[12], &Packet->Datagramme[16], IP_ADDR_LEN) == 0) 
									&& (memcmp (Packet->Dest_MAC, Mon_Adresse_MAC, MAC_ADDR_LEN) == 0) 
									&&  (memcmp (Packet->Source_MAC, Mon_Adresse_MAC, MAC_ADDR_LEN) != 0)) {
									GateWay_Detecter = TRUE;
							    }
							    break;
						}
						break;
					case UDP:
						Type_Trame = UDP;
					
						if ((Packet->Dest_MAC[3] == 255) && (Packet->Dest_MAC[4] == 255) && (Packet->Dest_MAC[5] == 255) 
							&& (Packet->Datagramme[22] == 0) && (Packet->Datagramme[23] == NBNS)) {
							INFO_NETBIOS = TRUE;
							if ((Packet->Datagramme[30] == 0x30) && (Packet->Datagramme[31] == 0x10) && (Packet->Datagramme[71] == 0x41) && (Packet->Datagramme[72] == 0x41))
								Arret_NBNS = TRUE;
						
							if ((Packet->Datagramme[30] == 0x28) && (Packet->Datagramme[31] == 0x10) && (Packet->Datagramme[71] == 0x41) && (Packet->Datagramme[72] == 0x44))
								Utilisateur = get_nbns_name(&Packet->Datagramme[40]);
						}
						if ( (Packet->Dest_MAC[3] == 255) && (Packet->Dest_MAC[4] == 255) && (Packet->Dest_MAC[5] == 255)
							&& (Packet->Datagramme[9] == UDP) 
							&& (Packet->Datagramme[22] == 0) && (Packet->Datagramme[23] == BROWSE)) {
							INFO_NETBIOS = TRUE;
					    	if (Packet->Datagramme[196] == MASTER_ANNONCE) {
							   
						    }
						    if (Packet->Datagramme[196] == HOST_ANNONCE) {
								Nom_Netbios = (char *) &Packet->Datagramme[202];
							   
					    	}	
				    	}
						break;
					case ICMP:
						Type_Trame = ICMP;
						Taille_Ip = (Packet->Datagramme[0] & 0x0F) << 2;
						Type_ICMP = Packet->Datagramme[Taille_Ip];
						Code_ICMP = Packet->Datagramme[Taille_Ip + 1];
						memcpy (&Icmp_Id, &Packet->Datagramme[Taille_Ip + 4], 2);
						Icmp_Id = htons (Icmp_Id);
						break;
				    }
			    }
			    break;
		    case TRAME_ARP:
			    Type_Trame = TRAME_ARP;
			    memcpy (&ID_Equipement[MAC_ADDR_LEN], &Packet->Datagramme[14], IP_ADDR_LEN);
				Equipement_Detecter = TRUE;
			    break;
		}
	
		if (memcmp (&ID_Equipement[6], Ip_Adresse_Bouclage, IP_ADDR_LEN) == 0)
			continue;			
		
	
		if (!Equipement_Detecter)
			continue;
		
		Machine_Local = (memcmp (&ID_Equipement[6], Mon_Adresse_Ip, IP_ADDR_LEN) == 0);
	
		if ((!Sem_Get_Boolean(&Scan_Machine_Local_Fait)) && (!Machine_Local))
			continue;
		
	
		if ((Sem_Get_Boolean(&Scan_Machine_Local_Fait)) && (Machine_Local) ){
			if (GateWay_Detecter) {
			   	
			   
			   
		    
		    
		    	Equipement_GateWay = Recherche_MAC_Gateway (ID_Equipement);
		    	PROTECT_CODE (Semaphore_Int);
				if (Equipement_GateWay != NULL) {
			    	if ((Port_Source == Equipement_GateWay->Port_Source_Test_GateWay)
					    && (Port_Destination == Equipement_GateWay->Port_Destination_Test_GateWay)) {
				    	DEBUG_INFO("GateWay %d %d %d %d", Port_Source, Port_Destination, Equipement_GateWay->Port_Source_Test_GateWay, Equipement_GateWay->Port_Destination_Test_GateWay);
				    	Equipement_GateWay->GateWay = TRUE;
			    	}
		    	}
		    	else
				    DEBUG_WARNING("Gateway non retrouvé : [%d.%d.%d.%d]", ID_Equipement[6], ID_Equipement[7], ID_Equipement[8], ID_Equipement[9]);
				DEPROTECT_CODE (Semaphore_Int);
			}
			continue;			
		}

		if (!((ID_Equipement[0] == 0) && (ID_Equipement[1] == 0) && (ID_Equipement[2] == 0) && (ID_Equipement[3] == 0))) {
		
			if (Recherche_Si_Sous_Reseau_Autoriser (&ID_Equipement[MAC_ADDR_LEN])) {
			
				Etat_Equipement = Recherche_Equipement(ID_Equipement,&Equipement);
				switch (Etat_Equipement) {
					case EQUIPEMENT_IP_CHANGE :
						if (memcmp (ID_Equipement, Mon_Adresse_MAC, MAC_ADDR_LEN) != 0) {
						    if (htons (Packet->Proto) == TRAME_ARP) {
								DEBUG_INFO("Equipement à changer d'IP : [%d.%d.%d.%d] <-- [%s] %s", ID_Equipement[6], ID_Equipement[7], ID_Equipement[8], ID_Equipement[9],  Equipement->Adresse_Ip_Char, Equipement->Adresse_MAC_Char);								
							
								if (Sem_Get_Boolean(&Equipement->Scan_Port_En_Cours)){
									DEBUG_INFO("Equipement à changer d'IP : [%s] %s en cours de scan", Equipement->Adresse_Ip_Char, Equipement->Adresse_MAC_Char);								
									break;
								}
							
							   
							   
							    if (Info_Demarrage.Ip_Dynamic)
									Recherche_Equipement_Perte_Ip (ID_Equipement);
							   
							   
							    switch (Marque_Equipement_Avec_Meme_Adresse_MAC (ID_Equipement)) {
									case DEJA_EN_COURS_CHANGE_IP:
											DEBUG_INFO("Equipement déja en cours de changement : [%s] %s en cours de scan", Equipement->Adresse_Ip_Char, Equipement->Adresse_MAC_Char);
											break;
									case EQUIPEMENT_MARQUER_CHANGE_IP:
											DEBUG_INFO("Equipement marquer pour changement : [%s] %s en cours de scan", Equipement->Adresse_Ip_Char, Equipement->Adresse_MAC_Char);
										
											ID_Equipement_Marquer = NULL; 
											MALLOC(ID_Equipement_Marquer,sizeof (unsigned char) * (MAC_ADDR_LEN + IP_ADDR_LEN + 1));
											memcpy (ID_Equipement_Marquer, ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
											Start_Thread (&Traite_Equipement_Marquer_Ip_Change, (gpointer *) ID_Equipement_Marquer);
											break;
									case AUCUN_EQUIPEMENT_MARQUER_CHANGE_IP:
											DEBUG_INFO("Equipement PAS marquer pour changement : [%s] %s en cours de scan", Equipement->Adresse_Ip_Char, Equipement->Adresse_MAC_Char);
											break;
									default:
										DEBUG_ERROR( "Retour inconnu");
							    }
							}
						}
						break;					
						
  					case EQUIPEMENT_UNKNOWN:
					
					    if (memcmp (ID_Equipement, Mon_Adresse_MAC, MAC_ADDR_LEN) != 0) {
						    if (htons (Packet->Proto) == TRAME_ARP) {
								DEBUG_INFO("Equipement détecté pris en compte : [%d.%d.%d.%d]", ID_Equipement[6], ID_Equipement[7], ID_Equipement[8], ID_Equipement[9]);
								Equipement = Ajout_Equipement (ID_Equipement,TRUE); 
						    }
						    else {
								if (!Recherche_Present_GList_Scan_Arp_Detect(ID_Equipement)){
								
									DEBUG_INFO("Equipement détecté NON pris en compte : [%d.%d.%d.%d]", ID_Equipement[6], ID_Equipement[7], ID_Equipement[8], ID_Equipement[9]);
									ID_Equipement_Arp = NULL;
									MALLOC(ID_Equipement_Arp,sizeof (unsigned char) * (MAC_ADDR_LEN + IP_ADDR_LEN + 1));
									memcpy (ID_Equipement_Arp, ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
									PROTECT_GLIST_SCAN_ARP_DETECT_NOUVEAU_EQUIPEMENT();
									G_LIST_APPEND (GList_Scan_Arp_Detect_Nouveau_Equipement, ID_Equipement_Arp);
									DEPROTECT_GLIST_SCAN_ARP_DETECT_NOUVEAU_EQUIPEMENT();
								}
					    	}
						}
					    else {
						   
						    if (memcmp (&ID_Equipement[6], Mon_Adresse_Ip, IP_ADDR_LEN) == 0) {
							    Equipement = Ajout_Equipement (ID_Equipement,FALSE);
						    }
					    }
					    break;
						
						
				    case EQUIPEMENT_NO_IP:
					
					    Recherche_Equipement_Perte_Ip (ID_Equipement);
					    Ip_Equipement_Retrouver (ID_Equipement);
					    break;
					
					
					
					case EQUIPEMENT_KNOWN:
				    
					   
					    if (INFO_NETBIOS) { 
							if (Arret_NBNS)
						    	Equipement_Arreter (Equipement);
					    	if (Utilisateur)
								Change_Name_Netbios(Equipement,Utilisateur);
							if ((Nom_Netbios) && (strcmp((char *) Equipement->Smb.HostName,Nom_Netbios)!=0))
								Change_HostName_Netbios(Equipement,Nom_Netbios);
						}
					    else {
						    Reception_Donnee_Equipement_Connu (Equipement);
						    switch (Type_Trame) {
							case TCP:
								if (!(Traitement_ScanOS_TCP (Equipement, Packet, n_read, Taille_Ip, Port_Source, Port_Destination))) {
								
									memcpy (&Ip_MonNum, &Packet->Datagramme[Taille_Ip + 8], 4);
									if (Port_Fermer) {
										Traitement_Port_TCP (Equipement, Port_Source,Port_Destination, FERMER,Ip_MonNum,Packet->Datagramme[Taille_Ip + 13]);
									}
									if (Port_Ouvert) {
										Traitement_Port_TCP (Equipement, Port_Source,Port_Destination, OUVERT,Ip_MonNum,Packet->Datagramme[Taille_Ip + 13]);
									}
								}
								break;
							case ICMP:
								Traitement_Port_UDP_Fermer (Equipement, Packet, n_read, Taille_Ip, Type_ICMP, Code_ICMP);
								if (!(Traitement_ScanOS_ICMP (Equipement, Packet, n_read, Taille_Ip, Type_ICMP, Code_ICMP))) {
								
								}
								if ((Sem_Get_Boolean(&Equipement->Scan_Signature)) && (Type_ICMP == 5) && (Code_ICMP == 1) && (memcmp (&Packet->Datagramme[Taille_Ip + 4], Mon_Adresse_Ip, IP_ADDR_LEN) == 0)) {
									DEBUG_INFO("Routage %s", Equipement->Adresse_Ip_Char);
									Equipement->Routage_Activer = TRUE;
								}
								break;
						    }
					    }
						break;
					default:
						DEBUG_ERROR("Retour inconnu");
				}
			}
		}
	}
}
