/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Semaphore.h"
#include "Initialisation_Com.h"
#include "Debug.h"


#ifndef         __WINDOWS_ENABLE	
#include <errno.h>

#ifdef   __MUTEX_ENABLE
static int sem_init(sem_t *sem, int pshared, int value) {
    int ret;
    sem->value = value;

    ret = pthread_mutex_init(&sem->mutex,NULL);
    if(ret < 0) return -1;
    ret = pthread_cond_init(&sem->cond,NULL);
    if(ret < 0) return -1;
    return 0;
}

static int sem_post(sem_t *sem) {
    if(pthread_mutex_lock(&sem->mutex) < 0)
        return -1;
    sem->value++;
    if(pthread_cond_signal(&sem->cond) < 0) {
        pthread_mutex_unlock(&sem->mutex);
        return -1;
    }
    if(pthread_mutex_unlock(&sem->mutex) < 0)
        return -1;
    return 0;
}

static int sem_wait(sem_t *sem) {
    if(pthread_mutex_lock(&sem->mutex) < 0)
        return -1;
    while(sem->value == 0) {
        pthread_cond_wait(&sem->cond,&sem->mutex);
    }
    sem->value--;
    if(pthread_mutex_unlock(&sem->mutex) < 0)
        return -1;
    return 0;
}

static int sem_destroy(sem_t *sem) {
    int ret;
    ret = pthread_cond_destroy(&sem->cond);
    if(ret < 0) return -1;
    ret = pthread_mutex_destroy(&sem->mutex);
    if(ret < 0) return -1;
    return 0;
}

static void sem_getvalue(sem_t *sem,int *value) {
	(*value) = sem->value;
}
#endif


void Attendre_Semaphore (sem_t * sem)
{
	ERROR_if (sem == NULL);
	if (sem_wait (sem) == -1) {
		switch (errno) {
		    case EINTR:
			    DEBUG_ERROR("EINTR : (A signal interrupted)  - %s", strerror (errno));
		    case EINVAL:
			    DEBUG_ERROR("EINVAL : (Does not refer to a valid semaphore)  - %s", strerror (errno));
		    case EAGAIN:
			    DEBUG_ERROR("EAGAIN  - %s", strerror (errno));
		    default:
			    DEBUG_ERROR("%s", strerror (errno));
			    break;
		}
	}
}

void Liberation_Semaphore (sem_t * sem)
{
	ERROR_if (sem == NULL);
	if (sem_post (sem) == -1) {
		switch (errno) {
		    case EINVAL:
			    DEBUG_ERROR("EINVAL : (Does not refer to a valid semaphore)  - %s", strerror (errno));
		    default:
			    DEBUG_ERROR("%s", strerror (errno));
			    break;
		}
	}
}

int Valeur_Semaphore (sem_t * sem)
{
	int       valeur;

	ERROR_if (sem == NULL);
	sem_getvalue (sem, &valeur);
	return valeur;
}


void Initialisation_Semaphore (void)
{
	sem_init (&Semaphore_Debug, 0, 1);
	sem_init (&Semaphore_Thread, 0, 1);
	sem_init (&Semaphore_Etat_Thread, 0, 1);
	sem_init (&Semaphore_Log_File, 0, 1);
	sem_init (&Semaphore_Mem, 0, 1); 
	sem_init (&Semaphore_Fonction_Externe, 0, 1);
	sem_init (&Semaphore_Control_Usage_Socket, 0, 1);
}

#else 
 
void Attendre_Semaphore (HANDLE sem) 
{ 
	if (WaitForSingleObject (sem, INFINITE)== WAIT_FAILED) 
		DEBUG_ERROR("WaitForSingleObject  error: %ld\n", GetLastError ()); 
	if ((sem==Semaphore_Nouveau_Equipement) || (sem==Semaphore_Thread_Scan)){ 
	
		if (WaitForSingleObject (Semaphore_Compteur, INFINITE)== WAIT_FAILED) 
			DEBUG_ERROR("WaitForSingleObject  error: %ld\n", GetLastError ()); 
		if (sem==Semaphore_Nouveau_Equipement) 
			Compteur_Nouveau_Equipement++; 
		if (sem==Semaphore_Thread_Scan) 
			Compteur_Thread_Scan--; 
		if (!ReleaseSemaphore (Semaphore_Compteur, 1, NULL)) 
			DEBUG_ERROR("ReleaseSemaphore  error: %ld\n", GetLastError ()); 
	 
	} 
} 
 

void Liberation_Semaphore (HANDLE sem) 
{ 
	if (!ReleaseSemaphore (sem, 1, NULL)) 
		DEBUG_ERROR("ReleaseSemaphore  error: %ld\n", GetLastError ()); 
	if ((sem==Semaphore_Nouveau_Equipement) || (sem==Semaphore_Thread_Scan)){ 
	
		if (WaitForSingleObject (Semaphore_Compteur, INFINITE)== WAIT_FAILED) 
			DEBUG_ERROR("WaitForSingleObject  error: %ld\n", GetLastError ()); 
		if (sem==Semaphore_Nouveau_Equipement) 
			Compteur_Nouveau_Equipement--; 
		if (sem==Semaphore_Thread_Scan) 
			Compteur_Thread_Scan++; 
		if (!ReleaseSemaphore (Semaphore_Compteur, 1, NULL)) 
			DEBUG_ERROR("ReleaseSemaphore  error: %ld\n", GetLastError ()); 
	} 
} 
 
 

int Valeur_Semaphore (HANDLE *sem) 
{ 
	int       valeur; 
 

	if (*sem==Semaphore_Nouveau_Equipement) 
		return Compteur_Nouveau_Equipement; 
	if (*sem==Semaphore_Thread_Scan) 
		return Compteur_Thread_Scan;	 
	return 1; 
} 
 
 


void Initialisation_Semaphore (void) 
{ 
	
	if ((Semaphore_Debug = CreateSemaphore (NULL, 1, 1, NULL)) == NULL) 
		DEBUG_ERROR("CreateSemaphore(Semaphore_Debug) error: %ld\n", GetLastError ()); 
	if ((Semaphore_Thread = CreateSemaphore (NULL, 1, 1, NULL)) == NULL) 
		DEBUG_ERROR("CreateSemaphore(Semaphore_Thread) error: %ld\n", GetLastError ()); 
	if ((Semaphore_Etat_Thread = CreateSemaphore (NULL, 1, 1, NULL)) == NULL) 
		DEBUG_ERROR("CreateSemaphore(Semaphore_Etat_Thread) error: %ld\n", GetLastError ()); 
	if ((Semaphore_Log_File = CreateSemaphore (NULL, 1, 1, NULL)) == NULL) 
		DEBUG_ERROR("CreateSemaphore(Semaphore_Log_File) error: %ld\n", GetLastError ()); 
	if ((Semaphore_Control_Usage_Socket = CreateSemaphore (NULL, 1, 1, NULL)) == NULL) 
		DEBUG_ERROR("CreateSemaphore(Semaphore_Control_Usage_Socket) error: %ld\n", GetLastError ()); 
	if ((Semaphore_Fonction_Externe = CreateSemaphore (NULL, 1, 1, NULL)) == NULL) 
		DEBUG_ERROR("CreateSemaphore(Semaphore_Fonction_Externe) error: %ld\n", GetLastError ()); 
	if ((Semaphore_Mem = CreateSemaphore (NULL, 1, 1, NULL)) == NULL) 
		DEBUG_ERROR("CreateSemaphore(Semaphore_Mem) error: %ld\n", GetLastError ()); 
} 
#endif 
