/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Enregistrement.h"
#include "Equipement.h"
#include "Equipement_Port.h"
#include "Equipement_Historique.h"
#include "Settings_Signature.h"
#include "Add_Network_Subnet.h"
#include "Reseau.h"
#include "Debug.h"
#include "Objet.h"
#include "Snmp.h"
#include "config.h"
#include "Mem.h"
#include "Xml.h"

extern struct Struct_Modele_Objet Liste_Modele_Objet[];
extern struct Struct_Liste_Port_UDP Liste_Port_UDP[];

static void Write_xml_Snmp_Equipement (xmlNodePtr xmlnode, struct Struct_Equipement *Equipement)
{
	xmlNodePtr newxml = NULL;

	ERROR_if (Equipement == NULL);
	XML_CREATE_NODE(newxml,xmlnode,"Snmp");

	XML_WRITE_STRING("Info_Device", Equipement->Snmp_Info_Device); 
	XML_WRITE_STRING("System_Name", Equipement->Snmp.System_Name);
	XML_WRITE_STRING("Description", Equipement->Snmp.Description);
	XML_WRITE_STRING("Contact", Equipement->Snmp.Contact);
	XML_WRITE_STRING("Location", Equipement->Snmp.Location);
	XML_WRITE_STRING("Uptime", Equipement->Snmp.Uptime);
	XML_WRITE_STRING("Ip_Forwarding", Equipement->Snmp.Ip_Forwarding);
	XML_WRITE_INT("Ram",Equipement->Snmp.Ram);
	XML_WRITE_INT("Default_TTL",Equipement->Snmp.Default_TTL);
	XML_WRITE_INT("Reassembly_Timeout",Equipement->Snmp.Reassembly_Timeout);
	XML_WRITE_INT("Need_To_Be_Reassembled",Equipement->Snmp.Need_To_Be_Reassembled);
	XML_WRITE_INT("Reassembled",Equipement->Snmp.Reassembled);
	XML_WRITE_INT("Reassembly_Failures",Equipement->Snmp.Reassembly_Failures);
	XML_WRITE_INT("Successfully_Fragmented",Equipement->Snmp.Successfully_Fragmented);
	XML_WRITE_INT("Fragmentation_Failures",Equipement->Snmp.Fragmentation_Failures);
	XML_WRITE_INT("Created_Fragments",Equipement->Snmp.Created_Fragments);
	XML_WRITE_INT("Discarded_Routing_Entries",Equipement->Snmp.Discarded_Routing_Entries);
	XML_WRITE_INT("All",Equipement->Snmp.All);
	XML_WRITE_INT("Forwarded",Equipement->Snmp.Forwarded);
	XML_WRITE_INT("Delivered",Equipement->Snmp.Delivered);
	XML_WRITE_INT("Header_Errors",Equipement->Snmp.Header_Errors);
	XML_WRITE_INT("Address_Errors",Equipement->Snmp.Address_Errors);
	XML_WRITE_INT("Unknown_Protocol",Equipement->Snmp.Unknown_Protocol);
	XML_WRITE_INT("Input_Buffer_Errors",Equipement->Snmp.Input_Buffer_Errors);
	XML_WRITE_INT("Requested",Equipement->Snmp.Requested);
	XML_WRITE_INT("Output_Buffer_Errors",Equipement->Snmp.Output_Buffer_Errors);
	XML_WRITE_INT("Not_Routed", Equipement->Snmp.Not_Routed);

}


static void Write_xml_Port_Equipement_Open (xmlNodePtr xmlnode, struct Struct_Port *Port)
{
	xmlNodePtr newxml = NULL;

	ERROR_if (Port == NULL);
	XML_CREATE_NODE(newxml,xmlnode,"Port");

	XML_WRITE_INT("Num",Port->Num_Port);
	XML_WRITE_BOOLEAN("Notification",Port->Notification_En_Cours);


}

static void Write_xml_Port_Equipement_FFW (xmlNodePtr xmlnode, struct Struct_Port_Fermer_Firewall *Port)
{
	xmlNodePtr newxml = NULL;

	ERROR_if (Port == NULL);
	XML_CREATE_NODE(newxml,xmlnode,"Port");

	XML_WRITE_INT("Num",Port->Num_Port);

}

static void Write_xml_Signature_Equipement (xmlNodePtr xmlnode, struct Struct_Equipement *Equipement)
{
	xmlNodePtr newxml = NULL;
	struct Struct_Signature *Signature = NULL;
	
	ERROR_if (Equipement == NULL);
	XML_CREATE_NODE(newxml,xmlnode,"FingerPrinting");

	XML_WRITE_STRING("MD5", Equipement->Md5_Signature);
	XML_WRITE_STRING("MD5_W", Equipement->Md5_Signature_W);
	XML_WRITE_STRING("Header_FTP", Equipement->Entete_FTP);
	XML_WRITE_STRING("Header_HTTP", Equipement->Entete_HTTP);
	XML_WRITE_STRING("Header_IMAP", Equipement->Entete_IMAP);
	XML_WRITE_STRING("Header_SMTP",Equipement->Entete_SMTP);
	XML_WRITE_STRING("Header_SSH",Equipement->Entete_SSH);
	XML_WRITE_STRING("Header_TELNET", Equipement->Entete_TELNET);
	XML_WRITE_STRING("Header_SIP", Equipement->Entete_SIP);
	XML_WRITE_STRING("Header_POP3", Equipement->Entete_POP3);
	XML_WRITE_STRING("Header_VNC0", Equipement->Entete_VNC0);
	XML_WRITE_STRING("Header_VNC1", Equipement->Entete_VNC1);
	XML_WRITE_STRING("Header_RTSP", Equipement->Entete_RTSP);
	XML_WRITE_STRING("Header_IPP", Equipement->Entete_IPP);
	XML_WRITE_STRING("Header_VNC_HTTP", Equipement->Entete_VNC_HTTP);
	XML_WRITE_STRING("Debug", Equipement->Debug_Signature);

	Signature = Equipement->Signature;
	if (Signature!=NULL){
		XML_WRITE_STRING("System_name", Signature->System_Name);
		XML_WRITE_STRING("Type", Liste_Modele_Objet[Equipement->Signature->Equipment_Type].Texte_Defaut);	
	}

}


static void Write_xml_Historique_Equipement (xmlNodePtr xmlnode, struct Struct_Historique *Historique)
{
	xmlNodePtr newxml = NULL;

	ERROR_if (Historique == NULL);
	XML_CREATE_NODE(newxml,xmlnode,"History");


	XML_WRITE_INT("Level",Historique->Niveau_Historique);
	XML_WRITE_STRING("Text", Historique->Historique);

}


static void Write_xml_Samba_Equipement (xmlNodePtr xmlnode, struct Struct_Equipement *Equipement, gint i)
{
	xmlNodePtr newxml = NULL;

	ERROR_if (Equipement == NULL);
	XML_CREATE_NODE(newxml,xmlnode,"Registration");

	XML_WRITE_INT("Number",Equipement->Smb.Smb_Registration[i].Numero);
	XML_WRITE_INT("Type",Equipement->Smb.Smb_Registration[i].Type);
	XML_WRITE_STRING("Name",(char *)Equipement->Smb.Smb_Registration[i].Name);

}


static void Write_xml_Subnet (xmlNodePtr xmlnode, struct Struct_Subnet *Subnet)
{
	xmlNodePtr newxml = NULL;

	ERROR_if (Subnet == NULL);
	XML_CREATE_NODE(newxml,xmlnode,"Subnet");

	XML_WRITE_INT("Masque",Subnet->Masque);
	XML_WRITE_INT("A",Subnet->A);
	XML_WRITE_INT("B",Subnet->B);
	XML_WRITE_INT("C",Subnet->C);

}


static void Write_xml_Equipment (xmlNodePtr xmlnode, struct Struct_Equipement *Equipement)
{
	xmlNodePtr Xml_Equipment = NULL, newxml_Port_Info = NULL, newxml_Port = NULL, newxml_Port_Udp = NULL, newxml_Rep = NULL,newxml = NULL;
	struct Struct_Port *Port = NULL;
	struct Struct_Port_Fermer_Firewall *Port_FFW = NULL;
	struct Struct_Historique *Historique = NULL;
	struct Struct_Modele_Objet *Objet = NULL;
	GtkTreePath *path = NULL;
	GtkTreeModel *model = NULL;
	guint     Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	XML_CREATE_NODE(Xml_Equipment,xmlnode,"Equipment");


	newxml = Xml_Equipment; 
	XML_WRITE_STRING("Address_IP", (char *)Equipement->Adresse_Ip_Char);
	XML_WRITE_STRING("Address_MAC", (char *)Equipement->Adresse_MAC_Char);
	XML_WRITE_STRING("Localisation", Equipement->Localisation);
	XML_WRITE_STRING("Admin_Email", Equipement->Contact_Admin);
	XML_WRITE_STRING("Dns", Equipement->Nom_DNS_Complet);
	XML_WRITE_STRING("Alias", Equipement->Alias);
	XML_WRITE_STRING("Serial_Number", Equipement->Num_Serie);
	XML_WRITE_STRING("Serial_Number_Display", Equipement->Num_Serie_Ecran);
	XML_WRITE_STRING("Serial_Number_Printer", Equipement->Num_Serie_Imprimante);
	XML_WRITE_STRING("Serial_Number_OS", Equipement->Num_Serie_OS);
	XML_WRITE_STRING("Serial_Number_Office_Suite", Equipement->Num_Serie_Office_Suite);
	XML_WRITE_STRING("Processor", Equipement->Processeur);
	XML_WRITE_STRING("Memory", Equipement->Memoire);
	XML_WRITE_STRING("Disk", Equipement->Disque);
	XML_WRITE_STRING("Contact", Equipement->Contact);
	XML_WRITE_STRING("Tel_Contact", Equipement->Tel_Contact);
	XML_WRITE_STRING("Notes", Equipement->Notes);
	XML_WRITE_BOOLEAN("Agent_AutoScan",Equipement->Agent_AutoScan);
	XML_WRITE_BOOLEAN("Notification",Equipement->Notification_En_Cours);
	XML_WRITE_BOOLEAN("Gateway",Equipement->GateWay);
	XML_WRITE_BOOLEAN("Routage",Equipement->Routage);
	XML_WRITE_BOOLEAN("Snmp_Open",Equipement->Snmp_Ouvert);
	XML_WRITE_BOOLEAN("Mss",Equipement->mss);
	XML_WRITE_BOOLEAN("User_SSH_Default",Equipement->User_SSH_Default);
	XML_WRITE_STRING("User_SSH", Equipement->User_SSH);
	XML_WRITE_BOOLEAN("Password_VNC_Default",Equipement->Password_VNC_Default);
	XML_WRITE_STRING("Password_Vnc", Equipement->password_vnc);	
	XML_WRITE_BOOLEAN("View_Only_VNC",Equipement->View_Only_VNC);
	

	if (!Reseau_Selectionner->Mode_Liste_Detaille) {
		Objet = Equipement->Objet_Liste_Par_Type; 
		Objet = (struct Struct_Modele_Objet *) Objet->Objet_Pere;
		if (Objet != NULL) {
			model = gtk_tree_view_get_model (GTK_TREE_VIEW (Reseau_Selectionner->Liste_Equipement_Type));
			path = gtk_tree_model_get_path (model, &Objet->ID_Arbre_Ligne);
			XML_WRITE_BOOLEAN("Expand",gtk_tree_view_row_expanded (GTK_TREE_VIEW (Reseau_Selectionner->Liste_Equipement_Type), path));
		}
		else
			XML_WRITE_BOOLEAN("Expand", FALSE);
	}
	else 
		XML_WRITE_BOOLEAN("Expand", Equipement->Deplier) ;


	
	


	Write_xml_Signature_Equipement (Xml_Equipment, Equipement);

	

	XML_CREATE_NODE(newxml_Port_Info,Xml_Equipment,"Ports");

	newxml = newxml_Port_Info; 
	XML_WRITE_INT("Open",Equipement->Nb_Port_Ouvert);
	XML_WRITE_INT("Total",Equipement->Nb_Port_Total_Scanner);
	XML_WRITE_INT("firewalled",Equipement->Nb_Port_Firewaller);
	XML_WRITE_INT("Close",Equipement->Nb_Port_Fermer);

	

	XML_CREATE_NODE(newxml_Port,newxml_Port_Info,"Open");

	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port = g_list_nth_data (Equipement->GList_Port, i);
		Write_xml_Port_Equipement_Open (newxml_Port, Port);
	}

		

	XML_CREATE_NODE(newxml_Port,newxml_Port_Info,"Close");

	Nb_Objet_Liste = g_list_length (Equipement->GList_Port_Fermer);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_FFW = g_list_nth_data (Equipement->GList_Port_Fermer, i);
		Write_xml_Port_Equipement_FFW(newxml_Port, Port_FFW);
	}

	

	XML_CREATE_NODE(newxml_Port,newxml_Port_Info,"Firewall");

	Nb_Objet_Liste = g_list_length (Equipement->GList_Port_Firewall);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_FFW = g_list_nth_data (Equipement->GList_Port_Firewall, i);
		Write_xml_Port_Equipement_FFW(newxml_Port, Port_FFW);
	}

		

	XML_CREATE_NODE(newxml_Port,newxml_Port_Info,"UDP");

	for (i = 0; i < NB_PORT_UDP; i++){
		XML_CREATE_NODE(newxml_Port_Udp,newxml_Port,"Port");
	
		newxml = newxml_Port_Udp; 
		XML_WRITE_INT("Num",Liste_Port_UDP[i].Num_Port_UDP);
		XML_WRITE_INT("State",Equipement->Liste_Etat_Udp[i]);
	
	}

		

	XML_CREATE_NODE(newxml_Rep,Xml_Equipment,"History");

	Nb_Objet_Liste = g_list_length (Equipement->GList_Historique);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Historique = g_list_nth_data (Equipement->GList_Historique, i);
		Write_xml_Historique_Equipement (newxml_Rep, Historique);
	}

	

	XML_CREATE_NODE(newxml_Rep,Xml_Equipment,"Smb");

	newxml = newxml_Rep; 
	XML_WRITE_STRING("Host_Name", (char *)Equipement->Smb.HostName);
	XML_WRITE_STRING("Domain_Name", (char *)Equipement->Smb.DomainName);
	XML_WRITE_STRING("User", (char *)Equipement->Smb.Utilisateur);
	XML_WRITE_STRING("User_Smb", Equipement->User_Smb);
	XML_WRITE_STRING("Password_Smb", Equipement->Password_Smb);
	i = 0; 
	while ((i < NUM_MAX_REGISTRATION) && (Equipement->Smb.Smb_Registration[i].Type != 0)) {
		Write_xml_Samba_Equipement (newxml_Rep, Equipement, i);
		i++;
	}


	Write_xml_Snmp_Equipement (Xml_Equipment, Equipement);
}


static gboolean write_xml (char *fname)
{
	xmlDocPtr doc = NULL;
	struct Struct_Equipement *Equipement = NULL;
	struct Struct_Modele_Objet *Objet = NULL;
	struct Struct_Subnet *Subnet;
	guint     Nb_Objet_Liste, i;
	xmlNodePtr Subnet_Rep = NULL;
	xmlNodePtr nStylesheet = NULL;       
	xmlNodePtr newxml = NULL;
	xmlNodePtr root_element = NULL;

	XML_CREATE(fname, doc, root_element, "autoscan-network", "Copyright (c) Lagarde thierry. \nThis Database is available for free use by open source software under the terms of the GNU General Public License.\nContact <tlagarde@users.sourceforge.net>  for more information", "type=\"text/xsl\" href=\"" DESTDIR "/usr/share/apps/autoscan-network/Stylesheet.xsl\"");


	newxml = root_element;   
	XML_WRITE_STRING("Network", Reseau_Selectionner->Nom_Reseau);
	XML_WRITE_STRING("Interface", Reseau_Selectionner->Interface);
	XML_WRITE_STRING("Ip", Reseau_Selectionner->Ip);
	XML_WRITE_STRING("Ip_Agent", Reseau_Selectionner->Ip_Agent);
	XML_WRITE_INT("Port_Agent",Reseau_Selectionner->Port_Agent);
	XML_WRITE_STRING("Communaute_Snmp", Reseau_Selectionner->Communaute_Snmp);
	XML_WRITE_STRING("Icon", Reseau_Selectionner->Icone_Reseau);
	XML_WRITE_BOOLEAN("Intrusion_Alert",Reseau_Selectionner->Alert_Intrusion);
	XML_WRITE_BOOLEAN("Mode_IP",Reseau_Selectionner->Mode_Liste_Detaille);
	XML_WRITE_BOOLEAN("Ip_Dynamic",Reseau_Selectionner->Ip_Dynamic);
	
	XML_WRITE_STRING("Email_Alert_Intrusion", Reseau_Selectionner->Email_Alert_Intrusion);
	XML_WRITE_STRING("Global_Ssh_User", Reseau_Selectionner->Global_Ssh_User);
	XML_WRITE_STRING("Global_Vnc_Password", Reseau_Selectionner->Global_Vnc_Password);	

	


	XML_CREATE_NODE(Subnet_Rep,root_element,"Subnet");
	Nb_Objet_Liste = g_list_length (Reseau_Selectionner->Glist_Subnet_Equipement);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Subnet = g_list_nth_data (Reseau_Selectionner->Glist_Subnet_Equipement, i);
		Write_xml_Subnet (Subnet_Rep, Subnet);
	}



	Nb_Objet_Liste = g_list_length (Reseau_Selectionner->GList_Liste_Equipement);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Objet = g_list_nth_data (Reseau_Selectionner->GList_Liste_Equipement, i);
		if (Objet->Nb_Enfant == 0) {
			Equipement = (struct Struct_Equipement *) Objet->Pointeur_Attacher;
			if (!Equipement->Equipement_Erase)
				Write_xml_Equipment (root_element, Equipement);
		}
	}

	XML_SAVE(fname, doc);
	XML_CLOSE (doc);
	return TRUE;
}


void Enregistrement (gchar * Chemin_Fichier)
{
	ERROR_if (Chemin_Fichier == NULL);
	DEBUG_INFO("Save : [%s]", Chemin_Fichier);
	write_xml (Chemin_Fichier);
}
