/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Xml.h"
#include "Debug.h"
#include "Files.h"

xmlNode  * OpenXml(gchar *filexml, gchar *name_root_element, xmlDocPtr *doc){
	xmlNode  *root_element = NULL;
	
	ERROR_if (filexml == NULL);
	if (!File_Exist(filexml,FALSE))
		return NULL;

	*doc = xmlParseFile (filexml);
	if (!*doc) {
		DEBUG_WARNING( "Error file not found  : [%s]", filexml);
		return NULL;
	}
	printf("Config : [%s]\n",filexml);
	root_element = xmlDocGetRootElement (*doc);

	if (!root_element || !root_element->name || g_strcasecmp ((const gchar *)root_element->name, name_root_element) != 0) {
		if (!root_element)
			DEBUG_WARNING("l'element n'est pas %s",name_root_element);
		DEBUG_WARNING("Unknown Format  : [%s]", filexml);
		xmlFreeDoc (*doc);
		return NULL;
	}
	return root_element;
}

gboolean CreateXml(char *filexml, xmlDocPtr *doc, xmlNodePtr *root_element, char * NameNodeRoot, char *copyright, char *stylesheet){
	xmlNodePtr nStylesheet = NULL;
		
	ERROR_if (filexml == NULL);

	*doc = xmlNewDoc (BAD_CAST "1.0");
	if (*doc == NULL)
		return FALSE;
	*root_element = xmlNewNode (NULL, (const xmlChar *) NameNodeRoot);
	if (*root_element == NULL)
		return FALSE;
	xmlDocSetRootElement (*doc, *root_element);

	if (copyright != NULL)
		xmlAddChild (*root_element, xmlNewDocComment (*doc, (xmlChar *)copyright));

	if (stylesheet != NULL){
  		nStylesheet = xmlNewPI (BAD_CAST "xml-stylesheet", (const xmlChar *) stylesheet);
  		xmlAddPrevSibling (*root_element, nStylesheet);
	}
	return TRUE;
}
	

gboolean SaveXml(char *filexml, xmlDocPtr *doc){
	ERROR_if (filexml == NULL);
	ERROR_if (doc == NULL);
	if (xmlSaveFile (filexml, *doc) == -1) {
		xmlFreeDoc (*doc);
		DEBUG_WARNING("impossible d'enregistrer le fichier xml [%s]",filexml);
		return FALSE;
	}
	return TRUE;
}

void CloseXml(xmlDocPtr *doc){
	ERROR_if (doc == NULL);
	xmlFreeDoc (*doc);
}


gboolean Check_Current_Node(xmlNodePtr xmlnode,char *name){
	
	ERROR_if (xmlnode == NULL);
	ERROR_if (name == NULL);
	if (!xmlnode->name || g_strcasecmp ((char *)xmlnode->name, name) != 0){
		DEBUG_WARNING("Aucun Noeud : [%s]", name);
		return FALSE;
	}
	return TRUE;
}

xmlNodePtr Create_Node(xmlNodePtr xmlnode,char *name){
	ERROR_if (xmlnode == NULL);
	ERROR_if (name == NULL);
	return xmlNewChild (xmlnode, NULL, (xmlChar *)name, NULL);
}

gchar *LitXml_string_default(gchar *key,gchar *default_reponse){
	gchar *text = NULL;
	ERROR_if (key == NULL);
	ERROR_if (Node_Courant == NULL);
	text = (gchar *) xmlGetProp (Node_Courant, (const xmlChar *)key);

	if ((text != NULL) && (strlen (text) == 0)) {
		MYFREE (text);  
		return default_reponse;
	}
	else
		return text;
	return default_reponse;	
}
gchar *LitXml_string(gchar *key){
	ERROR_if (key == NULL);
	return LitXml_string_default(key,NULL);	
}
void LitXml_string_copie(gchar *key,gchar *copie){
	gchar *text = NULL;
	ERROR_if (key == NULL);
	text = LitXml_string(key);
	if (text != NULL){
		strcpy (copie, text);
		MYFREE (text);
	}
	else
		strcpy(copie,"");
}

time_t64 LitXml_time(gchar *key){
	#define FORMAT_DATE "%a %b %e %H:%M:%S %Y"
	gchar    *Date = NULL;
	struct tm tm;
	ERROR_if (key == NULL);
#ifndef         __WINDOWS_ENABLE   
	ERROR_if (Node_Courant == NULL);
	Date = LitXml_string("Last_State_Change");
	if (Date != NULL){
		if ((strptime (Date, FORMAT_DATE, &tm)) == NULL)
			DEBUG_WARNING("Restoration Equipement date : [%s]", Date);
		MYFREE (Date);
		return mktime (&tm);
	}
	else
		return 0;
#endif     
	return 0;
}
gint LitXml_int_default(gchar *key,gint default_gint){
	gchar *text = NULL;
	gint entier;
	ERROR_if (key == NULL);
	ERROR_if (Node_Courant == NULL);
	text = (gchar *) xmlGetProp (Node_Courant, (const xmlChar *) key);
	if ((text == NULL) || (sscanf (text, "%d", &entier) != 1)) {
		DEBUG_WARNING("Recuperation Xml de [%s]=[%s]",key,text);
		entier = default_gint;
	}
	MYFREE (text);
	return entier;	
}
gint LitXml_int(gchar *key){
	return LitXml_int_default(key,0);
}
gboolean LitXml_boolean_default(gchar *key,gboolean default_boolean){
	gchar    *cetat = NULL;
	gboolean etat;
	ERROR_if (key == NULL);
	ERROR_if (Node_Courant == NULL);
	etat = FALSE;
	cetat = (gchar *) xmlGetProp (Node_Courant, (const xmlChar *) key);
	if (cetat != NULL) {
		etat = (cetat[0] == '1');
		MYFREE (cetat);
	}
	else
		etat = default_boolean;
	return etat;
}
gboolean LitXml_boolean(gchar *key){
	return LitXml_boolean_default(key,FALSE);
}
void StockXml_string(gchar *key,gchar *chaine){
	int taille,i;
	ERROR_if (key == NULL);
	ERROR_if (Node_Courant == NULL);	
	if ((chaine != NULL) && (strlen(chaine)!=0)){ 
	
		taille = strlen(chaine);
		i = 0;
		while ((i<taille) && (chaine[i]!=0)){
			if (((chaine[i]<32) || (chaine[i]>126)) && (chaine[i]!='\r') && (chaine[i]!='\n'))  
				chaine[i] = '_';
			i++;
		}
	
		xmlSetProp (Node_Courant, (const xmlChar *) key, (const xmlChar *) chaine);	
	}
}
void StockXml_time(gchar *key,time_t64 *time){
	ERROR_if (key == NULL);
#ifndef         __WINDOWS_ENABLE   
	gchar     Date[50]; 
	
	ctime_r ((const time_t *) time, Date);
	Date[strlen (Date) - 1] = '\0';
	StockXml_string(key, Date);
#endif   
}
void StockXml_int(gchar *key,gint etat){
	gchar *tmp=NULL;
	ERROR_if (key == NULL);
	ERROR_if (Node_Courant == NULL);	

	tmp = g_strdup_printf ("%d", etat);
	StockXml_string(key, tmp);
	MYFREE (tmp);
}

void StockXml_boolean(gchar *key,gboolean etat){
	ERROR_if (key == NULL);
	ERROR_if (Node_Courant == NULL);

	if (etat)
		xmlSetProp (Node_Courant, (const xmlChar *) key, (const xmlChar *) "1");
	else
		xmlSetProp (Node_Courant, (const xmlChar *)  key, (const xmlChar *) "0");
}
